
#ifndef _carreherbe_
#define _carreherbe_


//include io
#include <stdio.h>
#include <stdlib.h>

//include GL
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

//include base prairie
#include "GeometrieBase.hpp"

//include stl
#include <vector>
#include <algorithm>

//autres
#include "GrassDefines.h"
#include "GrassRandom.h"
#include "SharedData.h"
#include "Herbe.hpp"
#include "Noeud.hpp"
#include "Vitre.hpp"


class GrassEngine;

/** \class Carre_Herbe
\brief se caracterise par 4 noeuds.

cette classe permet d'afficher les herbes calculees grace au informations
des quatres noeuds (le plus souvent les informations sont calculees par interpolation lineaire)
Aussi, cette classe calcul son type d'affichage (3D, Vitres...)

*/

class Carre_Herbe
{
protected:
	///indice herbe aux coins du carre
	///   01----11
	///   |     |
	///  00----10
	Noeud *_n00,*_n10,*_n01,*_n11;
	NodeCrushing* _nc[9];
	///germe des valeurs aleatoires 
	short m_seedRandom;

public:
	short m_typeOfDrawing;
	GReal m_distanceToCamera;
	GReal m_density;
	bool m_isActive;

public:
	///constructeur
	Carre_Herbe();

	///destructeur
	~Carre_Herbe() {}
	
	void setNoeuds(Noeud *noeud0,
		       Noeud *noeud1,
	               Noeud *noeud2,
		       Noeud *noeud3);
	void setNodesCrushing(NodeCrushing *node00,
			      NodeCrushing *node10,
			      NodeCrushing *node20,
			      NodeCrushing *node01,
			      NodeCrushing *node11,
			      NodeCrushing *node21,
			      NodeCrushing *node02,
			      NodeCrushing *node12,
			      NodeCrushing *node22);
	
	
	//mise a jour type affichage et densite
	//retourne le type d'affichage
	inline void update();	
	
	//affichage de 2D sur le carre
	void draw2D() const;
	
	int drawTransition3D_2D5(const short& polygonsPerGrass,
				 const short& polygonsPerVitre) const;
	
	//affichage de vitres sur le carre
	int draw2D5(const short& polygonsPerVitre) const;
	
	//affichage d'herbes 3D sur le carre
	int draw3D(const short& polygonsPerGrass) const;
	
	///affichage du carre, couleur aux coins suivant influence
	/// de la trajectoire au noeud
	void AfficheInformationDensite() const;
};


struct SPatchesCoords
{
float m_x;
float m_y;
};
struct CompPatchesCoord_xIncrease
{
bool operator() (SPatchesCoords a, SPatchesCoords b)
	{
	return (a.m_x<b.m_x);
	}
};
struct CompPatchesCoord_xDecrease
{
bool operator() (SPatchesCoords a, SPatchesCoords b)
	{
	return (a.m_x>b.m_x);
	}
};
struct CompPatchesCoord_yIncrease
{
bool operator() (SPatchesCoords a, SPatchesCoords b)
	{
	return (a.m_y<b.m_y);
	}
};
struct CompPatchesCoord_yDecrease
{
bool operator() (SPatchesCoords a, SPatchesCoords b)
	{
	return (a.m_y>b.m_y);
	}
};



		



void Carre_Herbe::update()
{
if(!m_isActive) return;
if (!_n00->m_isVisible 
    && !_n10->m_isVisible 
    && !_n01->m_isVisible 
    && !_n11->m_isVisible) {m_typeOfDrawing=DRAWGRASS_NONE;return;}
//calcul type affichage
m_typeOfDrawing=_n00->m_typeOfDrawing;
if (_n10->m_typeOfDrawing < m_typeOfDrawing) m_typeOfDrawing=_n10->m_typeOfDrawing;
if (_n01->m_typeOfDrawing < m_typeOfDrawing) m_typeOfDrawing=_n01->m_typeOfDrawing;
if (_n11->m_typeOfDrawing < m_typeOfDrawing) m_typeOfDrawing=_n11->m_typeOfDrawing;

//distance a la camera	    
m_distanceToCamera = _n00->m_distanceToCamera;
if (_n11->m_distanceToCamera>m_distanceToCamera) m_distanceToCamera = _n11->m_distanceToCamera;

//test si en transition
if (m_typeOfDrawing==DRAWGRASS_3D)
	{
	if (m_distanceToCamera>SharedData::m_distanceTransition3D_2D5)
		m_typeOfDrawing=DRAWGRASS_TRANSITION_3D_2D5;
	}
}	






#endif /* _carreherbe_ */
