
#ifndef _geometrie_base_
#define _geometrie_base_
#include <math.h>
#include "GrassDefines.h"
#include <stdio.h>
//------------------------------------------------------------------------
//-----POINT
//------------------------------------------------------------------------


/** \class IPoint2
\brief structure de base pour un point en 2D coordonnees en entier

*/

struct IPoint2
{
  
  ///data
  int x, y;
    
  //CONSTRUCTEURS
  
  ///constructeur defaut
  inline IPoint2() : x(0), y(0) {}
  ///constructeur
  inline IPoint2(const int& X,const int& Y)
                                 : x(X), y(Y) {}
  
  //ACCES
  
  ///acces
  inline int &operator[](const int& i) const {return ((int *)&x)[i] ;}
  ///acces
  inline int &operator[](const int& i) {return ((int *)&x)[i] ;}


  //METHODES CONSTANTES 
        

        
  //METHODES NON CONSTANTES


  inline IPoint2& operator=(const int &d)
  {
  x = d;
  y = d;
  return *this;
  }

  
  //FRIEND FONCTION

  inline friend bool operator!=(const IPoint2 &a,const IPoint2 &b) {
  if (a.x!=b.x)
        return true;
  if (a.y!=b.y)
        return true;
  return false;
  }
};



//------------------------------------------------------------------------
//-----VECTEUR
//------------------------------------------------------------------------


/** \class Vec2
\brief structure de base pour un vecteur 2D

*/

struct Vec2
{
  ///data
  GReal x, y;
  
  ///construteur
  inline Vec2() : x(0.0), y(0.0) {}
  ///constructeur
  inline Vec2(GReal X, GReal Y) : x(X), y(Y) {}

  ///acces
  inline GReal &operator[](const int& i) const {return ((GReal *)&x)[i] ;}
  ///acces
  inline GReal &operator[](const int& i) {return ((GReal *)&x)[i] ;}

  
  ///operation arithmetic
  inline friend Vec2 operator+(const Vec2 &a, const Vec2 &b) {
    return Vec2(a.x+b.x, a.y+b.y);
  }
  ///operation arithmetic
  inline friend Vec2 operator-(const Vec2 &a, const Vec2 &b) {
    return Vec2(a.x-b.x, a.y-b.y);
  }
  ///operation arithmetic
  inline friend Vec2 operator-(const Vec2 &a) {
    return Vec2(-a.x, -a.y);
  }
  ///operation arithmetic
  inline friend Vec2 operator*(const Vec2 &a, GReal k) {
    return Vec2(a.x*k, a.y*k) ;
  }
  ///operation arithmetic
  inline friend Vec2 operator*(GReal k, const Vec2 &a) {
    return Vec2(a.x*k, a.y*k) ;
  }
  ///operation arithmetic
  inline friend Vec2 operator/(const Vec2 &a, GReal k) {
    return Vec2(a.x/k, a.y/k) ;
  }
  ///operation arithmetic
  inline Vec2& operator+=(const Vec2 &a) {
    x += a.x; y += a.y;
    return *this;
  }
  ///operation arithmetic
  inline Vec2& operator-=(const Vec2 &a) {
    x -= a.x; y -= a.y;
        return *this;
  }
  ///operation arithmetic
  inline Vec2& operator*=(GReal k) {
    x *= k; y *= k;
        return *this;
  }
  ///operation arithmetic
  inline Vec2& operator/=(GReal k) {
    x /= k; y /= k;
        return *this;
  }
  
  ///operation arithmetic  
  inline friend GReal operator*(const Vec2 &a, const Vec2 &b) {
    return a.x*b.x + a.y*b.y;
  }
  
  ///operation arithmetic
  inline friend GReal norm2(const Vec2 &a) {
    return a*a ;
  }
  ///operation arithmetic
  inline friend GReal norm(const Vec2 &a) {
    return sqrt(a*a) ;
  }
  ///operation arithmetic
  inline void normalize() {
    *this /= norm(*this) ;
  }
  
  ///calcul le vecteur normal dans le plan
  inline friend Vec2 CalculVecteurNormal(const Vec2& v) {
  return Vec2 (v.y,-v.x);
  }
};




/** \class Vec3
\brief structure de base pour un vecteur 3D

*/

struct Vec3
{
  ///data
  GReal x, y, z;
  
  ///constructeur
  inline Vec3() : x(0.0), y(0.0), z(0.0) {}
  ///constructeur
  inline Vec3(GReal X, GReal Y, GReal Z) : x(X), y(Y), z(Z) {}

  ///acces
  inline GReal &operator[](const int& i) const {return ((GReal *)&x)[i] ;}
  ///acces
  inline GReal &operator[](const int& i) {return ((GReal *)&x)[i] ;}

  
  ///operation arithmetic
  inline friend Vec3 operator+(const Vec3 &a, const Vec3 &b) {
    return Vec3(a.x+b.x, a.y+b.y, a.z+b.z);
  }
  ///operation arithmetic
  inline friend Vec3 operator-(const Vec3 &a, const Vec3 &b) {
    return Vec3(a.x-b.x, a.y-b.y, a.z-b.z);
  }
  ///operation arithmetic
  inline friend Vec3 operator-(const Vec3 &a) {
    return Vec3(-a.x, -a.y, -a.z);
  }
  ///operation arithmetic
  inline friend Vec3 operator*(const Vec3 &a, GReal k) {
    return Vec3(a.x*k, a.y*k, a.z*k) ;
  }
  ///operation arithmetic
  inline friend Vec3 operator*(GReal k, const Vec3 &a) {
    return Vec3(a.x*k, a.y*k, a.z*k) ;
  }
  ///operation arithmetic
  inline friend Vec3 operator/(const Vec3 &a, GReal k) {
    return Vec3(a.x/k, a.y/k, a.z/k) ;
  }
  ///operation arithmetic
  inline Vec3& operator+=(const Vec3 &a) {
    x += a.x; y += a.y; z += a.z;
    return *this;
  }
  ///operation arithmetic
  inline Vec3& operator-=(const Vec3 &a) {
    x -= a.x; y -= a.y; z -= a.z;
        return *this;
  }
  ///operation arithmetic
  inline Vec3& operator*=(GReal k) {
    x *= k; y *= k; z *= k;
        return *this;
  }
  ///operation arithmetic
  inline Vec3& operator/=(GReal k) {
    x /= k; y /= k; z /= k;
        return *this;
  }
  
  ///operation arithmetic  
  inline friend GReal operator*(const Vec3 &a, const Vec3 &b) {
    return a.x*b.x + a.y*b.y + a.z*b.z;
  }
  ///operation arithmetic
  inline friend Vec3 cross(const Vec3 &a, const Vec3 &b) {
    return Vec3(a.y*b.z - a.z*b.y,
                a.z*b.x - a.x*b.z,
                a.x*b.y - a.y*b.x);
  }
  ///operation arithmetic
  inline friend GReal norm2(const Vec3 &a) {
    return a*a ;
  }
  ///operation arithmetic
  inline friend GReal norm(const Vec3 &a) {
    return sqrt(a*a) ;
  }
  ///operation arithmetic
  inline void normalize() {
    *this /= norm(*this) ;
  }
};




/** \class Point2
\brief structure de base pour un point en 2D

*/

struct Point2
{
  
  ///data
  GReal x, y;
  
  
  //CONSTRUCTEURS
  
  ///constructeur defaut
  inline Point2() : x(0.0), y(0.0) {}
  ///constructeur
  inline Point2(const GReal& X,const GReal& Y)
                                 : x(X), y(Y) {}
  
  //ACCES
  
  ///acces
  inline GReal &operator[](const int& i) const {return ((GReal *)&x)[i] ;}
  ///acces
  inline GReal &operator[](const int& i) {return ((GReal *)&x)[i] ;}


  //METHODES CONSTANTES 
        
  ///operation arithmetic
  inline GReal distance (const Point2 &p)      const {
        return sqrt( (x-p.x)*(x-p.x) + (y-p.y)*(y-p.y) );
  } 
  
  ///operation arithmetic
  inline GReal distance2 (const Point2 &p)     const {
        return (x-p.x)*(x-p.x) + (y-p.y)*(y-p.y);
  }   
   
  //projection ortho du point sur la droite (a,b)
  inline Point2 proj2right(const Point2 &a,
                           const Point2 &b) const {
  Point2 m (b.x-a.x, b.y-a.y);
  return a + ((*this - a) * m) * m / a.distance2(b);
  }
  
  //distance du point a la droite (a,b)
  inline GReal dist2right(const Point2 &a,
                           const Point2 &b) const {
  return this->distance(proj2right(a, b));
  } 
  
        
  //METHODES NON CONSTANTES
   
  ///operation arithmetic 
  inline Point2& operator=(const GReal &d)
  {
  x = d;
  y = d;
  return *this;
  }
  
  ///operation arithmetic 
  inline Point2& operator+=(const Point2 &p) {
  x += p.x;
  y += p.y;
  return *this;
  }

  ///operation arithmetic 
  inline Point2& operator-=(const Point2 &p) {
  x -= p.x;
  y -= p.y;
  return *this;
  }
  
  ///operation arithmetic 
  inline Point2& operator*=(const GReal &d) {
  x *= d;
  y *= d;
  return *this;
  }

  ///operation arithmetic 
  inline Point2& operator/=(const GReal &d) {
  x /= d;
  y /= d;
  return *this;
  }
  
  ///operation arithmetic 
  inline void normalize() {
  const GReal tmp = sqrt(x * x + y * y);
  x /= tmp; y /= tmp;
  }
  
  //FRIEND FONCTION

  ///operation arithmetic 
  inline friend Point2 operator-(const Point2 &a) {
    return Point2(-a.x, -a.y);
  }
  
  ///operation arithmetic
  inline friend Point2 operator+(const Point2 &a, const Point2 &b){
    return Point2(a.x+b.x, a.y+b.y);
  }
  
  ///operation arithmetic
  inline friend Point2 operator-(const Point2 &a, const Point2 &b){
    return Point2(a.x-b.x, a.y-b.y);
  }
  
  ///operation arithmetic
  inline friend Point2 operator*(const GReal &d, const Point2 &p){
  return Point2(d * (p.x), d * (p.y));
  }
  
  ///operation arithmetic 
  inline friend Point2 operator*(const Point2 &p, const GReal &d){
  return Point2(d * (p.x), d * (p.y));
  }
  
  ///operation arithmetic 
  inline friend GReal operator*(const Point2 &a, const Point2 &b){
  return a.x * b.x + a.y * b.y;
  }
  
  ///operation arithmetic
  inline friend Point2 operator/(const Point2 &p, const GReal &d){
  return Point2(p.x / d, p.y / d);
  }
  
  ///operation arithmetic
  inline friend Point2 operator/(const Point2 &a, const Point2 &b){
  return Point2(a.x / b.x, a.y / b.y);
  } 
  
  inline friend bool operator!=(const Point2 &a,const Point2 &b) {
  if (a.x!=b.x)
        return true;
  if (a.y!=b.y)
        return true;
  return false;
  }
  
  //POINT INTERSECTION
  //POINT INTERSECTION 
  
  inline friend bool intersectDroiteDroiteX(const Point2& p0, const Vec2& v0,
                                             const GReal&valX,Point2& pResult){
  if (v0.x<=0.0001 && v0.x>=-0.0001) 
        return false;
  GReal a = v0.y/v0.x;
  GReal b = p0.y - a*p0.x;
  pResult.x=valX;
  pResult.y=a*valX+b;
  return true;                                           
  }
  
  inline friend bool intersectSegmentDroiteX(const Point2& p0, const Point2& p1,
                                                                                         const
GReal& valX,
                                                                                         Point2&
pResult){
  
  if (p0.x<p1.x)
        {if (valX<p0.x || valX>p1.x) return false;}
  else if (p0.x>p1.x)
        {if (valX<p1.x || valX>p0.x) return false;} 
  else return false;
  GReal a = (p1.y-p0.y)/(p1.x-p0.x);
  GReal b = p0.y - a*p0.x;
  pResult.x=valX;
  pResult.y=a*valX+b;
  return true;                                           
  } 
  
  inline friend bool IntersectionDroiteDroite (const Point2& p0,
                                                                                  const
Vec2& v0,
                                                                                  const
Point2& p1,
                                                                                  const
Vec2& v1,
                                                                                  Point2&
pResult){
  
  if (v0.x==0.0)
        {
        if (v1.x!=0.0) return intersectDroiteDroiteX(p1,v1,p0.x,pResult);
        else return false;
        }
  if (v1.x==0.0)
        {
        if (v0.x!=0.0) return intersectDroiteDroiteX(p0,v0,p1.x,pResult);
        else return false;
        }
  GReal a0 = v0.y/v0.x;
  GReal b0 = p0.y - a0*p0.x;
  
  GReal a1 = v1.y/v1.x;
  GReal b1 = p1.y - a1*p1.x;
  
  if (a0-a1<=0.0001 && a0-a1>=-0.0001) return false;
  pResult.x=(b1-b0)/(a0-a1);
  pResult.y=a0*pResult.x+b0;
  return true;
  }
  

  
  
  
};





/** \class Point3
\brief structure de base pour un point en 3D

*/

struct Point3
{
  
  ///data
  GReal x, y, z;
  
  
  //CONSTRUCTEURS
  
  ///constructeur defaut
  inline Point3() : x(0.0), y(0.0), z(0.0) {}
  ///constructeur
  inline Point3(const GReal& X,const GReal& Y,const GReal& Z)
                                 : x(X), y(Y), z(Z) {}
  
  //ACCES
  
  ///acces
  inline GReal &operator[](const int& i) const {return ((GReal *)&x)[i] ;}
  ///acces
  inline GReal &operator[](const int& i) {return ((GReal *)&x)[i] ;}


  //METHODES CONSTANTES 
        
  ///operation arithmetic
  inline GReal distance (const Point3 &p)      const {
        return sqrt( (x-p.x)*(x-p.x) + (y-p.y)*(y-p.y) + (z-p.z)*(z-p.z) );
  } 
  
  ///operation arithmetic
  inline GReal distance2 (const Point3 &p)     const {
        return (x-p.x)*(x-p.x) + (y-p.y)*(y-p.y) + (z-p.z)*(z-p.z);
  }   
   
  //projection du point sur la droite (a,b)
  inline Point3 proj2right(const Point3 &a,
                           const Point3 &b) const {
  Point3 m = (b - a);
  return a + ((*this - a) * m) * m / a.distance2(b);
  }
  
  //distance du point a la droite (a,b)
  inline GReal dist2right(const Point3 &a,
                           const Point3 &b) const {
  return this->distance(proj2right(a, b));
  } 
  
        
  //METHODES NON CONSTANTES
   
  ///operation arithmetic 
  inline Point3& operator=(const GReal &d)
  {
  x = d;
  y = d;
  z = d;
  return *this;
  }
  
  ///operation arithmetic 
  inline Point3& operator+=(const Point3 &p) {
  x += p.x;
  y += p.y;
  z += p.z;
  return *this;
  }

  ///operation arithmetic 
  inline Point3& operator-=(const Point3 &p) {
  x -= p.x;
  y -= p.y;
  z -= p.z;
  return *this;
  }
  
  ///operation arithmetic 
  inline Point3& operator*=(const GReal &d) {
  x *= d;
  y *= d;
  z *= d;
  return *this;
  }

  ///operation arithmetic 
  inline Point3& operator/=(const GReal &d) {
  x /= d;
  y /= d;
  z /= d;
  return *this;
  }
  
  ///operation arithmetic 
  inline void normalize() {
  const GReal tmp = sqrt(x * x + y * y + z * z);
  x /= tmp; y /= tmp; z /= tmp;
  }
  
  //FRIEND FONCTION

  ///operation arithmetic 
  inline friend Point3 operator-(const Point3 &a) {
    return Point3(-a.x, -a.y, -a.z);
  }
  
  ///operation arithmetic
  inline friend Point3 operator+(const Point3 &a, const Point3 &b){
    return Point3(a.x+b.x, a.y+b.y, a.z+b.z);
  }
  
  ///operation arithmetic
  inline friend Point3 operator-(const Point3 &a, const Point3 &b){
    return Point3(a.x-b.x, a.y-b.y, a.z-b.z);
  }
  
  ///operation arithmetic
  inline friend Point3 operator*(const GReal &d, const Point3 &p){
  return Point3(d * (p.x), d * (p.y), d * (p.z));
  }
  
  ///operation arithmetic 
  inline friend Point3 operator*(const Point3 &p, const GReal &d){
  return Point3(d * (p.x), d * (p.y), d * (p.z));
  }
  
  ///operation arithmetic 
  inline friend GReal operator*(const Point3 &a, const Point3 &b){
  return a.x * b.x + a.y * b.y + a.z * b.z;
  }
  
  ///operation arithmetic
  inline friend Point3 operator/(const Point3 &p, const GReal &d){
  return Point3(p.x / d, p.y / d, p.z / d);
  }
  
  ///operation arithmetic
  inline friend Point3 operator/(const Point3 &a, const Point3 &b){
  return Point3(a.x / b.x, a.y / b.y, a.z / b.z);
  } 
  
  inline friend bool operator!=(const Point3 &a,const Point3 &b) {
  if (a.x!=b.x)
        return true;
  if (a.y!=b.y)
        return true;
  if (a.z!=b.z)
        return true;
  return false;
  }
};


///Calcul le potentiel d'influence du point p par rapport au
/// segment [p0-p1].	
inline GReal PotentielPoint(const Point3& p,
			    const Point3& p0,
       			    const Point3& p1)
		{
		if (p.dist2right(p0,p1) == 0.0)
			return 0.0;
				
 	 	Point3 tmp2 = p1 - p0;
 		Point3 r = p0 + ((p - p0) * tmp2) * (tmp2 / p0.distance2(p1));
  		GReal cos0 = sqrt(p0.distance2(r) / p0.distance2(p));
 	 	GReal cos1 = sqrt(p1.distance2(r) / p1.distance2(p));
  		GReal h2 = p.distance2(r);
  		GReal tmp = r * (p1 - p0);
		
 	 	if (tmp < p0 * tmp2)
  	  			return (0.5 * (cos1 - cos0) / h2);
  		if (p1 * tmp2 < tmp)
  	  			return (0.5 * (cos0 - cos1) / h2); 		
		return (0.5 * (cos0 + cos1) / h2);
		}
	
///Calcul le potentiel d'influence du point p par rapport au
/// segment [p0-p1] -> calcul en 2D (plus rapide), evite le 
/// probleme d'une distance == 0 ou proche de 0 (le point
/// p est toujours au dessus de la droite) 
inline GReal PotentielPointMod(const Point2& p,
			       const Point2& p0,
       			       const Point2& p1,
			       const GReal& minDistance)
		{			
		Point2 tmp2 = p1 - p0;
 		Point2 r = p0 + ((p - p0) * tmp2) * (tmp2 / p0.distance2(p1));
  		GReal cos0 = sqrt(p0.distance2(r) / (p0.distance2(p)+minDistance));
 	 	GReal cos1 = sqrt(p1.distance2(r) / (p1.distance2(p)+minDistance));
  		GReal h2 = p.distance2(r) + minDistance;
  		GReal tmp = r * (p1 - p0);
 	 	if (tmp < p0 * tmp2)
  	  			return (0.5 * (cos1 - cos0) / h2);
  		if (p1 * tmp2 < tmp)
  	  			return (0.5 * (cos0 - cos1) / h2); 		
		return (0.5 * (cos0 + cos1) / h2);
		}
	
	
	
///renvoi vrai si les deux points 2D sont voisins
inline bool PointsVoisins(const IPoint2& p1, const IPoint2& p2)
	{
	if (Abs(p1.x-p2.x)>1)
		return false;
	if (Abs(p1.y-p2.y)>1)
		return false;
	return true;
	} 



#endif /* _geometrie_base_ */
