#ifndef _grass_defines_
#define _grass_defines_

#define PI 3.14159f	

#define GRASS_NUMBER_PATCHE_TYPE	4
#define GRASS_NUMBER_PATCHE_TYPE_1	0.25f

#define GRASS_TEXTURE2D_SIZE_X		128
#define GRASS_TEXTURE2D_SIZE_Y		32

#define DRAWGRASS_3D 			4
#define DRAWGRASS_TRANSITION_3D_2D5	3
#define DRAWGRASS_2D5			2
#define DRAWGRASS_2D			1
#define DRAWGRASS_NONE			0

#define DRAWOPTION_NONE 		0
#define DRAWOPTION_WIRE 		1
#define DRAWOPTION_TRAJECTORY 		2
#define DRAWOPTION_DENSITY 		3

#define CONV_RAD_TO_DEGREE 		57.29583f
#define CONV_DEGREE_TO_RAD 		0.017453f

#define GRASS_DEPTH_XMIN_NEAREST 	0x01
#define GRASS_DEPTH_XMAX_FURTHER 	GRASS_DEPTH_XMIN_NEAREST
#define GRASS_DEPTH_XMAX_NEAREST 	0x02
#define GRASS_DEPTH_XMIN_FURTHER 	GRASS_DEPTH_XMAX_NEAREST
#define GRASS_DEPTH_YMIN_NEAREST 	0x04
#define GRASS_DEPTH_YMAX_FURTHER 	GRASS_DEPTH_YMIN_NEAREST
#define GRASS_DEPTH_YMAX_NEAREST 	0x08
#define GRASS_DEPTH_YMIN_FURTHER 	GRASS_DEPTH_YMAX_NEAREST

typedef float GReal;
typedef unsigned char GBitfield;



//fonction simple---------------------------	
inline short Abs(const short& d)
	{
	return d<0?-d:d;
	}

inline int Abs(const int& d)
	{
	return d<0?-d:d;
	}

inline GReal Abs(const GReal& d)
	{
	return d<0.0f?-d:d;
	}

inline short Max(const short& a,const short& b)
	{
	if (a>b) return a;
	return b;
	}

inline int Max(const int& a,const int& b)
	{
	if (a>b) return a;
	return b;
	}
	
inline GReal Max(const GReal& a,const GReal& b)
	{
	if (a>b) return a;
	return b;
	}

inline short Min(const short& a,const short& b)
	{
	if (a>b) return b;
	return a;
	}

inline int Min(const int& a,const int& b)
	{
	if (a>b) return b;
	return a;
	}

inline GReal Min(const GReal& a,const GReal& b)
	{
	if (a>b) return b;
	return a;
	}

inline short Signe(const short& s)
	{
	if (s>0) return 1;
	else if (s==0) return 0;
	return -1;
	}
		
inline int Signe(const int& s)
	{
	if (s>0) return 1;
	else if (s==0) return 0;
	return -1;
	}
	
inline GReal Signe(const GReal& s)
	{
	if (s>0.0f) return 1.0f;
	else if (s==0.0f) return 0.0f;
 	return -1.0f;
	}

inline GReal powShort(GReal a, short b)
	{
	GReal result = 1.0f;
	while (b > 0) 
	    {
	    if (b & 1) result *= a;
	    a *= a;
	    b /= 2;
	    }
	return result;
	}


//-----------------------------------------


/// Courbes de Hermite: les 4 fonctions de base-------
inline GReal H0( const GReal &t )
	{
        return (2.0f*t-3.0f)*t*t+1.0f;
	}
inline GReal H_0( const GReal &t )
	{
        return ((t-2.0f)*t+1.0f)*t;
	}
inline GReal H1( const GReal &t )
	{
        return (-2.0f*t+3.0f)*t*t;
	}
inline GReal H_1( const GReal &t )
	{
        return (t-1.0f)*t*t;
	}	
inline GReal Hermite(const GReal &t, 
		     const GReal &coef0,const GReal &coef_0,
		     const GReal &coef1,const GReal &coef_1)
	{
	GReal tt=t*t;
	return (((2.0f*t-3.0f)*tt+1.0f)*coef0 + (((t-2.0f)*t+1.0f)*t)*coef_0
	       +((-2.0f*t+3.0f)*tt)*coef1 + ((t-1.0f)*tt)*coef_1);
	} 	
//-----------------------------------------------------

#endif
