#ifndef _grass_engine_
#define _grass_engine_

//include base prairie
#include "GeometrieBase.hpp"
#include "InformationPrimitive.h"

//autres
#include "GrassDefines.h"
#include "GrassRandom.h"
#include "SharedData.h"
#include "VolumeVision.hpp"
#include "Noeud.hpp"
#include "Carre_Herbe.hpp"
#include "Herbe.hpp"
#include "GrassTrajectory.h"

//autre
#include "TextureGrass.h"

//--------------------------------------
//--------------------------------------

class Carre_Herbe;

class GrassEngine
{
protected:  
  //texture
  static TextureGrass ms_texture2D;
  static TextureGrass ms_texture2D5;
  static TextureGrass ms_textureOnGrass;
  int m_graineTexture2D5;
  int m_graineTexture2D;
  GReal m_antiAliasingTexture3D;
  GReal m_antiAliasingTexture2D5;
  GReal m_antiAliasingTexture2D;
  int m_Texture2D5SizeX;
  int m_Texture2D5SizeY;
  
  //area, patches, nodes
  Carre_Herbe** m_pElements;
  Noeud**       m_pNoeuds;
  NodeCrushing**  m_pNodesCrushing;
  
  GReal m_area[4];
  GReal m_areaLength;
  GReal m_areaWidth;  
  short m_numberOfPatchesInLength;
  short m_numberOfPatchesInWidth;
  int m_numberOfNoeuds;
  int m_numberOfElements;
  GReal m_patchesLength;
  GReal m_patchesWidth;
  
  //density
  char m_fileDensityMap[256];
  char m_fileTextureOnGrass[256];
  
  //flags	
  bool m_initDisplay;
  bool m_textureOnGrass;

public:
  GrassEngine();
  ~GrassEngine();
  
  //fonction initialisation
  void init(const float area[4],const short& numberOfPatchesInLength,const short& numberOfPatchesInWidth,char* fileDensityMap);

  const GReal* getArea(){return m_area;}
  GReal getAreaXMin(){return m_area[0];}
  GReal getAreaXMax(){return m_area[2];}
  GReal getAreaYMin(){return m_area[1];}
  GReal getAreaYMax(){return m_area[3];}	
  GReal getAreaLength(){return m_areaLength;}
  GReal getAreaWidth(){return m_areaWidth;}

  short getNumberOfPatchesInLength(){return m_numberOfPatchesInLength;}
  short getNumberOfPatchesInWidth(){return m_numberOfPatchesInWidth;}
  
  const char* getFileDensityMap(){return m_fileDensityMap;}

  //fonctions modif/acces attributs
  void setGrassHeight(const GReal& d)
  	{
	short i,j;
	Herbe::ms_height=d;
	for (i=0;i<m_numberOfPatchesInLength+1;i++)
	for (j=0;j<m_numberOfPatchesInWidth+1;j++)
   		m_pNoeuds[i][j].SetGrassHeight(d);
	}
  void setGrassWidth(const GReal& d)
  	{
	Herbe::ms_width=d;
	if (m_initDisplay)
		{
		makeTexture2D();
		makeTexture2D5();
		}
	}
  void setNoisePositions(const GReal& d) {Herbe::m_noisePositions=d;}
  void setNoiseNormals(const GReal& d) {Herbe::m_noiseNormals=d;}
  void setNumberPerSquareMeters(const short& n)
  	{
	SharedData::setNumberGrassesPerSquareMeters(n);
	SharedData::setNumberMaxGrassesPerPatche(
	  short(SharedData::m_numberGrassesPerSquareMeters
	  	*m_patchesLength*m_patchesWidth*0.1f));	
	if (m_initDisplay)
		{
		makeTexture2D();
		makeTexture2D5();
		}
	}
  void setTextureOnGrass(bool b);
  void setFileTextureOnGrass(const char* file);
  void setAntiAliasingTextureOnGrass(GReal r);
  void regenerateTexture2D5();
  void setGraineTexture2D5(int i) {m_graineTexture2D5=i;makeTexture2D5();}
  void setSizeTexture2D5(int x, int y)
		{
		m_Texture2D5SizeX=x;
  		m_Texture2D5SizeY=y;
		makeTexture2D5();
		}
		
		
  void setAntiAliasingTexture2D5(GReal r);	
  	
	
  GReal getGrassHeight(){return Herbe::ms_height;}
  GReal getGrassWidth(){return Herbe::ms_width;}
  GReal getNoisePositions(){return Herbe::m_noisePositions;}
  GReal getNoiseNormals(){return Herbe::m_noiseNormals;}
  short getNumberPerSquareMeters()
  	{
	return SharedData::m_numberGrassesPerSquareMeters;
	}
  bool isTextureOnGrass() {return m_textureOnGrass;}
  const char* getFileTextureOnGrass()  {return m_fileTextureOnGrass;}
  GReal getAntiAliasingTextureOnGrass() {return m_antiAliasingTexture3D;}
  GReal getAntiAliasingTexture2D5() {return m_antiAliasingTexture2D5;}
  int getGraineTexture2D5() {return m_graineTexture2D5;}
  void getSizeTexture2D5(int& x, int &y){x=m_Texture2D5SizeX;y=m_Texture2D5SizeY;}


  //LOD
  void setDistance3D(const GReal& d){SharedData::setDistance3D(d);}
  void setDistance2D5(const GReal& d){SharedData::setDistance2D5(d);}
  void setDistance2D(const GReal& d){SharedData::setDistance2D(d);}
  void setNumberPG3D(const short& n){SharedData::setNumberPG3D(n);}
  void setNumberPG2D5(const short& n){SharedData::setNumberPG2D5(n);}
  void setAjustmentHeight2D5(bool b) {SharedData::setAjustmentHeight2D5(b);}

	
  GReal getDistance3D(){return SharedData::m_distance3D;}
  GReal getDistance2D5(){return SharedData::m_distance2D5;}
  GReal getDistance2D(){return SharedData::m_distance2D;}
  short getNumberPG3D(){return SharedData::m_numberPG3D;}
  short getNumberPG2D5(){return SharedData::m_numberPG2D5;}
  bool getAjustmentHeight2D5() {return SharedData::m_enableAjustmentHeight2D5;}
   
  //crushing behavior
  void setCrushingHighPrecision(bool b){SharedData::setCrushingHighPrecision(b);}
  void setCrushingMaxTime(const GReal& d){NodeCrushing::ms_crushingMaxTime=d;}
  void setCrushingDamping(const GReal& d){NodeCrushing::ms_crushingDamping=d;}
  
  bool getCrushingHighPrecision(){return SharedData::m_enableCrushingHighPrecision;}
  GReal getCrushingMaxTime(){return NodeCrushing::ms_crushingMaxTime;}
  GReal getCrushingDamping(){return NodeCrushing::ms_crushingDamping;}
  
  //return info
  GReal getDensityAtPosition(const GReal& x, const GReal& y);	  
  
  //fonction animation
  void update(GReal dt,
	      GrassTrajectory* pTrajectories,
	      const short& numberOfTrajectoiries);
  
  //fonction dessin
  void predraw(const Point3& positionCamera);
  
  //fonction dessin
  void draw();

  //options dessin    
  void drawNoeuds() const;
  void drawDensity() const;
  void drawTrajectory() const;
  void drawCrushingVector() const;
  void drawTexture2D() const;
  void drawTexture2D5() const;
  
  
protected:
   void drawInWire();
   
   void initCreateTexture();
   void endCreateTexture();
   void makeTexture2D();
   void makeTexture2D5();
   
   
   GReal getDensityMapAtPosition(const GReal& x, const GReal& y,
                                  const unsigned char* densityMap,
                                  const int densityMapCols,const int densityMapRows) const;
  
   ///mise a jour de l'influence d'une trajectoire sur les noeuds
   inline void UpdateInfluenceTrajectoire(GrassTrajectory * trajectory);
   inline void PropagationInfluenceSegmentTrajectoire(short indice1,
   						      short indice2,
   						      short propagation11,
						      short propagation12,
						      short propagation21,
						      short propagation22,						      
						      const Point3& lastPosition,
						      const Point3& currentPosition,
						      const Vec2& vecteurC,
						      const Vec2& vecteurN,
						      const GrassTrajectory* trajectory);
   inline void PropagationInfluenceSegmentTrajectoireBiResolution
						      (short indice1,
   						       short indice2,
						       short propagationV,
						       bool propagationVBoth,
						       short propagationH,
						       bool propagationHBoth,						      
						       const Point3& lastPosition,
						       const Point3& currentPosition,
						       const Vec2& vecteurC,
						       const Vec2& vecteurN,
						       const GrassTrajectory* trajectory);

};


//------------------------------------------------------------------------
//-------------fonction calcul infuence trajectoire-----------------------
//------------------------------------------------------------------------


//mise a jour de l'influence d'une trajectoire sur les noeuds
void GrassEngine::UpdateInfluenceTrajectoire(GrassTrajectory* trajectory)
{
Point3 lastPosition	(GReal(trajectory->m_lastPosition[0]),
			 GReal(trajectory->m_lastPosition[1]),
			 GReal(trajectory->m_lastPosition[2]));
Point3 currentPosition	(GReal(trajectory->m_currentPosition[0]),
			 GReal(trajectory->m_currentPosition[1]),
			 GReal(trajectory->m_currentPosition[2]));
if (currentPosition!=lastPosition)
	{
	short CoordPatche[2];		
	
	//calcul des coordonnees du premier carre
	CoordPatche[0] = short((lastPosition.x-m_area[0])/(m_patchesLength*0.5f));
	CoordPatche[1] = short((lastPosition.y-m_area[1])/(m_patchesWidth*0.5f));
	if (CoordPatche[0]<0 || CoordPatche[0]>=m_numberOfPatchesInLength*2) return;	
	if (CoordPatche[1]<0 || CoordPatche[1]>=m_numberOfPatchesInWidth*2) return;		
	//vecteurs
	Vec2 vecteurColineaire(currentPosition.x-lastPosition.x,
			       currentPosition.y-lastPosition.y);
	if (vecteurColineaire.x!=0.0 && vecteurColineaire.y!=0.0) vecteurColineaire.normalize();
	Vec2 vecteurNormal;
	vecteurNormal.x=vecteurColineaire.y;
	vecteurNormal.y=-vecteurColineaire.x;
					
	PropagationInfluenceSegmentTrajectoire(
			      CoordPatche[0],
			      CoordPatche[1],
			      -1,
			      0,
			      0,
			      -1,
			      lastPosition,
			      currentPosition,
			      vecteurColineaire,
			      vecteurNormal,
			      trajectory);
	PropagationInfluenceSegmentTrajectoire(
			      CoordPatche[0]+1,
			      CoordPatche[1],
			      1,
			      0,
			      0,
			      -1,
			      lastPosition,
			      currentPosition,
			      vecteurColineaire,
			      vecteurNormal,
			      trajectory);
	PropagationInfluenceSegmentTrajectoire(
			      CoordPatche[0],
			      CoordPatche[1]+1,
			      -1,
			      0,
			      0,
			      1,
			      lastPosition,
			      currentPosition,
			      vecteurColineaire,
			      vecteurNormal,
			      trajectory);
	PropagationInfluenceSegmentTrajectoire(
			      CoordPatche[0]+1,
			      CoordPatche[1]+1,
			      1,
			      0,
			      0,
			      1,
			      lastPosition,
			      currentPosition,
			      vecteurColineaire,
			      vecteurNormal,
			      trajectory);
	
		
	//mise a jour de la trajectoire
	trajectory->m_lastPosition[0]=trajectory->m_currentPosition[0];
	trajectory->m_lastPosition[1]=trajectory->m_currentPosition[1];
	trajectory->m_lastPosition[2]=trajectory->m_currentPosition[2];			
	}		
}	
	

void GrassEngine::PropagationInfluenceSegmentTrajectoire(short indice1,
   							 short indice2,
   							 short propagation11,
							 short propagation12,
							 short propagation21,
							 short propagation22,						      
							 const Point3& lastPosition,
							 const Point3& currentPosition,
							 const Vec2& vecteurC,
							 const Vec2& vecteurN,
							 const GrassTrajectory* trajectory)
{

if (indice1 <0 || indice1>m_numberOfPatchesInLength*2) return;
if (indice2 <0 || indice2>m_numberOfPatchesInWidth*2) return;
if (m_pNodesCrushing[indice1][indice2].UpdateInfluenceTrajectoire(lastPosition,
						currentPosition,
						vecteurC,
						vecteurN,
						trajectory->m_power,
						trajectory->m_coefCrushingAccordingToNormal)>trajectory->m_propagationThreshold)
	{
	if (propagation11!=0 || propagation12!=0)
	PropagationInfluenceSegmentTrajectoire(
					indice1+propagation11,
					indice2+propagation12,
					propagation11,
					propagation12,
					propagation21,
					propagation22,
					lastPosition,currentPosition,
					vecteurC,vecteurN,trajectory);
	PropagationInfluenceSegmentTrajectoire(
					indice1+propagation21,
					indice2+propagation22,
					0,
					0,
					propagation21,
					propagation22,
					lastPosition,currentPosition,
					vecteurC,vecteurN,trajectory);
	}
}
	



#endif
