#ifndef _grass_interface_
#define _grass_interface_

#include "GrassTrajectory.h"

//XML
#include <libxml/parser.h>

//-----------------------------------------------------------------------------
//----------------------------fonctions a implementer--------------------------
//-----------------------------------------------------------------------------

//calcul du vent en fonction d'une position (x,y) 
//Cette fonction doit etre implementer dans votre code pour le link
void grassGetLocalWind(const float& x, const float& y,
                       const float& windInitPower,
		       const float& windInitDirX,
		       const float& windInitDirY,
		       float& windPower, 
		       float& windDirX, 
		       float& windDirY);

//calcul de la hauteur en fonction d'une position (x,y) 
//Cette fonction doit etre implementer dans votre code pour le link
float grassGetHeightAtPosition(const float& x, const float& y);



//-----------------------------------------------------------------------------
//----------------------------fonctions principales----------------------------
//-----------------------------------------------------------------------------

//initialisation
//area : {xmin,ymin,xmax,ymax}
//fileDensityMap : type bmp ou ppm
void grassInit(const float area[4],
               const short& numPatchesInLength, const short& numPatchesInWidth,
	       const char* fileDensityMap);
//mise a jour animation -> ecrasement
void grassUpdate(float dt,  
	         GrassTrajectory* pTrajectories,
	         const short& numberOfTrajectories);
//calcul affichage
//ne pas l'appeler (quand l'herbe est deja en place sinon vous ne verrez jamais rien)
//est utile pour les tests car la fonction grassDraw affichera ensuite
//toujours la meme chose en toutes circonstances
void grassPreDraw(float cameraPosition[3]);
//affichage
void grassDraw(); 
	      

//-----------------------------------------------------------------------------
//----------------------------fonctions secondaires----------------------------
//-----------------------------------------------------------------------------

//ATTRIBUTES
void grassSetHeight(const float& d);
void grassSetWidth(const float& d);
void grassSetNoisePositions(const float& d);
void grassSetNoiseNormals(const float& d);
void grassSetNumberPerSquareMeters(const short& n);
void grassSetTextureOnGrass(bool b);
void grassSetFileTextureOnGrass(const char* file);
void grassSetAntiAliasingTextureOnGrass(float d);
void grassRegenerateTexture2D5();
void grassSetAntiAliasingTexture2D5(float d);
void grassSetSizeTexture2D5(int x, int y);
void grassSetGraineTexture2D5(int i);
//CRUSHING BEHAVIOR
void grassSetCrushingMaxTime(const float& d);
void grassSetCrushingDamping(const float& d);

//LOD
void grassSetDistance3D(const float& d);
void grassSetDistance2D5(const float& d);
void grassSetDistance2D(const float& d);
void grassSetPG3DGrass(const short& n);
void grassSetPG2D5Grass(const short& n);

void grassSetAjustmentHeight2D5(bool b);

//RETURN INFORMATIONS
int grassGetPolygonsDrawedPerFrame();
float grassGetDensityPerSquareMeterAtPosition(const float& x, const float& y);

//RETURN PARAMS
float grassGetAreaXMin();
float grassGetAreaXMax();
float grassGetAreaYMin();
float grassGetAreaYMax();
float grassGetAreaLength();
float grassGetAreaWidth();
short grassGetNumberPatchesInLength();
short grassGetNumberPatchesInWidth();
const char* grassGetFileDensityMap();
bool grassIsTextureOnGrass();
const char* grassGetFileTextureOnGrass();
float grassGetAntiAliasingTextureOnGrass();
float grassGetAntiAliasingTexture2D5();
int grassGetGraineTexture2D5();
void grassGetSizeTexture2D5(int& x, int &y);

float grassGetHeight();
float grassGetWidth();
float grassGetNoisePositions();
float grassGetNoiseNormals();
short grassGetNumberPerSquareMeters();
float grassGetCrushingMaxTime();
bool grassGetAjustmentHeight2D5();
float grassGetCrushingDamping();
float grassGetDistance3D();
float grassGetDistance2D5();
float grassGetDistance2D();
short grassGetPG3DGrass();
short grassGetPG2D5Grass();

//-----------------------------------------------------------------------------
//----------------------------fonctions de tests-------------------------------
//-----------------------------------------------------------------------------

void grassDrawNoeuds();
void grassDrawDensity();
void grassDrawTrajectory();
void grassDrawCrushingVector();
void grassDrawInWireMode();
void grassDrawTexture2D();
void grassDrawTexture2D5();

//XML
bool grassLoadXML(const char* filename);
bool grassLoadXML(xmlNodePtr p_nodePrairie);
void grassSaveXML(xmlNodePtr p_nodePrairie);

#endif
