
#include "GrassInterface.h"

#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/parserInternals.h>

#include <string>
#include <vector>



void grassXMLtokenize(const std::string& str,
	      std::vector<std::string>& tokens,
	      const std::string& delimiters) 
{
  std::string::size_type lastPos = str.find_first_not_of(delimiters, 0);
  std::string::size_type pos = str.find_first_of(delimiters, lastPos);
  
  while (std::string::npos != pos || std::string::npos != lastPos) {
    tokens.push_back(str.substr(lastPos, pos - lastPos));
    lastPos = str.find_first_not_of(delimiters, pos);
    pos = str.find_first_of(delimiters, lastPos);
  }
}

void grassXMLgetFloatArray(xmlNodePtr node, char* attribname, float x[]) 
{
  std::string text = (const char*) xmlGetProp(node, (const xmlChar *) attribname);
  std::vector<std::string> tokens;
  grassXMLtokenize(text, tokens, " ");
  for (int i=0; i<(int)tokens.size(); i++) 
    {
    std::string test = tokens[i];
    x[i] = atof(test.c_str());
    }
}


bool grassLoadNode(xmlNodePtr p_nodeGrass)
{
//variables en attente d'initialisation
float area[]={grassGetAreaXMin(),grassGetAreaYMin(),
	      grassGetAreaXMax(),grassGetAreaYMax()};
float grass_height=grassGetHeight();
float grass_width=grassGetWidth();
float noise_positions=grassGetNoisePositions();
float noise_normals=grassGetNoiseNormals();
short patches_number[]={grassGetNumberPatchesInLength(),
			grassGetNumberPatchesInWidth()};

short lod_3D_pg=grassGetPG3DGrass();
float lod_3D_dist=grassGetDistance3D();
short lod_2D5_pg=grassGetPG2D5Grass();
float lod_2D5_dist=grassGetDistance2D5();
float lod_2D_dist=grassGetDistance2D();

const char* density_map=grassGetFileDensityMap();
short number_squaremeters=grassGetNumberPerSquareMeters();
float crushing_maxtime=grassGetCrushingMaxTime();
float crushing_damping=grassGetCrushingDamping();

bool ajustment_height_2D5=grassGetAjustmentHeight2D5();

bool texture3D=grassIsTextureOnGrass();
const char* file_texture3D=grassGetFileTextureOnGrass();
float antialiasing_texture3D=grassGetAntiAliasingTextureOnGrass();
float antialiasing_texture2D5=grassGetAntiAliasingTexture2D5();
int graine_texture2D5=grassGetGraineTexture2D5();
int sizex_texture2D5;
int sizey_texture2D5;
grassGetSizeTexture2D5(sizex_texture2D5,sizey_texture2D5);	
	
	
//parcours		 
xmlAttrPtr attr = p_nodeGrass->properties;
while (attr != NULL) 
   {
   std::string attribute = (const char*) attr->name;
   if (attribute=="area")
   	{
	grassXMLgetFloatArray(p_nodeGrass, "area", area);
   	}  
   if (attribute=="grass_height")
   	{
   	grass_height = atof((const char*) xmlGetProp(p_nodeGrass, (const xmlChar *)"grass_height"));
   	}
   if (attribute=="grass_width")
   	{
   	grass_width = atof((const char*) xmlGetProp(p_nodeGrass, (const xmlChar *)"grass_width"));
   	}
   if (attribute=="noise_positions")
   	{
   	noise_positions = atof((const char*) xmlGetProp(p_nodeGrass, (const xmlChar*)"noise_positions"));
   	}
   if (attribute=="noise_normals")
   	{
   	noise_normals = atof((const char*) xmlGetProp(p_nodeGrass, (const xmlChar*)"noise_normals"));
   	}
   if (attribute=="patches_number")
   	{
	float tmp[2];
	grassXMLgetFloatArray(p_nodeGrass, "patches_number", tmp);
   	patches_number[0]=short(tmp[0]);
	patches_number[1]=short(tmp[1]);
   	}
   if (attribute=="lod_3D")
   	{
	float tmp[2];
	grassXMLgetFloatArray(p_nodeGrass, "lod_3D", tmp);
   	lod_3D_pg=short(tmp[0]);
	lod_3D_dist=tmp[1];
   	}
   if (attribute=="lod_2D5")
   	{
	float tmp[2];
	grassXMLgetFloatArray(p_nodeGrass, "lod_2D5", tmp);
   	lod_2D5_pg=short(tmp[0]);
	lod_2D5_dist=tmp[1];
   	}
   if (attribute=="lod_2D")
   	{
	float tmp[2];
	grassXMLgetFloatArray(p_nodeGrass, "lod_2D", tmp);
	lod_2D_dist=tmp[1];
   	}
   if (attribute=="density_map")
   	{
	density_map = (const char*) xmlGetProp(p_nodeGrass, (const xmlChar *) "density_map");
   	}
   if (attribute=="number_squaremeters")
   	{
	number_squaremeters = atoi((const char*) xmlGetProp(p_nodeGrass, (const xmlChar*)"number_squaremeters"));
   	}
   if (attribute=="crushing_maxtime")
   	{
	crushing_maxtime = atof((const char*) xmlGetProp(p_nodeGrass, (const xmlChar *)"crushing_maxtime"));
   	}
   if (attribute=="crushing_damping")
   	{
	crushing_damping = atof((const char*) xmlGetProp(p_nodeGrass, (const xmlChar *)"crushing_damping"));
   	}
   if (attribute=="ajustment_height_2D5")
   	{
	if (!strcmp("true",((const char*) xmlGetProp(p_nodeGrass, (const xmlChar *)"ajustment_height_2D5"))))
   	   ajustment_height_2D5=true;
	else
	   ajustment_height_2D5=false;
	}
   if (attribute=="texture3D")
   	{
	if (!strcmp("true",((const char*) xmlGetProp(p_nodeGrass, (const xmlChar *)"texture3D"))))
   	   texture3D=true;
	else
	   texture3D=false;
	}
   if (attribute=="file_texture3D")
   	{
	file_texture3D = (const char*) xmlGetProp(p_nodeGrass, (const xmlChar *) "file_texture3D");
	}
   if (attribute=="antialiasing_texture3D")
   	{
	antialiasing_texture3D = atof((const char*) xmlGetProp(p_nodeGrass, (const xmlChar *) "antialiasing_texture3D"));
	}
   if (attribute=="antialiasing_texture2D5")
   	{
	antialiasing_texture2D5 = atof((const char*) xmlGetProp(p_nodeGrass, (const xmlChar *) "antialiasing_texture2D5"));
	}
   if (attribute=="graine_texture2D5")
   	{
	graine_texture2D5 = atoi((const char*) xmlGetProp(p_nodeGrass, (const xmlChar *) "graine_texture2D5"));
	}
   if (attribute=="sizex_texture2D5")
   	{
	sizex_texture2D5 = atoi((const char*) xmlGetProp(p_nodeGrass, (const xmlChar *) "sizex_texture2D5"));
	}
   if (attribute=="sizey_texture2D5")
   	{
	sizey_texture2D5 = atoi((const char*) xmlGetProp(p_nodeGrass,(const xmlChar *) "sizey_texture2D5"));
	}
   attr=attr->next;
   }


//hack  la con 
//a faire pour eviter cela : dans grassEngine faire un fonction
//'build' qui construit la scene apres en avoir modifier les options
//pour l'instant  chaque fois que l'on change une option, la scene 
//se reconstruit, ce qui peut etre long si la texture est grande.  
grassSetSizeTexture2D5(256,32);
grassInit(area,
	  1,1,
	  density_map);
//fin hack  la con
	  	  
grassSetNumberPerSquareMeters(number_squaremeters);
grassSetHeight(grass_height);
grassSetWidth(grass_width);
grassSetNoisePositions(noise_positions);
grassSetNoiseNormals(noise_normals);
grassSetCrushingMaxTime(crushing_maxtime);
grassSetCrushingDamping(crushing_damping);
grassSetDistance3D(lod_3D_dist);
grassSetDistance2D5(lod_2D5_dist);
grassSetDistance2D(lod_2D_dist);
grassSetPG3DGrass(lod_3D_pg);
grassSetPG2D5Grass(lod_2D5_pg);
grassSetAjustmentHeight2D5(ajustment_height_2D5);
grassSetAntiAliasingTextureOnGrass(antialiasing_texture3D);
grassSetAntiAliasingTexture2D5(antialiasing_texture2D5);
grassSetGraineTexture2D5(graine_texture2D5);
grassSetTextureOnGrass(texture3D);
grassSetFileTextureOnGrass(file_texture3D);
grassSetSizeTexture2D5(sizex_texture2D5,sizey_texture2D5);
grassInit(area,
	  patches_number[0],patches_number[1],
	  density_map);
return true;
}

bool grassLoadXML(xmlNodePtr p_nodePrairie)
{
xmlNodePtr child = p_nodePrairie->children;
while (child != NULL) 
   {
   if (child->type == XML_ELEMENT_NODE)
      {   
      std::string name = (const char*) child->name;
      if (name == "grass")
         {
	 return (grassLoadNode(child));	 
	 }
      }
   child = child->next;
   }
return false;
}

bool grassLoadXML(const char* filename)
{
xmlNodePtr p_node;
xmlDocPtr  p_doc;

//initialisation
if (filename==NULL) return false;
p_doc = xmlParseFile(filename);
if (p_doc == NULL) return false;
p_node = xmlDocGetRootElement(p_doc);

//validation
xmlValidCtxt cvp; 
cvp.userData = stderr;
cvp.error = (xmlValidityErrorFunc) fprintf;
cvp.warning = (xmlValidityWarningFunc) fprintf;  
if (!xmlValidateDocument(&cvp, p_doc)) return false;

xmlNodePtr child = p_node->children;
while (child != NULL) 
   {
   if (child->type == XML_ELEMENT_NODE)
      {   
      std::string name = (const char*) child->name;
      if (name == "prairie")
	 grassLoadXML(child);
      }
   child = child->next;
   }
return true;
}

void grassSaveXML(xmlNodePtr p_nodePrairie)
{
int tmp, tmp2;
char value[256];
xmlNodePtr p_nodeGrass = xmlNewNode (NULL, (const xmlChar*)"grass");

//area
sprintf(value,"%f %f %f %f", grassGetAreaXMin(), grassGetAreaYMin(),
			     grassGetAreaXMax(), grassGetAreaYMax());
xmlNewProp(p_nodeGrass,(const xmlChar*)"area",(const xmlChar*)value);

//grass_height
sprintf(value,"%f", grassGetHeight());
xmlNewProp(p_nodeGrass,(const xmlChar*)"grass_height",(const xmlChar*)value);

//grass_width
sprintf(value,"%f", grassGetWidth());
xmlNewProp(p_nodeGrass,(const xmlChar*)"grass_width",(const xmlChar*)value);

//noise
sprintf(value,"%f", grassGetNoisePositions());
xmlNewProp(p_nodeGrass,(const xmlChar*)"noise_positions",(const xmlChar*)value);
sprintf(value,"%f", grassGetNoiseNormals());
xmlNewProp(p_nodeGrass,(const xmlChar*)"noise_normals",(const xmlChar*)value);

//patches_number
sprintf(value,"%d %d", grassGetNumberPatchesInLength(), grassGetNumberPatchesInWidth());
xmlNewProp(p_nodeGrass,(const xmlChar*)"patches_number",(const xmlChar*)value);

//lod_3D
sprintf(value,"%d %f", grassGetPG3DGrass(), grassGetDistance3D());
xmlNewProp(p_nodeGrass,(const xmlChar*)"lod_3D",(const xmlChar*)value);

//lod_2D5
sprintf(value,"%d %f", grassGetPG2D5Grass(), grassGetDistance2D5());
xmlNewProp(p_nodeGrass,(const xmlChar*)"lod_2D5",(const xmlChar*)value);

//lod_2D
sprintf(value,"%d %f", 1, grassGetDistance2D());
xmlNewProp(p_nodeGrass,(const xmlChar*)"lod_2D",(const xmlChar*)value);

//density_map
sprintf(value,"%s", grassGetFileDensityMap());
xmlNewProp(p_nodeGrass,(const xmlChar*)"density_map",(const xmlChar*)value);

//density_squaremeters
sprintf(value,"%d", grassGetNumberPerSquareMeters());
xmlNewProp(p_nodeGrass,(const xmlChar*)"number_squaremeters",(const xmlChar*)value);

//crushing_maxtime
sprintf(value,"%f", grassGetCrushingMaxTime());
xmlNewProp(p_nodeGrass,(const xmlChar*)"crushing_maxtime",(const xmlChar*)value);

//crushing_damping
sprintf(value,"%f", grassGetCrushingDamping());
xmlNewProp(p_nodeGrass,(const xmlChar*)"crushing_damping",(const xmlChar*)value);

//ajustment_height_2D5
if (grassGetAjustmentHeight2D5())
xmlNewProp(p_nodeGrass,(const xmlChar*)"ajustment_height_2D5",(const xmlChar*)"true");
else
xmlNewProp(p_nodeGrass,(const xmlChar*)"ajustment_height_2D5",(const xmlChar*)"false");

//texture
if (grassIsTextureOnGrass())
xmlNewProp(p_nodeGrass,(const xmlChar*)"texture3D",(const xmlChar*)"true");
else
xmlNewProp(p_nodeGrass,(const xmlChar*)"texture3D",(const xmlChar*)"false");
sprintf(value,"%s", grassGetFileTextureOnGrass());
xmlNewProp(p_nodeGrass,(const xmlChar*)"file_texture3D",(const xmlChar*)value);
//aliasing texture
sprintf(value,"%f", grassGetAntiAliasingTextureOnGrass());
xmlNewProp(p_nodeGrass,(const xmlChar*)"antialiasing_texture3D",(const xmlChar*)value);
//aliasing texture
sprintf(value,"%f", grassGetAntiAliasingTexture2D5());
xmlNewProp(p_nodeGrass,(const xmlChar*)"antialiasing_texture2D5",(const xmlChar*)value);
//graine texture
sprintf(value,"%d", grassGetGraineTexture2D5());
xmlNewProp(p_nodeGrass,(const xmlChar*)"graine_texture2D5",(const xmlChar*)value);
//size
grassGetSizeTexture2D5(tmp,tmp2);
sprintf(value,"%d", tmp);
xmlNewProp(p_nodeGrass,(const xmlChar*)"sizex_texture2D5",(const xmlChar*)value);
sprintf(value,"%d", tmp2);
xmlNewProp(p_nodeGrass,(const xmlChar*)"sizey_texture2D5",(const xmlChar*)value);

xmlAddChild(p_nodePrairie,p_nodeGrass);
}


