
#include "Herbe.hpp"
#include "GrassDefines.h"
#include "GrassRandom.h"

GReal Herbe::ms_height = 1.5f;
GReal Herbe::ms_width = 0.04f;

GReal Herbe::m_noisePositions=0.0f;
GReal Herbe::m_noiseNormals=0.0f;

Herbe::Herbe(const Herbe& h)
		{
		_pos0.x=h._pos0.x;
		_pos0.y=h._pos0.y;
		_pos0.z=h._pos0.z;
		_pos1.x=h._pos1.x;
		_pos1.y=h._pos1.y;
		_pos1.z=h._pos1.z;
		
		_tan0.x=h._tan0.x;
		_tan0.y=h._tan0.y;
		_tan0.z=h._tan0.z;
		_tan1.x=h._tan1.x;
		_tan1.y=h._tan1.y;
		_tan1.z=h._tan1.z;
		
		_norm0z=h._norm0z;
		_norm1z=h._norm1z;
		}	

void Herbe::Draw(const short& NbPolygones,
		 const Vec3 &orientation,
		 const GReal& coefWidth) const
		{	
		Vec3 billboard(0.0f,0.0f,0.0f);
		Vec3 axe(0.0f,0.0f,0.0f);
		Vec3 axeBillboard;
		GReal x, y, z, pas, compteur_pas;
		GReal demims_width_max,demims_width_min,demims_width;
		GReal type_texture=GRand();
		
		//initialisation compteur
		compteur_pas= 0.0f;
		pas=1.0f/NbPolygones;

		//initialisation epaisseur	
		GRand(demims_width_max);
		demims_width_max*=0.02f;
		demims_width_max+=ms_width;
		GRand(demims_width_min);
		demims_width_min*=0.01f;
		demims_width_min*=coefWidth;
		demims_width_max*=coefWidth;
		demims_width=demims_width_max+demims_width_min;


		//calcul du billboard			
		billboard.x=orientation.x*demims_width;
		billboard.y=orientation.y*demims_width;
		billboard.z=0.0;	

		//premier calcule -> x,y,z sont connus
		x = _pos0.x;y = _pos0.y;z = _pos0.z;

		//debut affichage	
		glBegin(GL_QUAD_STRIP);
		
		glNormal3f(0.0f,0.0f,_norm0z);
		glTexCoord2f(0.0f,type_texture);
		glVertex3f(x-billboard.x, y-billboard.y, z);
		glTexCoord2f(1.0f,type_texture);
		glVertex3f(x+billboard.x, y+billboard.y, z);
		for (int i=1;i<=NbPolygones;i++)
			{			 				
			compteur_pas = i*pas;				
			//sauvegarde ancienne valeur
			axe.x=-x;axe.y=-y;axe.z=-z;
			//calcul du nouveau point	
			x = Hermite(compteur_pas,_pos0.x,_tan0.x,_pos1.x,_tan1.x);
			y = Hermite(compteur_pas,_pos0.y,_tan0.y,_pos1.y,_tan1.y);
			z = Hermite(compteur_pas,_pos0.z,_tan0.z,_pos1.z,_tan1.z);	
			//calcul nouvel epaisseur
			demims_width = (1.0f-compteur_pas*compteur_pas)*demims_width_max+demims_width_min; 
			//calcul nouveau vecteur normal -> que a partir de extremite herbe -> vitesse
			glNormal3f(0.0f, 0.0f,compteur_pas * _norm1z + (1.0f - compteur_pas) * _norm0z);	
			//calcul vecteur axe herbe
			axe.x+=x;axe.y+=y;axe.z+=z;	
			//calcul vecteur billboard
			billboard.x=orientation.x*demims_width;
			billboard.y=orientation.y*demims_width;
			billboard.z=0.0f;

			//detection simple -> si la courbure est superieur a 45 degrees
			if (Abs(axe.x) > axe.z || Abs(axe.y) >axe.z)
				{
				//modifiaction du billboard pour obtenir un meilleur rendu de la courbure	
				//projection axe herbe sur billboard
				axeBillboard = billboard*(axe*billboard)/(demims_width*demims_width);
				//calcul axe du nouveau vecteur billboard
				if (axe*billboard>0.0f)
				billboard.z-=Abs(axeBillboard.x+axeBillboard.y);
				else
				billboard.z+=Abs(axeBillboard.x+axeBillboard.y);	
				//le billboard prend une norme = demi-epaisseur
				billboard.normalize();
				billboard *= demims_width;	
				}
			glTexCoord2f(0.0f,type_texture);
			glVertex3f(x - billboard.x,
				   y - billboard.y,
				   z - billboard.z);
			glTexCoord2f(1.0f,type_texture);
			glVertex3f(x + billboard.x,
				   y + billboard.y,
				   z + billboard.z);
			}
		glEnd();	
		}
		

