
#ifndef _information_primitive_
#define _information_primitive_

//include base prairie
#include "GrassDefines.h"
#include "GeometrieBase.hpp"

//include io
#include <stdio.h>
#include <stdlib.h>


/** \class InformationVent
\brief stockage des informations de vent
(==wind_info)


*/

class InformationVent
{
public:
	///la puissance du vent
	GReal _power;
	///le vecteur de vent
	Vec2 _vecteur;

public:
	/// constructeur par defaut
	/// le vecteur vent est nulle
	InformationVent()
		{
//		printf("constructeur informationVent...");
		_power=0;
		_vecteur[0]=0.0;
		_vecteur[1]=0.0;
		}
		
	/// constructeur
	InformationVent(GReal p,GReal v1,GReal v2)
		{
//		printf("constructeur informationVent...");
		_power=p;
		_vecteur[0]=v1;
		_vecteur[1]=v2;
		}
	
	/// destructeur	
	~InformationVent(){}
	
	/// constructeur par recopie
	InformationVent(const InformationVent &iv )
		{
//		printf("constructeur recopie informationVent...");
		_power=iv._power;
		_vecteur[0]=iv._vecteur[0];
		_vecteur[1]=iv._vecteur[1];
		}
	
	///initialisation
	void Initialise()
		{
		_power=0;
		_vecteur[0]=0.0;
		_vecteur[1]=0.0;
		}
	
	/// surdefinition de l'affectation
	InformationVent& operator=(const InformationVent& iv)
		{
//		printf("affectation informationVent...");
		_power=iv._power;
		_vecteur.x=iv._vecteur.x;
		_vecteur.y=iv._vecteur.y;	
		return *this;
		}
	
	///permet d'ajouter deux InformationVent
	void operator+=(const InformationVent& iv)
		{
		_power+=iv._power;
		_vecteur.x+=iv._vecteur.x;
		_vecteur.y+=iv._vecteur.y;	
		}
	
	void operator*=(const GReal& v)
		{
		_power*=v;
		_vecteur.x*=v;
		_vecteur.y*=v;	
		}
		
	///normalisation des vecteurs 
	void normalize()
		{
		static const GReal seuil_att=30.0;
		static const GReal seuil_moy=1.0;
		
		if (_power>seuil_moy)
		{
			_vecteur[0]/=_power;
			_vecteur[1]/=_power;	
		}
		else
		{
			_vecteur[0]=0.0;
			_vecteur[1]=0.0;	
		}
		
		if (_power<seuil_att)
		{
			const GReal tmp=_power/seuil_att;
			// attenuation
			_vecteur[0]*=tmp;
			_vecteur[1]*=tmp;
		}
	
		}
		
	///permet d'ajouter deux InformationVent
	friend InformationVent operator+(const InformationVent& iv1,
									 const InformationVent& iv2)
		{
//		printf("ajout de deux informationVent...");
		InformationVent resultat;
		resultat._vecteur[0]=iv1._vecteur[0]+iv2._vecteur[0];
		resultat._vecteur[1]=iv1._vecteur[1]+iv2._vecteur[1];	
		return resultat;
		}
		
};

///l'information que les primitives de vent manipulent
typedef InformationVent wind_info;



/** \class InformationTrajectoire
\brief information elementaire d'une trajectoire 

*/
class InformationTrajectoire
{
public:
	///position information
	Point3 _position;
	///vecteur colineaire trjectoire
	Vec2 _vecteur;
	///vecteur normal trajectoire
	Vec2 _vecteurN;
	///temps de vie
	float _time;
	///
	bool _linkPreceding;

public:
	///affichage des donnees
	void Affiche() const
		{
		printf("InformationTrajectoire :: point %f,%f\n",
					_position[0],_position[1]);
		}

};



/** \class InformationEcrasement
\brief contient les informations envoyees par les primitives d'ecrasement

les primitives d'ecrasement modifient directement cette classe.
permet de definir le vecteur d'ecrasement (en 2D)

remis a jour avant chaque affichage d'une herbe
*/
class InformationEcrasement
{
public:
	///puissance d'ecrasement
	GReal _power;
	///le vecteur d'ecrasement
	Vec2 _vecteur;

public:
	/// constructeur par defaut
	/// le vecteur ecrasement est nulle
	InformationEcrasement()
		{
//		printf("constructeur informationVent...");
		_vecteur[0]=0.0;
		_vecteur[1]=0.0;
		_power=0.0;
		}
		
	/// constructeur par defaut
	/// le vecteur ecrasement est nulle
	InformationEcrasement(GReal x,GReal y,GReal p)
		{
		_vecteur[0]=x;
		_vecteur[1]=y;
		_power=p;
		}
	
	/// constructeur par recopie
	InformationEcrasement(const InformationEcrasement &iv )
		{
		_vecteur[0]=iv._vecteur[0];
		_vecteur[1]=iv._vecteur[1];
		_power=iv._power;
		}
	
	/// destructeur	
	~InformationEcrasement(){}
	
	
	/// surdefinition de l'affectation
	InformationEcrasement& operator=(const InformationEcrasement& iv)
		{
		_vecteur[0]=iv._vecteur[0];
		_vecteur[1]=iv._vecteur[1];	
		_power=iv._power;
		return *this;
		}
	
	///permet d'ajouter deux wind_info
	void operator+=(const InformationEcrasement iv)
		{
		_power+=iv._power;
		_vecteur[0]+=iv._power*iv._vecteur[0];
		_vecteur[1]+=iv._power*iv._vecteur[1];	
		}

	///permet d'ajouter deux wind_info
	void normalize()
		{		
		static const GReal seuil_moy=1.0;
		
		if (_power>seuil_moy)
		{
			_vecteur[0]/=_power;
			_vecteur[1]/=_power;	
		}
		
	
		
		}			
	
	
	//initialisation
	void Initialise()
		{
		_vecteur[0]=0.0;
		_vecteur[1]=0.0;
		_power=0.0;
		}

		
	///affichage des donnees
	void Affiche() const
		{
		printf("InformationEcrasement :: vecteur %f,%f ; power : %f\n",
					_vecteur[0],_vecteur[1],_power);
		}

};


#endif /* _information_primitive_ */
