#include "TextureGrass.h"

#include "GrassDefines.h"
#include "GeometrieBase.hpp"
#include "Herbe.hpp"

#include <stdio.h>
#include <string.h>
#include <math.h>

TextureGrass::TextureGrass()
	{
	_map=NULL;
	_cols=0;
	_rows=0;
	_textureId=0;
	_alphaChannel=false;
	}
	
TextureGrass::~TextureGrass()
	{
	if (_map!=NULL) delete _map;
	if (_textureId!=0) glDeleteTextures(1,&_textureId);
	}


bool TextureGrass::loadMap(const char * filename)
        {
	const char *pStr;
	if (_map!=NULL) {delete _map;_map=NULL;}
	pStr=filename;
	while (pStr[0]!=0) pStr++;
        if (!strcmp(pStr-4,".ppm")) make_with_PPM(filename);
        if (!strcmp(pStr-4,".bmp")) make_with_BMP(filename);
	if (!strcmp(pStr-4,".tga")) make_with_TGA(filename);
	if (_map==NULL) printf("Warning:Texture: unable to load map %s;\n",filename);
	return (_map!=NULL);
	}

bool TextureGrass::addAlphaChannel(GLubyte defaultValue)
	{
	if (_map==NULL)
		{	
		printf("Warning:Texture: unable to add alpha channel : no map;\n");
		return false;
		}
	int i;
	int j;
	if (!_alphaChannel)
		{
		GLubyte* newMap=new GLubyte[_cols*_rows*4];
		j=0;
		for (i=0;i<_cols*_rows*4;i+=4)
			{
			j+=3;
			newMap[i]=_map[j];
			newMap[i+1]=_map[j+1];
			newMap[i+2]=_map[j+2];
			}
		delete _map;
		_map=newMap;
		_alphaChannel=true;
		}
	for (i=0;i<_cols*_rows*4;i+=4)
		{
		_map[i+3]=defaultValue;
		}
	return true;
	}

bool TextureGrass::decreaseOpacityOnVBorder(float coef)
	{
	if (_map==NULL)
		{	
		printf("Warning:Texture: unable to decrease opacity : map==NULL;\n");
		return false;
		}
	if (coef<=0.0f || coef >1.0f)
		{	
		printf("Warning:Texture: unable to decrease opacity : coef not in ]0,1];\n");
		return false;
		}
	if (!_alphaChannel) 
		{	
		printf("Warning:Texture: unable to decrease opacity : no alpha channel;\n");
		return false;
		}
	int colend = int((_cols/2)*coef);
	int colbeg = _cols-colend-1;
	for (int i=0;i<_cols*_rows*4;i+=4)
	  {
	  int col=((i/4)%_cols);
	  if (col < colend)
		_map[i+3]=GLubyte(pow((float(col)/colend),1.0f)*_map[i+3]);
	  if (col > colbeg)
		_map[i+3]=GLubyte(pow((1.0f-float(col-colbeg)/colend),1.0f)*_map[i+3]);
	  }
	return true;
	}



bool TextureGrass::filteringChannelOnMap(short channel, short coefLastValue)
	{
	//channel (R:0,G:1,B:2,A:3)
	if (_map==NULL)
		{	
		printf("Warning:Texture: unable to filtering channel :no map;\n");
		return false;
		}
	if (channel < 0 || channel >3)
		{	
		printf("Warning:Texture: unable to filtering channel :channel not in [0,3];\n");
		return false;
		}
	if (!_alphaChannel && channel == 3)
		{	
		printf("Warning:Texture: unable to filtering channel 3: no alpha channel;\n");
		return false;
		}
	if (coefLastValue<1)
		{	
		printf("Warning:Texture: unable to filtering channel : coefLastValue<1;\n");
		return false;
		}
	
	int iteration;
	int indice, indiceUp, indiceDown;
	int compteur;short value;
	if (_alphaChannel) iteration=4; else iteration=3;
	GLubyte* newMap=new GLubyte[_cols*_rows*iteration];
	memcpy(newMap, _map, sizeof(GLubyte)*_cols*_rows*iteration);
	
	for (indice=0; indice<_cols*_rows*iteration; indice+=iteration)
		{
		indiceUp = indice - _cols*iteration;
		indiceDown = indice + _cols*iteration;
		
		value=_map[indice+channel];
		compteur=coefLastValue;
		value*=compteur;		
	
		if (indice+iteration<_cols*_rows*iteration) {value+=_map[indice+iteration+channel];compteur++;}
		if (indice-iteration>0) {value+=_map[indice-iteration+channel];compteur++;}
		if (indiceDown<_cols*_rows*iteration) {value+=_map[indiceDown+channel];compteur++;}
		if (indiceDown+iteration<_cols*_rows*iteration){value+=_map[indiceDown+iteration+channel];compteur++;}
		if (indiceDown-iteration<_cols*_rows*iteration) {value+=_map[indiceDown-iteration+channel];compteur++;}
		if (indiceUp>0) {value+=_map[indiceUp+channel];compteur++;}
		if (indiceUp+iteration>0) {value+=_map[indiceUp+iteration+channel];compteur++;}
		if (indiceUp-iteration>0) {value+=_map[indiceUp-iteration+channel];compteur++;}
		
		value/=compteur;
		newMap[indice+channel]=(GLubyte)value;
		}
	delete _map;
	_map=newMap;
	return true;
	}

bool TextureGrass::extendColorOnMap(short nbpixels)
	{
	if (_map==NULL)
		{	
		printf("Warning:Texture: unable to extend color :no map;\n");
		return false;
		}
	
	int i;
	int iteration;
	int indice, indiceUp, indiceDown;
	if (_alphaChannel) iteration=4; else iteration=3;
	GLubyte* newMap;
	
	for (;nbpixels>0;nbpixels--)
	{
	newMap=new GLubyte[_cols*_rows*iteration];
	memcpy(newMap, _map, sizeof(GLubyte)*_cols*_rows*iteration);
	for (indice=0; indice<_cols*_rows*iteration; indice+=iteration)
		{
		if (_map[indice]+_map[indice+1]+_map[indice+2]>0) continue; 
		indiceUp = indice - _cols*iteration;
		indiceDown = indice + _cols*iteration;	
		for (i=0;i<3;i++)
		   {
		   //on prends le max des valeurs voisines
		   if (indice+iteration<_cols*_rows*iteration) 
		      {newMap[indice+i]=Max(newMap[indice+i],_map[indice+i+iteration]);}
		   if (indice-iteration>0)
		      {newMap[indice+i]=Max(newMap[indice+i],_map[indice+i-iteration]);}
		   if (indiceDown<_cols*_rows*iteration) 
		      {newMap[indice+i]=Max(newMap[indice+i],_map[indiceDown+i]);}
		   if (indiceDown+iteration<_cols*_rows*iteration)
		      {newMap[indice+i]=Max(newMap[indice+i],_map[indiceDown+i+iteration]);}
		   if (indiceDown-iteration<_cols*_rows*iteration)
		      {newMap[indice+i]=Max(newMap[indice+i],_map[indiceDown+i-iteration]);}
		   if (indiceUp>0)
		      {newMap[indice+i]=Max(newMap[indice+i],_map[indiceUp+i]);}
		   if (indiceUp+iteration>0)
		      {newMap[indice+i]=Max(newMap[indice+i],_map[indiceUp+i+iteration]);}
		   if (indiceUp-iteration>0)
		      {newMap[indice+i]=Max(newMap[indice+i],_map[indiceUp+i-iteration]);}
		   }
		}
	delete _map;
	_map=newMap;
	}
	return true;
	}

bool TextureGrass::concatMap(const GLubyte* map, 
			     int rows, int cols, bool alphaChannel)
	{
	int i;
	int iteration;
	int lastIndiceMap;
	int indiceMap;
	if (map==NULL)
	if (_map==NULL)
		{	
		printf("Warning:Texture: unable to concat map : no map;\n");
		return false;
		}
	if (_map==NULL)
		{
		_rows=rows;
		_cols=cols;
		_alphaChannel=alphaChannel;
		if (_alphaChannel) iteration=4; else iteration=3;
		_map=new GLubyte[_cols*_rows*iteration];
		for (i=0;i<_cols*_rows*iteration;i++)
			_map[i]=map[i];
		return true;
		}
	if (_rows!=rows)
		{	
		printf("Warning:Texture: unable to concat map : rows!=rows;\n");
		return false;
		}
	if (_alphaChannel!=alphaChannel)
		{	
		printf("Warning:Texture: unable to concat map : channel!=channel;\n");
		return false;
		}
	if (_alphaChannel) iteration=4; else iteration=3;
	GLubyte* newMap=new GLubyte[(_cols+cols)*_rows*iteration];
	//insertion _map...
	indiceMap=0;
	lastIndiceMap=indiceMap;
	for (i=0; i<_cols*_rows*iteration; i+=iteration)
	     {
	     if (indiceMap>(_cols+cols)*_rows*iteration) continue;
	     newMap[indiceMap]   = _map[i];
	     newMap[indiceMap+1] = _map[i+1];
	     newMap[indiceMap+2] = _map[i+2];
	     if (_alphaChannel) newMap[indiceMap+3] = _map[i+3];
	     indiceMap+=iteration;
	     if (indiceMap-lastIndiceMap==_cols*iteration)
	     	{
		indiceMap+=cols*iteration;
		lastIndiceMap=indiceMap;
		}
	     }
	//insertion map
	indiceMap=_cols*iteration;
	lastIndiceMap=indiceMap;
	for (i=0; i<cols*rows*iteration; i+=iteration)
	     {
	     newMap[indiceMap]   = map[i];
	     newMap[indiceMap+1] = map[i+1];
	     newMap[indiceMap+2] = map[i+2];
	     if (_alphaChannel) newMap[indiceMap+3] = map[i+3];
	     indiceMap+=iteration;
	     if (indiceMap-lastIndiceMap==cols*iteration) 
	     	{
		indiceMap+=_cols*iteration;
		lastIndiceMap=indiceMap;
		}
	     }   
	delete _map;
	_map=newMap;
	_cols=_cols+cols;
	_rows=_rows;
	return true;
	}


bool TextureGrass::concatMap(const TextureGrass& textureGrass)
	{
	return concatMap(textureGrass._map,
			 textureGrass._rows,
			 textureGrass._cols,
			 textureGrass._alphaChannel);
	} 


void TextureGrass::glPushAttribAndMatrix()
	{
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	glOrtho(0.0,1.0,0.0,1.0,-1.0,1.0);
	
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
	
	glReadBuffer(GL_BACK);
	}

void TextureGrass::glPopAttribAndMatrix()
	{
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();

	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
	}

bool TextureGrass::loadMapGrassTex2D5(int size_x, int size_y, int nb_grasses,
				      GReal coef_GrassWidth)
	{
	int i,j;
	
	const short nb_densite=GRASS_NUMBER_PATCHE_TYPE;
	GLubyte *_listmap[nb_densite+1];
	
	int subTexSize=size_x/nb_densite;
	int subNbGrasses=Max(nb_grasses/nb_densite,1);
		
	//initialisation map	
	if (_map!=NULL) {delete _map;_map=NULL;}
	
	//push GL
	glPushAttribAndMatrix();
	//viewport
	glRotatef(-90,1,0,0);
	GLint lastViewPortMatrix[4];
	glGetIntegerv(GL_VIEWPORT,lastViewPortMatrix); 	
	glViewport(0, 0, subTexSize, size_y);
	//orientation
	GLfloat modelViewMatrix[16];
	glGetFloatv(GL_MODELVIEW_MATRIX, modelViewMatrix);
	Vec3 orientation(modelViewMatrix[0], modelViewMatrix[4], modelViewMatrix[8]);	
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	//affichage
	for (j=1; j<=nb_densite; j++)
	  {
	  for (i=0; i<subNbGrasses*j; i++)
		{
		Herbe herbe;
		herbe._pos0[0]=GReal(i)/(subNbGrasses*j)
			      +(0.2f+0.8f*GRandRealRandom())/(subNbGrasses*j);
		herbe._pos0[1]=0.0f;
		herbe._pos0[2]=0.0f;
		
		//fabrication herbe
		Vec2 informationEcrasement;
		informationEcrasement.x=0.0;
		informationEcrasement.y=0.0;
		InformationVent informationVent;
		informationVent._power=1.0f;
		informationVent._vecteur[0]=0.1f*GRandRealRandom()-0.05f;
		informationVent._vecteur[1]=0.0f;
		herbe.Make(informationVent,informationEcrasement,0.0f,0.8f+0.2f*GRandRealRandom());
		herbe.Draw(10, orientation, coef_GrassWidth);	
		} 
	  _listmap[j]=new GLubyte[subTexSize*size_y*4];
	  //glFlush();
	  glReadPixels(0, 0, subTexSize, size_y, GL_RGBA,GL_UNSIGNED_BYTE,_listmap[j]);
	  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	  }

	glPopAttribAndMatrix();
	glViewport(lastViewPortMatrix[0], lastViewPortMatrix[1],
		   lastViewPortMatrix[2], lastViewPortMatrix[3]);
	
	//construction de la map
	for (j=1; j<=nb_densite; j++)
	  {
	  if (!concatMap(_listmap[j],size_y,subTexSize,true))
		{
		printf("Error:Texture: erreur dans la construction de la texture 2D5\n");
		_map=NULL;_cols=0;_rows=0;
		return false;
		}
	  delete _listmap[j];
	  }
	
	//modification de la map
	//modif alpha : simple
	//le mipmapping fera le reste
	//TODO -> supprimer la texture de niveau 0 : jamais besoin.
	for (i=0; i<_cols*_rows*4; i+=4)
	  {
	  _map[i+3]=Min(255,(_map[i]+_map[i+1]+_map[i+2])*2);
	  }
	
	return (_map!=NULL);	
	}

	
bool TextureGrass::loadMapGrassTex2D(int size_x, int size_y,
		                     int nb_grasses, GReal coef_GrassWidth)
	{
	int i,j;
	
	const short nb_densite=GRASS_NUMBER_PATCHE_TYPE;
	GLubyte *_listmap[nb_densite+1];
	
	int subTexSize=size_x/nb_densite;
	int subNbGrasses=Max(nb_grasses/nb_densite,1);
		
	//initialisation map	
	if (_map!=NULL) {delete _map;_map=NULL;}
	
	//push GL
	glPushAttribAndMatrix();
	//viewport
	glRotatef(-45,1,0,0);
	GLint lastViewPortMatrix[4];
	glGetIntegerv(GL_VIEWPORT,lastViewPortMatrix); 	
	glViewport(0, 0, subTexSize, size_y);
	//orientation
	GLfloat modelViewMatrix[16];
	glGetFloatv(GL_MODELVIEW_MATRIX, modelViewMatrix);
	Vec3 orientation(modelViewMatrix[0], modelViewMatrix[4], modelViewMatrix[8]);	

	//affichage
	for (j=1; j<=nb_densite; j++)
	  {
	  for (i=0; i<subNbGrasses*j; i++)
		{
		Herbe herbe;
		herbe._pos0[0]=GReal(i)/(subNbGrasses*j)
			      +(0.2f+0.8f*GRandRealRandom())/(subNbGrasses*j);
		herbe._pos0[1]=0.0f;
		herbe._pos0[2]=1.0f*GRandRealRandom();
		
		//fabrication herbe
		Vec2 informationEcrasement;
		informationEcrasement.x=0.0;
		informationEcrasement.y=0.0;
		InformationVent informationVent;
		informationVent._power=1.0f;
		informationVent._vecteur[0]=GRandRealRandom()-0.5f;
		informationVent._vecteur[1]=0.0f;
		herbe.Make(informationVent,informationEcrasement,0.0f,1.2f+0.4f*GRandRealRandom());
		
		herbe.Draw(10, orientation, coef_GrassWidth);	
		} 
	  _listmap[j]=new GLubyte[subTexSize*size_y*4];
	  glFlush();
	  glReadPixels(0, 0, subTexSize, size_y, GL_RGBA,GL_UNSIGNED_BYTE,_listmap[j]);
	  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	  }

	glPopAttribAndMatrix();
	glViewport(lastViewPortMatrix[0], lastViewPortMatrix[1],
		   lastViewPortMatrix[2], lastViewPortMatrix[3]);
	
	//construction de la map
	for (j=1; j<=nb_densite; j++)
	  {
	  if (!concatMap(_listmap[j],size_y,subTexSize,true))
		{
		printf("Error:Texture: erreur dans la construction de la texture 2D\n");
		_map=NULL;_cols=0;_rows=0;
		return false;
		}
	  delete _listmap[j];
	  }
	
	//modification de la map
	//modif alpha : simple
	//le mipmapping fera le reste
	//TODO -> supprimer la texture de niveau 0 : jamais besoin.
	for (i=0; i<_cols*_rows*4; i+=4)
	 {
	  if (_map[i]+_map[i+1]+_map[i+2]==0) _map[i+3]=0;
	  else _map[i+3]=255;
	  }
	//modif color : pour eviter l'effet d'assombrissement de la texture
	//lors de l'utilisation des mipmaps : peut etre existe t-il une
	//technique connu mais je n'ai pas trouve...
	//TODO -> demander a Fabrice.
	extendColorOnMap(_cols/20);
	
	return (_map!=NULL);
	}

void TextureGrass::unloadMap()
	{
	if (_map!=NULL) {delete _map;_map=NULL;}
	}



void TextureGrass::loadGLTexture(GLint internalFormat, bool mipmap)
	{
	if (_map==NULL) return;
	if (_textureId!=0) {glDeleteTextures(1,&_textureId); _textureId=0;}
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
        glGenTextures(1, &_textureId);
        glBindTexture(GL_TEXTURE_2D,_textureId);
	
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER,GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER, GL_LINEAR);
        
	if (mipmap)
		{
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,
				GL_NEAREST_MIPMAP_NEAREST);
		if (_alphaChannel)
		gluBuild2DMipmaps(GL_TEXTURE_2D, internalFormat, _cols, _rows,
				  GL_RGBA, GL_UNSIGNED_BYTE, _map);
		else
		gluBuild2DMipmaps(GL_TEXTURE_2D, internalFormat, _cols, _rows,
				  GL_RGB, GL_UNSIGNED_BYTE, _map);
		}
	else
		{
		if (_alphaChannel)
		glTexImage2D(GL_TEXTURE_2D, 0, internalFormat, _cols, _rows,
		     		  0, GL_RGBA, GL_UNSIGNED_BYTE, _map);
		else
        	glTexImage2D(GL_TEXTURE_2D, 0, internalFormat, _cols, _rows,
		     		  0, GL_RGB, GL_UNSIGNED_BYTE, _map);
		}
	}

/*bool TextureGrass::loadGLTextureWithHorizontalMipMaps(GLint internalFormat)
	{
	if (_map==NULL) return false;
	if (!_alphaChannel) return false;
	if (_cols<4) return false;
	if (_textureId!=0) {glDeleteTextures(1,&_textureId); _textureId=0;}
	
	
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
        glGenTextures(1, &_textureId);
        glBindTexture(GL_TEXTURE_2D,_textureId);
	
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	
	GLubyte *listmap[10];
	int cols=_cols;
	int nb_mipmaps=0;
	int i;
	while (cols!=4)
		{
		cols/=2;	
		listmap[nb_mipmaps]=new GLubyte[_rows*cols*4];
		if (nb_mipmaps==0)
			{
			for (i=0;i<_rows*cols*4;i++)
				{
				listmap[0][i]=_map[i*2]/2
					     +_map[i*2+4]/2;;
					
				}
			}
		else
			{
			for (i=0;i<_rows*cols*4;i++)
				{
				listmap[nb_mipmaps][i]=
					listmap[nb_mipmaps-1][i*2]/2
				       +listmap[nb_mipmaps-1][i*2+4]/2;
					
				}
			}
		nb_mipmaps++;
		}
	glTexImage2D(GL_TEXTURE_2D, 0, internalFormat, _cols, _rows,
		     0, GL_RGBA, GL_UNSIGNED_BYTE, _map);	
	int current_mipmap=1;
	cols=_cols;
	for (i=1;i<=nb_mipmaps;i++)
		{
		cols/=2;	
		glTexImage2D(GL_TEXTURE_2D, i, internalFormat, cols, _rows,
		     	     0, GL_RGBA, GL_UNSIGNED_BYTE, listmap[i-1]);	
		}	
	for (i=0;i<nb_mipmaps;i++) delete listmap[i];	
	return true;
	}*/

void TextureGrass::unloadGLTexture()
	{
	if (_textureId!=0) {glDeleteTextures(1,&_textureId); _textureId=0;}
	}






void TextureGrass::make_with_PPM(const char *filename)
{
FILE* ifp;
int ich;
int i;
        
ifp = fopen(filename, "r");
        
/* Lecture du Magic number */
getc( ifp );
ich = getc( ifp );
if(ich != '6') {fclose(ifp);return;}
    
/* Lecture des dimensions */
_cols = pm_getint( ifp );
_rows = pm_getint( ifp );

/* Allocation memoire  */
if (_cols+_rows==0) {fclose(ifp);return;}
_map=new GLubyte[_cols * _rows * 3];

for(i=0; i < _cols * _rows * 3; i+=3)
                {
         	_map[i+2] = pm_getrawbyte(ifp);
                 _map[i  ] = pm_getrawbyte(ifp);
                 _map[i+1] = pm_getrawbyte(ifp);        
                }
_alphaChannel=false;
fclose(ifp);
}

void TextureGrass::make_with_BMP(const char *File)
{               
FILE                    *fichier;
unsigned char   Header[0x36];
GLuint                  DataPos,DataSize;

fichier = fopen(File,"rb");if (!fichier) return;
if (fread(Header,1,0x36,fichier)!=0x36) {fclose(fichier);return;}
if (Header[0]!='B' || Header[1]!='M')   {fclose(fichier);return;}
if ((*(int*)&(Header[0x1E]))!=0)                {fclose(fichier);return;}
if ((*(int*)&(Header[0x1C]))!=24)               {fclose(fichier);return;}

DataPos                 = (*(int*)&(Header[0x0A]));
DataSize                = (*(int*)&(Header[0x22]));
_cols                   = (*(int*)&(Header[0x12]));
_rows                   = (*(int*)&(Header[0x16]));     

//!!!!
if (DataSize==0) DataSize=_rows*_cols*3;
if (DataPos==0)  DataPos=0x36;

fseek(fichier,DataPos,0);
_map = new GLubyte[DataSize];
if (!_map) {fclose(fichier);return;}

if (fread(_map,1,DataSize,fichier)!=DataSize) 
{
        delete _map;
        fclose(fichier);
        return;
}

fclose(fichier);

unsigned char t;
for (int x=0;x<_rows*_cols;x++) 
{
        t=_map[x*3];
        _map[x*3]=_map[x*3+2];
        _map[x*3+2]=t;
} 

_alphaChannel=false;
}


void TextureGrass::make_with_TGA(const char* File)
{
unsigned char type[4];
unsigned char info[7];
int imageWidth, imageHeight;
int imageBits, size;
FILE *s;
if (!(s = fopen (File, "rb"))) return;
fread (&type, sizeof (char), 3, s); // read in colormap info and image type, byte 0 ignored
fseek (s, 12, SEEK_SET);			// seek past the header and useless info
fread (&info, sizeof (char), 6, s);
if (type[1] != 0 || (type[2] != 2 && type[2] != 3)) {fclose (s);return;}
imageWidth = info[0] + info[1]*256; 
imageHeight = info[2] + info[3]*256;
imageBits =info[4]; 
size = imageWidth * imageHeight;
_cols=imageWidth;
_rows=imageHeight;
if (imageBits == 32)
	{
	GLubyte temp;
	_map= new GLubyte[size*4];
	fread (_map, sizeof (GLubyte), size * 4, s); 
	for (int i = 0; i < size * 4; i += 4 )
    		{
        	temp = _map[i];
        	_map[i] = _map[i + 2];
        	_map[i + 2] = temp;
    		}
	_alphaChannel=true;
	}
else if (imageBits == 24)
	{
	GLubyte temp;
	_map= new GLubyte[size*3];
	fread (_map, sizeof (GLubyte), size * 3, s); 
	for (int i = 0; i < size * 3; i += 3)
    		{
       		temp = _map[i];
        	_map[i] = _map[i + 2];
        	_map[i + 2] = temp;
    		}
	_alphaChannel=false;
	}
else {fclose (s);return;}
}


//lecture fichier, renvoi un char
char TextureGrass::pm_getc(FILE* file)
    {
    register int ich;
    register char ch;
 
    ich = getc( file );
    if ( ich == EOF )
        printf("read error" );
    ch = (char) ich;
    
    if ( ch == '#' )
        {
        do
            {
            ich = getc( file );
            if ( ich == EOF )
                printf("read error" );
            ch = (char) ich;
            }
        while ( ch != '\n' && ch != '\r' );
        }
 
    return ch;
    }

//lecture fichier ASCII, renvoi un int
int TextureGrass::pm_getint( FILE* file)
    {
    register char ch;
    register int i;
 
    do
        {
        ch = pm_getc( file );
        }
    while ( ch == ' ' || ch == '\t' || ch == '\n' || ch == '\r' );
 
    if ( ch < '0' || ch > '9' )
        printf( "read error" );
 
    i = 0;
    do
        {
        i = i * 10 + ch - '0';
        ch = pm_getc( file );
        }
    while ( ch >= '0' && ch <= '9' );
 
    return i;
    }

//lecture fichier non ASCII, renvoi un int      
int TextureGrass::pm_getrawbyte(FILE* file)
    {
    register int iby;
 
    iby = getc( file );
    if ( iby == EOF )
        printf("read error" );
    return (int) iby;
    }
