
#ifndef _vitre_
#define _vitre_

#include <GL/gl.h>
#include <GL/glu.h>
#include <stdio.h>

#include "GrassDefines.h"
#include "GrassRandom.h"
#include "Herbe.hpp"
#include "GeometrieBase.hpp"

/** \class Vitre
\brief permet uniquement d'afficher une texture representant un groupe
d'herbe 

la position et la forme de la texture sont deduites des deux herbes 

*/

class Vitre
{
public:
	//constructeur vide	
	Vitre() {}
	 
	 //destructeur
	~Vitre() {}
	
	///construction en TRIANGLES - texture non deforme, herbe directeur _h0 
	inline int draw(const Herbe& _h0,
		        const Herbe& _h1,
		        const short& nbPolygones,
			const GReal coordTex0,
			const GReal coordTex1) const;
	
	inline void draw(const Point3& position0,
			 const Point3& position1,
			 const Herbe& grassCourbureInPosition0,
			 const GReal& norm00,
			 const GReal& norm01,
			 const GReal& norm10,
			 const GReal& norm11,
		         const short& nbPolygones,
			 const GReal coordTex0,
			 const GReal coordTex1) const;	 	  
};


int Vitre::draw(const Herbe& _h0,
		const Herbe& _h1,
		const short& nbPolygones,
		const GReal coordTex0,
		const GReal coordTex1 ) const
{	
GReal pas, compteur, compteur_pas;
GReal pos0[3], pos1[3];	
int nb_polygons=0;

//base de la vitre
pas=1.0f/nbPolygones;
compteur=1;
compteur_pas = pas;

glBegin(GL_QUAD_STRIP);
glNormal3f(0.0f,0.0f,_h0._norm0z);
glTexCoord2f(coordTex0 , compteur_pas - pas);
glVertex3f(_h0._pos0.x, _h0._pos0.y, _h0._pos0.z);
glNormal3f(0.0f,0.0f,_h1._norm0z);
glTexCoord2f(coordTex1 , compteur_pas - pas);
glVertex3f(_h1._pos0.x, _h1._pos0.y, _h1._pos0.z);
while(compteur<=nbPolygones)
	{
	//TODO -> optimisation les herbes ont la meme forme...
	//1 calcul necessaire...
	pos0[0] = Hermite(compteur_pas,
			  _h0._pos0.x,_h0._tan0.x,
			  _h0._pos1.x,_h0._tan1.x);
	pos0[1] = Hermite(compteur_pas,
			  _h0._pos0.y,_h0._tan0.y,
			  _h0._pos1.y,_h0._tan1.y);
	pos0[2] = Hermite(compteur_pas,
			  _h0._pos0.z,_h0._tan0.z,
			  _h0._pos1.z,_h0._tan1.z);
	pos1[0] = Hermite(compteur_pas,
			  _h1._pos0.x,_h1._tan0.x,
			  _h1._pos1.x,_h1._tan1.x);
	pos1[1] = Hermite(compteur_pas,
			  _h1._pos0.y,_h1._tan0.y,
			  _h1._pos1.y,_h1._tan1.y);
	pos1[2] = Hermite(compteur_pas,
			  _h1._pos0.z,_h1._tan0.z,
			  _h1._pos1.z,_h1._tan1.z);
	glNormal3f(0.0f,0.0f,compteur_pas*_h0._norm1z+(1.0f-compteur_pas)*_h0._norm0z);
	glTexCoord2f(coordTex0 , compteur_pas);
	glVertex3fv(pos0);
	glNormal3f(0.0f,0.0f,compteur_pas*_h1._norm1z+(1.0f - compteur_pas)*_h1._norm0z);
	glTexCoord2f(coordTex1 , compteur_pas);
	glVertex3fv(pos1);	
	compteur++;
	compteur_pas = pas * compteur;	
	nb_polygons+=2;		
	}
glEnd();
return nb_polygons;	
}

void Vitre::draw(const Point3& position0,
		 const Point3& position1,
		 const Herbe& grassCourbureInPosition0,
		 const GReal& norm00,
		 const GReal& norm01,
		 const GReal& norm10,
		 const GReal& norm11,
		 const short& nbPolygones,
		 const GReal coordTex0,
		 const GReal coordTex1) const
{	
GReal pas, compteur, compteur_pas;
GReal pos[3];	

//base de la vitre
pas=1.0f/nbPolygones;
compteur=1;
compteur_pas = pas;

glBegin(GL_QUAD_STRIP);
glNormal3f(0.0f,0.0f,norm00);
glTexCoord2f(coordTex0 , 0.0f);
glVertex3fv(&position0.x);
glNormal3f(0.0f,0.0f,norm10);
glTexCoord2f(coordTex1 , 0.0f);
glVertex3fv(&position1.x);
while(compteur<=nbPolygones)
	{
	pos[0] = Hermite(compteur_pas,
			  grassCourbureInPosition0._pos0.x,
			  grassCourbureInPosition0._tan0.x,
			  grassCourbureInPosition0._pos1.x,
			  grassCourbureInPosition0._tan1.x);
	pos[1] = Hermite(compteur_pas,
			  grassCourbureInPosition0._pos0.y,
			  grassCourbureInPosition0._tan0.y,
			  grassCourbureInPosition0._pos1.y,
			  grassCourbureInPosition0._tan1.y);
	pos[2] = Hermite(compteur_pas,
			  grassCourbureInPosition0._pos0.z,
			  grassCourbureInPosition0._tan0.z,
			  grassCourbureInPosition0._pos1.z,
			  grassCourbureInPosition0._tan1.z);
	glNormal3f(0.0f,0.0f,compteur_pas* norm01+(1.0f-compteur_pas)* norm00);
	glTexCoord2f(coordTex0 , compteur_pas);
	glVertex3fv(pos);
	
	pos[0] += position1.x - position0.x;
	pos[1] += position1.y - position0.y;
	pos[2] += position1.z - position0.z;	
	glNormal3f(0.0f,0.0f,compteur_pas* norm11+(1.0f-compteur_pas)* norm10);
	glTexCoord2f(coordTex1 , compteur_pas);
	glVertex3fv(pos);	
	compteur++;
	compteur_pas = pas * compteur;		
	}
glEnd();	
}
	


#endif /* _vitre_ */
