
/** \class VolumeVision
\brief 

permet de calculer le volume de vision en fonction de la matrice
de projection et de la matrice point de vue (OpenGL)
permet de savoir si un point appartient a ce volume

*/

#ifndef _VolumeVision_
#define _VolumeVision_

#include <GL/gl.h>
#include "GrassDefines.h"

class VolumeVision{

public:

VolumeVision(){}

/// Calcul les equations des 6 plans du volume visionne
/// en fonction de la matrice de projection 
/// et de la matrice point de vue OpenGL courante
inline void MettreAJour() 
	{ 
	glGetFloatv(GL_MODELVIEW_MATRIX,modelmat);
	glGetFloatv(GL_PROJECTION_MATRIX, projmat);
  	ConcatMatrice(ccmat, modelmat, projmat);
    
  	planEqs[0][0] = ccmat[3] - ccmat[0]; 
  	planEqs[0][1] = ccmat[7] - ccmat[4]; 
  	planEqs[0][2] = ccmat[11] - ccmat[8]; 
  	planEqs[0][3] = ccmat[15] - ccmat[12]; 

  	planEqs[1][0] = ccmat[3] + ccmat[0]; 
  	planEqs[1][1] = ccmat[7] + ccmat[4]; 
  	planEqs[1][2] = ccmat[11] + ccmat[8]; 
  	planEqs[1][3] = ccmat[15] + ccmat[12]; 

  	planEqs[2][0] = ccmat[3] + ccmat[1]; 
  	planEqs[2][1] = ccmat[7] + ccmat[5]; 
  	planEqs[2][2] = ccmat[11] + ccmat[9]; 
  	planEqs[2][3] = ccmat[15] + ccmat[13]; 

  	planEqs[3][0] = ccmat[3] - ccmat[1]; 
  	planEqs[3][1] = ccmat[7] - ccmat[5]; 
  	planEqs[3][2] = ccmat[11] - ccmat[9]; 
  	planEqs[3][3] = ccmat[15] - ccmat[13]; 

  	planEqs[4][0] = ccmat[3] + ccmat[2]; 
  	planEqs[4][1] = ccmat[7] + ccmat[6]; 
  	planEqs[4][2] = ccmat[11] + ccmat[10]; 
  	planEqs[4][3] = ccmat[15] + ccmat[14]; 

  	planEqs[5][0] = ccmat[3] - ccmat[2]; 
  	planEqs[5][1] = ccmat[7] - ccmat[6]; 
  	planEqs[5][2] = ccmat[11] - ccmat[10]; 
  	planEqs[5][3] = ccmat[15] - ccmat[14];
	}	

///vrai si le point (x,y,z) est dans le volume de vision
///ou si le point (x,y,z) est a une distance imprecision du volume de vision
inline GReal Distance(const GReal& x, const GReal& y, const GReal& z) const
	{
	GReal distance=0.0f;
	GReal calcul;
	for (short i=0;i<6;i++)
	  {
	  calcul = DistanceAuPlan(planEqs[i],x,y,z);
	  if (calcul<distance) distance=calcul;
	  }
	return (-distance);
	}

inline void getWorldCoordinatesOfCameraX(GReal& x, GReal& y, GReal& z)
	{
	x=modelmat[0]; 
	y=modelmat[4];
	z=modelmat[8];
	}

private:
  float ccmat[16],modelmat[16],projmat[16];
  float planEqs[6][4];
  
  
inline void ConcatMatrice(float *result, float *ma, float *mb)
	{
  	int i;
  	float mb00, mb01, mb02, mb03,
  	      mb10, mb11, mb12, mb13,
  	      mb20, mb21, mb22, mb23,
  	      mb30, mb31, mb32, mb33;
  	float mai0, mai1, mai2, mai3;

  	mb00 = mb[0];  mb01 = mb[1];
  	mb02 = mb[2];  mb03 = mb[3];
  	mb10 = mb[4];  mb11 = mb[5];
  	mb12 = mb[6];  mb13 = mb[7];
  	mb20 = mb[8];  mb21 = mb[9];
  	mb22 = mb[10];  mb23 = mb[11];
  	mb30 = mb[12];  mb31 = mb[13];
  	mb32 = mb[14];  mb33 = mb[15];

        for (i = 0; i < 4; i++) 
	  {
	  mai0 = ma[i*4+0];  mai1 = ma[i*4+1];
	  mai2 = ma[i*4+2];  mai3 = ma[i*4+3];

	  result[i*4+0] = mai0 * mb00 + mai1 * mb10 + mai2 * mb20 + mai3 * mb30;
	  result[i*4+1] = mai0 * mb01 + mai1 * mb11 + mai2 * mb21 + mai3 * mb31;
	  result[i*4+2] = mai0 * mb02 + mai1 * mb12 + mai2 * mb22 + mai3 * mb32;
	  result[i*4+3] = mai0 * mb03 + mai1 * mb13 + mai2 * mb23 + mai3 * mb33;
  	  }
	}
	
// calcul la distance d'un pointa un plan
inline GReal DistanceAuPlan(const float *plan, 
		     const GReal& x, const GReal& y, const GReal& z) const
{
  return(plan[0]*x + plan[1]*y + plan[2]*z + plan[3]);
}

}; 
#endif /* _VolumeVision_ */

