
#ifndef _geometrie_baseground_
#define _geometrie_baseground_
#include <math.h>


//------------------------------------------------------------------------
//-----POINT
//------------------------------------------------------------------------


/** \class IPoint2
\brief structure de base pour un point en 2D coordonnees en entier

*/

struct IPoint2
{
  
  ///data
  int x, y;
    
  //CONSTRUCTEURS
  
  ///constructeur defaut
  inline IPoint2() : x(0), y(0) {}
  ///constructeur
  inline IPoint2(const int& X,const int& Y)
                                 : x(X), y(Y) {}
  
  //ACCES
  
  ///acces
  inline int &operator[](const int& i) const {return ((int *)&x)[i] ;}
  ///acces
  inline int &operator[](const int& i) {return ((int *)&x)[i] ;}


  //METHODES CONSTANTES 
        

        
  //METHODES NON CONSTANTES


  inline IPoint2& operator=(const int &d)
  {
  x = d;
  y = d;
  return *this;
  }

  
  //FRIEND FONCTION

  inline friend bool operator!=(const IPoint2 &a,const IPoint2 &b) {
  if (a.x!=b.x)
        return true;
  if (a.y!=b.y)
        return true;
  return false;
  }
};



//------------------------------------------------------------------------
//-----VECTEUR
//------------------------------------------------------------------------


/** \class Vec2
\brief structure de base pour un vecteur 2D

*/

struct Vec2
{
  ///data
  float x, y;
  
  ///construteur
  inline Vec2() : x(0.0), y(0.0) {}
  ///constructeur
  inline Vec2(float X, float Y) : x(X), y(Y) {}

  ///acces
  inline float &operator[](const int& i) const {return ((float *)&x)[i] ;}
  ///acces
  inline float &operator[](const int& i) {return ((float *)&x)[i] ;}

  
  ///operation arithmetic
  inline friend Vec2 operator+(const Vec2 &a, const Vec2 &b) {
    return Vec2(a.x+b.x, a.y+b.y);
  }
  ///operation arithmetic
  inline friend Vec2 operator-(const Vec2 &a, const Vec2 &b) {
    return Vec2(a.x-b.x, a.y-b.y);
  }
  ///operation arithmetic
  inline friend Vec2 operator-(const Vec2 &a) {
    return Vec2(-a.x, -a.y);
  }
  ///operation arithmetic
  inline friend Vec2 operator*(const Vec2 &a, float k) {
    return Vec2(a.x*k, a.y*k) ;
  }
  ///operation arithmetic
  inline friend Vec2 operator*(float k, const Vec2 &a) {
    return Vec2(a.x*k, a.y*k) ;
  }
  ///operation arithmetic
  inline friend Vec2 operator/(const Vec2 &a, float k) {
    return Vec2(a.x/k, a.y/k) ;
  }
  ///operation arithmetic
  inline Vec2& operator+=(const Vec2 &a) {
    x += a.x; y += a.y;
    return *this;
  }
  ///operation arithmetic
  inline Vec2& operator-=(const Vec2 &a) {
    x -= a.x; y -= a.y;
        return *this;
  }
  ///operation arithmetic
  inline Vec2& operator*=(float k) {
    x *= k; y *= k;
        return *this;
  }
  ///operation arithmetic
  inline Vec2& operator/=(float k) {
    x /= k; y /= k;
        return *this;
  }
  
  ///operation arithmetic  
  inline friend float operator*(const Vec2 &a, const Vec2 &b) {
    return a.x*b.x + a.y*b.y;
  }
  
  ///operation arithmetic
  inline friend float norm2(const Vec2 &a) {
    return a*a ;
  }
  ///operation arithmetic
  inline friend float norm(const Vec2 &a) {
    return sqrt(a*a) ;
  }
  ///operation arithmetic
  inline void normalize() {
    *this /= norm(*this) ;
  }
  
  ///calcul le vecteur normal dans le plan
  inline friend Vec2 CalculVecteurNormal(const Vec2& v) {
  return Vec2 (v.y,-v.x);
  }
};




/** \class Vec3
\brief structure de base pour un vecteur 3D

*/

struct Vec3
{
  ///data
  float x, y, z;
  
  ///constructeur
  inline Vec3() : x(0.0), y(0.0), z(0.0) {}
  ///constructeur
  inline Vec3(float X, float Y, float Z) : x(X), y(Y), z(Z) {}

  ///acces
  inline float &operator[](const int& i) const {return ((float *)&x)[i] ;}
  ///acces
  inline float &operator[](const int& i) {return ((float *)&x)[i] ;}

  
  ///operation arithmetic
  inline friend Vec3 operator+(const Vec3 &a, const Vec3 &b) {
    return Vec3(a.x+b.x, a.y+b.y, a.z+b.z);
  }
  ///operation arithmetic
  inline friend Vec3 operator-(const Vec3 &a, const Vec3 &b) {
    return Vec3(a.x-b.x, a.y-b.y, a.z-b.z);
  }
  ///operation arithmetic
  inline friend Vec3 operator-(const Vec3 &a) {
    return Vec3(-a.x, -a.y, -a.z);
  }
  ///operation arithmetic
  inline friend Vec3 operator*(const Vec3 &a, float k) {
    return Vec3(a.x*k, a.y*k, a.z*k) ;
  }
  ///operation arithmetic
  inline friend Vec3 operator*(float k, const Vec3 &a) {
    return Vec3(a.x*k, a.y*k, a.z*k) ;
  }
  ///operation arithmetic
  inline friend Vec3 operator/(const Vec3 &a, float k) {
    return Vec3(a.x/k, a.y/k, a.z/k) ;
  }
  ///operation arithmetic
  inline Vec3& operator+=(const Vec3 &a) {
    x += a.x; y += a.y; z += a.z;
    return *this;
  }
  ///operation arithmetic
  inline Vec3& operator-=(const Vec3 &a) {
    x -= a.x; y -= a.y; z -= a.z;
        return *this;
  }
  ///operation arithmetic
  inline Vec3& operator*=(float k) {
    x *= k; y *= k; z *= k;
        return *this;
  }
  ///operation arithmetic
  inline Vec3& operator/=(float k) {
    x /= k; y /= k; z /= k;
        return *this;
  }
  
  ///operation arithmetic  
  inline friend float operator*(const Vec3 &a, const Vec3 &b) {
    return a.x*b.x + a.y*b.y + a.z*b.z;
  }
  ///operation arithmetic
  inline friend Vec3 cross(const Vec3 &a, const Vec3 &b) {
    return Vec3(a.y*b.z - a.z*b.y,
                a.z*b.x - a.x*b.z,
                a.x*b.y - a.y*b.x);
  }
  ///operation arithmetic
  inline friend float norm2(const Vec3 &a) {
    return a*a ;
  }
  ///operation arithmetic
  inline friend float norm(const Vec3 &a) {
    return sqrt(a*a) ;
  }
  ///operation arithmetic
  inline void normalize() {
    *this /= norm(*this) ;
  }
};




/** \class Point2
\brief structure de base pour un point en 2D

*/

struct Point2
{
  
  ///data
  float x, y;
  
  
  //CONSTRUCTEURS
  
  ///constructeur defaut
  inline Point2() : x(0.0), y(0.0) {}
  ///constructeur
  inline Point2(const float& X,const float& Y)
                                 : x(X), y(Y) {}
  
  //ACCES
  
  ///acces
  inline float &operator[](const int& i) const {return ((float *)&x)[i] ;}
  ///acces
  inline float &operator[](const int& i) {return ((float *)&x)[i] ;}


  //METHODES CONSTANTES 
        
  ///operation arithmetic
  inline float distance (const Point2 &p)      const {
        return sqrt( (x-p.x)*(x-p.x) + (y-p.y)*(y-p.y) );
  } 
  
  ///operation arithmetic
  inline float distance2 (const Point2 &p)     const {
        return (x-p.x)*(x-p.x) + (y-p.y)*(y-p.y);
  }   
   
  //projection ortho du point sur la droite (a,b)
  inline Point2 proj2right(const Point2 &a,
                           const Point2 &b) const {
  Point2 m (b.x-a.x, b.y-a.y);
  return a + ((*this - a) * m) * m / a.distance2(b);
  }
  
  //distance du point a la droite (a,b)
  inline float dist2right(const Point2 &a,
                           const Point2 &b) const {
  return this->distance(proj2right(a, b));
  } 
  
        
  //METHODES NON CONSTANTES
   
  ///operation arithmetic 
  inline Point2& operator=(const float &d)
  {
  x = d;
  y = d;
  return *this;
  }
  
  ///operation arithmetic 
  inline Point2& operator+=(const Point2 &p) {
  x += p.x;
  y += p.y;
  return *this;
  }

  ///operation arithmetic 
  inline Point2& operator-=(const Point2 &p) {
  x -= p.x;
  y -= p.y;
  return *this;
  }
  
  ///operation arithmetic 
  inline Point2& operator*=(const float &d) {
  x *= d;
  y *= d;
  return *this;
  }

  ///operation arithmetic 
  inline Point2& operator/=(const float &d) {
  x /= d;
  y /= d;
  return *this;
  }
  
  ///operation arithmetic 
  inline void normalize() {
  const float tmp = sqrt(x * x + y * y);
  x /= tmp; y /= tmp;
  }
  
  //FRIEND FONCTION

  ///operation arithmetic 
  inline friend Point2 operator-(const Point2 &a) {
    return Point2(-a.x, -a.y);
  }
  
  ///operation arithmetic
  inline friend Point2 operator+(const Point2 &a, const Point2 &b){
    return Point2(a.x+b.x, a.y+b.y);
  }
  
  ///operation arithmetic
  inline friend Point2 operator-(const Point2 &a, const Point2 &b){
    return Point2(a.x-b.x, a.y-b.y);
  }
  
  ///operation arithmetic
  inline friend Point2 operator*(const float &d, const Point2 &p){
  return Point2(d * (p.x), d * (p.y));
  }
  
  ///operation arithmetic 
  inline friend Point2 operator*(const Point2 &p, const float &d){
  return Point2(d * (p.x), d * (p.y));
  }
  
  ///operation arithmetic 
  inline friend float operator*(const Point2 &a, const Point2 &b){
  return a.x * b.x + a.y * b.y;
  }
  
  ///operation arithmetic
  inline friend Point2 operator/(const Point2 &p, const float &d){
  return Point2(p.x / d, p.y / d);
  }
  
  ///operation arithmetic
  inline friend Point2 operator/(const Point2 &a, const Point2 &b){
  return Point2(a.x / b.x, a.y / b.y);
  } 
  
  inline friend bool operator!=(const Point2 &a,const Point2 &b) {
  if (a.x!=b.x)
        return true;
  if (a.y!=b.y)
        return true;
  return false;
  }
  
  //POINT INTERSECTION
  //POINT INTERSECTION 
  
  inline friend bool intersectDroiteDroiteX(const Point2& p0, const Vec2& v0,
                                                                          const float&
valX,
                                                                          Point2&
pResult){
  if (v0.x<=0.0001 && v0.x>=-0.0001) 
        return false;
  float a = v0.y/v0.x;
  float b = p0.y - a*p0.x;
  pResult.x=valX;
  pResult.y=a*valX+b;
  return true;                                           
  }
  
  inline friend bool intersectSegmentDroiteX(const Point2& p0, const Point2& p1,
                                                                                         const
float& valX,
                                                                                         Point2&
pResult){
  
  if (p0.x<p1.x)
        {if (valX<p0.x || valX>p1.x) return false;}
  else if (p0.x>p1.x)
        {if (valX<p1.x || valX>p0.x) return false;} 
  else return false;
  float a = (p1.y-p0.y)/(p1.x-p0.x);
  float b = p0.y - a*p0.x;
  pResult.x=valX;
  pResult.y=a*valX+b;
  return true;                                           
  } 
  
  inline friend bool IntersectionDroiteDroite (const Point2& p0,
                                                                                  const
Vec2& v0,
                                                                                  const
Point2& p1,
                                                                                  const
Vec2& v1,
                                                                                  Point2&
pResult){
  
  if (v0.x==0.0)
        {
        if (v1.x!=0.0) return intersectDroiteDroiteX(p1,v1,p0.x,pResult);
        else return false;
        }
  if (v1.x==0.0)
        {
        if (v0.x!=0.0) return intersectDroiteDroiteX(p0,v0,p1.x,pResult);
        else return false;
        }
  float a0 = v0.y/v0.x;
  float b0 = p0.y - a0*p0.x;
  
  float a1 = v1.y/v1.x;
  float b1 = p1.y - a1*p1.x;
  
  if (a0-a1<=0.0001 && a0-a1>=-0.0001) return false;
  pResult.x=(b1-b0)/(a0-a1);
  pResult.y=a0*pResult.x+b0;
  return true;
  }
  

  
  
  
};





/** \class Point3
\brief structure de base pour un point en 3D

*/

struct Point3
{
  
  ///data
  float x, y, z;
  
  
  //CONSTRUCTEURS
  
  ///constructeur defaut
  inline Point3() : x(0.0), y(0.0), z(0.0) {}
  ///constructeur
  inline Point3(const float& X,const float& Y,const float& Z)
                                 : x(X), y(Y), z(Z) {}
  
  //ACCES
  
  ///acces
  inline float &operator[](const int& i) const {return ((float *)&x)[i] ;}
  ///acces
  inline float &operator[](const int& i) {return ((float *)&x)[i] ;}


  //METHODES CONSTANTES 
        
  ///operation arithmetic
  inline float distance (const Point3 &p)      const {
        return sqrt( (x-p.x)*(x-p.x) + (y-p.y)*(y-p.y) + (z-p.z)*(z-p.z) );
  } 
  
  ///operation arithmetic
  inline float distance2 (const Point3 &p)     const {
        return (x-p.x)*(x-p.x) + (y-p.y)*(y-p.y) + (z-p.z)*(z-p.z);
  }   
   
  //projection du point sur la droite (a,b)
  inline Point3 proj2right(const Point3 &a,
                           const Point3 &b) const {
  Point3 m = (b - a);
  return a + ((*this - a) * m) * m / a.distance2(b);
  }
  
  //distance du point a la droite (a,b)
  inline float dist2right(const Point3 &a,
                           const Point3 &b) const {
  return this->distance(proj2right(a, b));
  } 
  
        
  //METHODES NON CONSTANTES
   
  ///operation arithmetic 
  inline Point3& operator=(const float &d)
  {
  x = d;
  y = d;
  z = d;
  return *this;
  }
  
  ///operation arithmetic 
  inline Point3& operator+=(const Point3 &p) {
  x += p.x;
  y += p.y;
  z += p.z;
  return *this;
  }

  ///operation arithmetic 
  inline Point3& operator-=(const Point3 &p) {
  x -= p.x;
  y -= p.y;
  z -= p.z;
  return *this;
  }
  
  ///operation arithmetic 
  inline Point3& operator*=(const float &d) {
  x *= d;
  y *= d;
  z *= d;
  return *this;
  }

  ///operation arithmetic 
  inline Point3& operator/=(const float &d) {
  x /= d;
  y /= d;
  z /= d;
  return *this;
  }
  
  ///operation arithmetic 
  inline void normalize() {
  const float tmp = sqrt(x * x + y * y + z * z);
  x /= tmp; y /= tmp; z /= tmp;
  }
  
  //FRIEND FONCTION

  ///operation arithmetic 
  inline friend Point3 operator-(const Point3 &a) {
    return Point3(-a.x, -a.y, -a.z);
  }
  
  ///operation arithmetic
  inline friend Point3 operator+(const Point3 &a, const Point3 &b){
    return Point3(a.x+b.x, a.y+b.y, a.z+b.z);
  }
  
  ///operation arithmetic
  inline friend Point3 operator-(const Point3 &a, const Point3 &b){
    return Point3(a.x-b.x, a.y-b.y, a.z-b.z);
  }
  
  ///operation arithmetic
  inline friend Point3 operator*(const float &d, const Point3 &p){
  return Point3(d * (p.x), d * (p.y), d * (p.z));
  }
  
  ///operation arithmetic 
  inline friend Point3 operator*(const Point3 &p, const float &d){
  return Point3(d * (p.x), d * (p.y), d * (p.z));
  }
  
  ///operation arithmetic 
  inline friend float operator*(const Point3 &a, const Point3 &b){
  return a.x * b.x + a.y * b.y + a.z * b.z;
  }
  
  ///operation arithmetic
  inline friend Point3 operator/(const Point3 &p, const float &d){
  return Point3(p.x / d, p.y / d, p.z / d);
  }
  
  ///operation arithmetic
  inline friend Point3 operator/(const Point3 &a, const Point3 &b){
  return Point3(a.x / b.x, a.y / b.y, a.z / b.z);
  } 
  
  inline friend bool operator!=(const Point3 &a,const Point3 &b) {
  if (a.x!=b.x)
        return true;
  if (a.y!=b.y)
        return true;
  if (a.z!=b.z)
        return true;
  return false;
  }
};





#endif /* _geometrie_baseground_ */
