

#include "Ground.h"
#include "Image.h"

#include <string.h>


Terrain::Terrain()
{
m_displayList=0;
m_pListNoeuds=NULL;
m_init=false;

m_length=1.0;
m_width=1.0;
m_maxHeight=0.0;
}

Terrain::~Terrain()
{
if (m_displayList!=0) glDeleteLists(m_displayList,1);
if (m_pListNoeuds!=NULL) delete[] m_pListNoeuds;
m_texture.unloadMap();
m_texture.unloadGLTexture();
}

void Terrain::init(const float area[4],const float& maxHeight,
		   int& numberOfQuadsMax,const char* fileHeightMap,
		   const char* fileTexture, const float& repeatTexture)
{
int i,j;
int numberOfNoeuds;
int numberOfNoeudsPerLine;
int numberOfNoeudsPerRow;
int indice;
int indice1,indice2,indice3,indice4;
 
//donnes pour hauteur du terrain
int heightMapCols=0;
int heightMapRows=0;
unsigned char* heightMap=NULL;

//initialisation
m_init=true;
m_repeatTexture=repeatTexture;
m_numberOfQuadsMax=numberOfQuadsMax;
if (fileHeightMap==NULL) m_fileHeightMap[0]=0;
else strcpy(m_fileHeightMap,fileHeightMap);
if (fileTexture==NULL) m_fileTexture[0]=0;
else strcpy(m_fileTexture,fileTexture);

if (m_displayList!=0)
	{glDeleteLists(m_displayList,1);m_displayList=0;}
if (m_pListNoeuds!=NULL)
	{delete[] m_pListNoeuds;m_pListNoeuds=NULL;}

//initialisation
m_area[0]=area[0];m_area[1]=area[1];
m_area[2]=area[2];m_area[3]=area[3];
m_length=m_area[2]-m_area[0];
m_width=m_area[3]-m_area[1];
m_maxHeight=maxHeight;
if (m_length<0.0) m_length=1.0;
if (m_width<0.0) m_width=1.0;
if (numberOfQuadsMax<2) numberOfQuadsMax=2;
m_precision=int(sqrt(float(numberOfQuadsMax)));
numberOfQuadsMax=m_precision*m_precision;
numberOfNoeuds=(m_precision+1)*(m_precision+1);
numberOfNoeudsPerLine=m_precision+1;
numberOfNoeudsPerRow =m_precision+1;
m_pListNoeuds= new SNoeudTerrain[numberOfNoeuds];

//load heightMap
if (fileHeightMap!=NULL)
        {
        m_texture.loadMap(fileHeightMap);
        if (!m_texture.haveMap()) printf("  heightmap not loaded\n");
	else printf("  heightmap loaded\n");  
	m_texture.getMap(&heightMap,&heightMapCols,&heightMapRows);
	}
//construction des donnees hauteur aux noeuds
for (j=0;j<numberOfNoeudsPerRow;j++)
for (i=0;i<numberOfNoeudsPerLine;i++)
  {
  Point3 p;     
  p.x=m_area[0] + i*(m_length/m_precision);
  p.y=m_area[1] + j*(m_width/m_precision);                  
  p.z= getHeightMapAtPosition(p.x,p.y,maxHeight,heightMap,
			      heightMapCols,heightMapRows);
  m_pListNoeuds[j*numberOfNoeudsPerLine+i].position=p;
  }
m_texture.unloadMap();


//construction des donnees vecteur normal aux noeuds
for (j=0;j<numberOfNoeudsPerRow;j++)
for (i=0;i<numberOfNoeudsPerLine;i++)
  {     
  Vec3 normal(0.0,0.0,0.0);
  indice=j*numberOfNoeudsPerLine+i;
  bool noeudOk[]={false,false,false,false};
  
  if ((indice+1) % (numberOfNoeudsPerLine) != 0)
  noeudOk[0]=true;
  
  if (indice<numberOfNoeudsPerLine*(numberOfNoeudsPerRow-1))
  noeudOk[1]=true;
  
  if (indice % (numberOfNoeudsPerLine) != 0)
  noeudOk[2]=true;
 
  if (indice>=numberOfNoeudsPerLine)
  noeudOk[3]=true;
  
  indice1 = indice+1;
  indice2 = indice+numberOfNoeudsPerLine;
  indice3 = indice-1;
  indice4 = indice-numberOfNoeudsPerLine;

  if (noeudOk[0] && noeudOk[1])
        {
        Vec3 vec1(m_pListNoeuds[indice1].position[0]-m_pListNoeuds[indice].position[0],
                  m_pListNoeuds[indice1].position[1]-m_pListNoeuds[indice].position[1],
                  m_pListNoeuds[indice1].position[2]-m_pListNoeuds[indice].position[2]);
        Vec3 vec2(m_pListNoeuds[indice2].position[0]-m_pListNoeuds[indice].position[0],
                  m_pListNoeuds[indice2].position[1]-m_pListNoeuds[indice].position[1],
                  m_pListNoeuds[indice2].position[2]-m_pListNoeuds[indice].position[2]);
        normal += cross(vec1, vec2);
        }
  if (noeudOk[1] && noeudOk[2])
        {

        Vec3 vec2(m_pListNoeuds[indice2].position[0]-m_pListNoeuds[indice].position[0],
                  m_pListNoeuds[indice2].position[1]-m_pListNoeuds[indice].position[1],
                  m_pListNoeuds[indice2].position[2]-m_pListNoeuds[indice].position[2]);
        Vec3 vec3(m_pListNoeuds[indice3].position[0]-m_pListNoeuds[indice].position[0],
                  m_pListNoeuds[indice3].position[1]-m_pListNoeuds[indice].position[1],
                  m_pListNoeuds[indice3].position[2]-m_pListNoeuds[indice].position[2]);
        normal += cross(vec2, vec3);
        }
  if (noeudOk[2] && noeudOk[3])
        {
        Vec3 vec3(m_pListNoeuds[indice3].position[0]-m_pListNoeuds[indice].position[0],
                  m_pListNoeuds[indice3].position[1]-m_pListNoeuds[indice].position[1],
                  m_pListNoeuds[indice3].position[2]-m_pListNoeuds[indice].position[2]);
        Vec3 vec4(m_pListNoeuds[indice4].position[0]-m_pListNoeuds[indice].position[0],
                  m_pListNoeuds[indice4].position[1]-m_pListNoeuds[indice].position[1],
                  m_pListNoeuds[indice4].position[2]-m_pListNoeuds[indice].position[2]);
        normal += cross(vec3, vec4);
        }
  if (noeudOk[3] && noeudOk[0])
        {
        Vec3 vec1(m_pListNoeuds[indice1].position[0]-m_pListNoeuds[indice].position[0],
                  m_pListNoeuds[indice1].position[1]-m_pListNoeuds[indice].position[1],
                  m_pListNoeuds[indice1].position[2]-m_pListNoeuds[indice].position[2]);
        Vec3 vec4(m_pListNoeuds[indice4].position[0]-m_pListNoeuds[indice].position[0],
                  m_pListNoeuds[indice4].position[1]-m_pListNoeuds[indice].position[1],
                  m_pListNoeuds[indice4].position[2]-m_pListNoeuds[indice].position[2]);
        normal += cross(vec4, vec1);
        }
  normal.normalize();
  m_pListNoeuds[indice].normal=normal;
  }
m_texture.loadMap(fileTexture);
}

bool Terrain::isPositionOk(const float& x, const float& y) const
{
if (!m_init) return false;
if (x>m_area[2]) return false;
if (x<m_area[0]) return false;
if (y>m_area[3]) return false;
if (y<m_area[1]) return false;
return true;
}

float Terrain::getHeightAtPosition(const float& x, const float& y) 
const
{
int indiceNoeuds[4];
int ix,iy;
float lx,ly;
float height;
float height01,height03,height02;

if (!isPositionOk(x,y)) return 0.0;

//recuperation indices noeuds alentours
ix= (int) (((x-m_area[0]) * m_precision) / m_length);
iy= (int) (((y-m_area[1]) * m_precision) / m_width);
indiceNoeuds[0] = ix + iy*(m_precision+1);
indiceNoeuds[1] = indiceNoeuds[0] + 1;
indiceNoeuds[2] = ix + (iy+1)*(m_precision+1);
indiceNoeuds[3] = indiceNoeuds[2] + 1;

//coordonnees local au carre
lx = ((x-m_pListNoeuds[indiceNoeuds[0]].position.x) * m_precision) / m_length;
ly = ((y-m_pListNoeuds[indiceNoeuds[0]].position.y) * m_precision) / m_width;

if (lx>ly)
        {
        if (lx==0.0)
                height=m_pListNoeuds[indiceNoeuds[0]].position.z;
        else
                {
                //interpolation des noeuds 0,1 et 3
                height03 = (1.0-lx) *  m_pListNoeuds[indiceNoeuds[0]].position.z 
                               +lx  *  m_pListNoeuds[indiceNoeuds[3]].position.z; 
                height01 = (1.0-lx) *  m_pListNoeuds[indiceNoeuds[0]].position.z
                               +lx  *  m_pListNoeuds[indiceNoeuds[1]].position.z;
                
                height   = (1.0-(ly/lx))  * height01 + ((ly/lx)) * height03;
                }
        }
else
        {
        if (ly==0.0)
                height=m_pListNoeuds[indiceNoeuds[0]].position.z;
        else
                {
                //interpolation des noeuds 0,3 et 2
                height03 = (1.0-ly) *  m_pListNoeuds[indiceNoeuds[0]].position.z 
                               +ly  *  m_pListNoeuds[indiceNoeuds[3]].position.z; 
                height02 = (1.0-ly) *  m_pListNoeuds[indiceNoeuds[0]].position.z
                               +ly  *  m_pListNoeuds[indiceNoeuds[2]].position.z;
                height   = (1.0-(lx/ly)) * height02 + ((lx/ly)) * height03;
                }
        }
return height;
}

Vec3 Terrain::getNormalAtPosition(const float& x, const float& y) 
const
{
int indiceNoeuds[4];
int ix,iy;
float lx,ly;
Vec3 normal;
Vec3 normal01, normal02, normal03;
if (!isPositionOk(x,y))
        {
        normal.x=0.0;
        normal.y=0.0;
        normal.z=1.0;
        return normal;
        }

//recuperation indices noeuds
ix= (int) (((x-m_area[0]) * m_precision) / m_length);
iy= (int) (((y-m_area[1]) * m_precision) / m_width);
indiceNoeuds[0] = ix + iy*(m_precision+1);
indiceNoeuds[1] = indiceNoeuds[0] + 1;
indiceNoeuds[2] = ix + (iy+1)*(m_precision+1);
indiceNoeuds[3] = indiceNoeuds[2] + 1;

//coordonnees local au carre
lx = ((x-m_pListNoeuds[indiceNoeuds[0]].position.x) * m_precision) / m_length;
ly = ((y-m_pListNoeuds[indiceNoeuds[0]].position.y) * m_precision) / m_width;

//interpolation normal
if (lx>ly)
        {
        if (lx==0.0)
                normal=m_pListNoeuds[indiceNoeuds[0]].normal;
        else
                {
                //interpolation des noeuds 0,1 et 3
                normal03 = (1.0-lx) *  m_pListNoeuds[indiceNoeuds[0]].normal 
                                           +lx  *  m_pListNoeuds[indiceNoeuds[3]].normal; 
                normal01 = (1.0-lx) *  m_pListNoeuds[indiceNoeuds[0]].normal
                                           +lx  *  m_pListNoeuds[indiceNoeuds[1]].normal;
                
                normal   = (1.0-(ly/lx))  * normal01 + ((ly/lx)) * normal03;
                }
        }
else
        {
        if (ly==0.0)
                normal=m_pListNoeuds[indiceNoeuds[0]].normal;
        else
                {
                //interpolation des noeuds 0,3 et 2
                normal03 = (1.0-ly) *  m_pListNoeuds[indiceNoeuds[0]].normal 
                                           +ly  *  m_pListNoeuds[indiceNoeuds[3]].normal; 
                normal02 = (1.0-ly) *  m_pListNoeuds[indiceNoeuds[0]].normal
                                           +ly  *  m_pListNoeuds[indiceNoeuds[2]].normal;
                normal   = (1.0-(lx/ly)) * normal02 + ((lx/ly)) * normal03;
                }
        }
return normal;
}

bool Terrain::getDataAtPosition(const float& x, const float& y, 
				float* height, Vec3* vecNormal) const
{
int indiceNoeuds[4];
int ix,iy;
float lx,ly;
float height01,height03,height02;
Vec3 normal01, normal02, normal03;

if (!isPositionOk(x,y))
        {
        *height=0.0;
        vecNormal->x=0.0;
        vecNormal->y=0.0;
        vecNormal->z=1.0;
        return false;
        }

//recuperation indices noeuds
ix= (int) (((x-m_area[0]) * m_precision) / m_length);
iy= (int) (((y-m_area[1]) * m_precision) / m_width);
indiceNoeuds[0] = ix + iy*(m_precision+1);
indiceNoeuds[1] = indiceNoeuds[0] + 1;
indiceNoeuds[2] = ix + (iy+1)*(m_precision+1);
indiceNoeuds[3] = indiceNoeuds[2] + 1;

//coordonnees local au carre
lx = ((x-m_pListNoeuds[indiceNoeuds[0]].position.x) * m_precision) / m_length;
ly = ((y-m_pListNoeuds[indiceNoeuds[0]].position.y) * m_precision) / m_width;

//interpolation hauteur
if (lx>ly)
        {
        if (lx==0.0)
                *height=m_pListNoeuds[indiceNoeuds[0]].position.z;
        else
                {
                //interpolation des noeuds 0,1 et 3
                height03 = (1.0-lx) *  m_pListNoeuds[indiceNoeuds[0]].position.z 
                                           +lx  *  m_pListNoeuds[indiceNoeuds[3]].position.z; 
                height01 = (1.0-lx) *  m_pListNoeuds[indiceNoeuds[0]].position.z
                                           +lx  *  m_pListNoeuds[indiceNoeuds[1]].position.z;
                
                *height  = (1.0-(ly/lx))  * height01 + ((ly/lx)) * height03;
                }
        }
else
        {
        if (ly==0.0)
                *height=m_pListNoeuds[indiceNoeuds[0]].position.z;
        else
                {
                //interpolation des noeuds 0,3 et 2
                height03 = (1.0-ly) *  m_pListNoeuds[indiceNoeuds[0]].position.z 
                                           +ly  *  m_pListNoeuds[indiceNoeuds[3]].position.z; 
                height02 = (1.0-ly) *  m_pListNoeuds[indiceNoeuds[0]].position.z
                                           +ly  *  m_pListNoeuds[indiceNoeuds[2]].position.z;
                *height  = (1.0-(lx/ly)) * height02 + ((lx/ly)) * height03;
                }
        }
                   
//interpolation normal
if (lx>ly)
        {
        if (lx==0.0)
                *vecNormal=m_pListNoeuds[indiceNoeuds[0]].normal;
        else
                {
                //interpolation des noeuds 0,1 et 3
                normal03 = (1.0-lx) *  m_pListNoeuds[indiceNoeuds[0]].normal 
                                           +lx  *  m_pListNoeuds[indiceNoeuds[3]].normal; 
                normal01 = (1.0-lx) *  m_pListNoeuds[indiceNoeuds[0]].normal
                                           +lx  *  m_pListNoeuds[indiceNoeuds[1]].normal;
                
                *vecNormal       = (1.0-(ly/lx))  * normal01 + ((ly/lx)) * normal03;
                }
        }
else
        {
        if (ly==0.0)
                *vecNormal=m_pListNoeuds[indiceNoeuds[0]].normal;
        else
                {
                //interpolation des noeuds 0,3 et 2
                normal03 = (1.0-ly) *  m_pListNoeuds[indiceNoeuds[0]].normal 
                                           +ly  *  m_pListNoeuds[indiceNoeuds[3]].normal; 
                normal02 = (1.0-ly) *  m_pListNoeuds[indiceNoeuds[0]].normal
                                           +ly  *  m_pListNoeuds[indiceNoeuds[2]].normal;
                *vecNormal       = (1.0-(lx/ly)) * normal02 + ((lx/ly)) * normal03;
                }
        }
return true; 
}


void Terrain::draw()
{
if (m_displayList==0) loadDisplayList();
glCallList(m_displayList);
}

void Terrain::drawNormals(const float& length) const
{
int i;
if (!m_init) return;
glPushAttrib(GL_LIGHTING_BIT);
glPushAttrib(GL_TEXTURE_BIT);
glDisable(GL_LIGHTING);
glDisable(GL_TEXTURE_2D);
for (i=0;i<(m_precision+1)*(m_precision+1);i++)
        {
        glBegin(GL_LINES);
        
glColor3d(m_pListNoeuds[i].normal.x,m_pListNoeuds[i].normal.y,m_pListNoeuds[i].normal.z);
        
glVertex3d(m_pListNoeuds[i].position.x,m_pListNoeuds[i].position.y,m_pListNoeuds[i].position.z);
        
glVertex3d(m_pListNoeuds[i].position.x+length*m_pListNoeuds[i].normal.x,
                           m_pListNoeuds[i].position.y+length*m_pListNoeuds[i].normal.y,
                           m_pListNoeuds[i].position.z+length*m_pListNoeuds[i].normal.z);
        glEnd();
        }
glPopAttrib();
glPopAttrib();
}



float Terrain::getHeightMapAtPosition(const float& x, const float& y,const float& maxZ,
                                      const unsigned char* heightMap,
                                      const int heightMapCols,const int heightMapRows) const
{
int indice;
float rValue;
float gValue;
float bValue;      
if (heightMapCols==0 && heightMapRows==0) return 0.0;
if (heightMap==NULL) return 0.0;
indice=int( ((x-m_area[0])/(m_area[2]-m_area[0]))*(heightMapCols-1))*3 + 
	int( ((y-m_area[1])/(m_area[3]-m_area[1]))*(heightMapRows-1))*heightMapCols*3;

rValue=float(heightMap[indice]);
gValue=float(heightMap[indice+1]);
bValue=float(heightMap[indice+2]);
return maxZ*(rValue+gValue+bValue)*(1.0/765); 
}



void Terrain::loadDisplayList()
{
int i,j;
int numberOfNoeudsPerRow =m_precision+1;
int numberOfNoeudsPerLine=m_precision+1;
int indice;

if (!m_init) return;
GLfloat materialAmbient[]   = {0.3f, 0.3f, 0.3f, 1.0f} ;  
GLfloat materialDiffuse[]   = {1.0f, 1.0f, 1.0f, 1.0f} ;
GLfloat materialSpecular[]  = {0.0f, 0.0f, 0.0f, 1.0f} ;

m_displayList=glGenLists(1);
if (m_texture.haveMap())
  {
  printf("  texture loaded\n");
  m_texture.loadGLTexture();
  m_texture.unloadMap();
  
  glNewList(m_displayList, GL_COMPILE);
        
  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D,m_texture.getGLTextureId());
  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
  
  glMaterialfv(GL_FRONT, GL_AMBIENT, materialAmbient) ; 
  glMaterialfv(GL_FRONT, GL_DIFFUSE, materialDiffuse) ;
  glMaterialfv(GL_FRONT, GL_SPECULAR, materialSpecular) ;
  
  for (j=0;j<(numberOfNoeudsPerRow-1);j++)
        {
        glBegin(GL_TRIANGLE_STRIP);
        for (i=0;i<numberOfNoeudsPerLine;i++)
          {
          indice= (j+1)*numberOfNoeudsPerLine + i;
          glTexCoord2d(m_repeatTexture* float(i)/(numberOfNoeudsPerLine-1),m_repeatTexture* float(j+1)/(numberOfNoeudsPerRow-1));  
	  glNormal3d(m_pListNoeuds[indice].normal.x,m_pListNoeuds[indice].normal.y,m_pListNoeuds[indice].normal.z);
	  glVertex3d(m_pListNoeuds[indice].position.x,m_pListNoeuds[indice].position.y,m_pListNoeuds[indice].position.z);
                
          indice = j*numberOfNoeudsPerLine + i;
          glTexCoord2d(m_repeatTexture* float(i)/(numberOfNoeudsPerLine-1),m_repeatTexture* float(j)/(numberOfNoeudsPerRow-1));    
	  glNormal3d(m_pListNoeuds[indice].normal.x,m_pListNoeuds[indice].normal.y,m_pListNoeuds[indice].normal.z);
	  glVertex3d(m_pListNoeuds[indice].position.x,m_pListNoeuds[indice].position.y,m_pListNoeuds[indice].position.z);         
          }
        glEnd();
        }
  glDisable(GL_TEXTURE_2D);
  glEndList();
  }
else
  {
  printf("  texture not loaded\n");
  glNewList(m_displayList, GL_COMPILE);

  glMaterialfv(GL_FRONT, GL_AMBIENT, materialAmbient) ; 
  glMaterialfv(GL_FRONT, GL_DIFFUSE, materialDiffuse) ;
  glMaterialfv(GL_FRONT, GL_SPECULAR, materialSpecular) ;
  
  for (j=0;j<(numberOfNoeudsPerRow-1);j++)
        {
        glBegin(GL_TRIANGLE_STRIP); 
        for (i=0;i<numberOfNoeudsPerLine;i++)
          {
          indice= (j+1)*numberOfNoeudsPerLine + i;
	  glNormal3d(m_pListNoeuds[indice].normal.x,m_pListNoeuds[indice].normal.y,m_pListNoeuds[indice].normal.z);
	  glVertex3d(m_pListNoeuds[indice].position.x,m_pListNoeuds[indice].position.y,m_pListNoeuds[indice].position.z); 
          
	  indice = j*numberOfNoeudsPerLine + i;
	  glNormal3d(m_pListNoeuds[indice].normal.x,m_pListNoeuds[indice].normal.y,m_pListNoeuds[indice].normal.z);
	  glVertex3d(m_pListNoeuds[indice].position.x,m_pListNoeuds[indice].position.y,m_pListNoeuds[indice].position.z);         
          }
        glEnd();
        }
  glEndList();
  }
}
