#ifndef _terrain_
#define _terrain_

//include GL
#include <GL/gl.h>
#include <GL/glu.h>
//include io
#include <stdio.h>
#include <stdlib.h>

#include "GeometrieBaseGround.hpp"
#include "Image.h"


struct SNoeudTerrain
{
Point3 position;
Vec3   normal;
};


class Terrain
{
protected:
    float m_area[4];
    float m_length;
    float m_width;
    float m_maxHeight;
    
    int m_precision;
    SNoeudTerrain* m_pListNoeuds;
    
    //PARAM GL
    GLint m_displayList;
    Image m_texture;
    
    //PARAM
    char m_fileHeightMap[256];
    char m_fileTexture[256];
    int m_numberOfQuadsMax;
    float m_repeatTexture;
    
    bool m_init;
public:
    Terrain();	
    ~Terrain();
    
    void init(const float area[4], const float& maxHeight,
              int& numberOfQuadsMax, const char* fileHeightMap =NULL,
	      const char* fileTexture = NULL, const float& repeatTexture =1.0);
    
   const float* getArea() {return m_area;}
   float getXMin() {return m_area[0];}
   float getXMax() {return m_area[2];}
   float getYMin() {return m_area[1];}
   float getYMax() {return m_area[3];}
   float getLength() {return m_length;}
   float getWidth()  {return m_width;} 
   float getMaxHeight() {return m_maxHeight;}
   
   bool isPositionOk(const float& x, const float& y) const;
   float getHeightAtPosition(const float& x, const float& y) const;
   Vec3 getNormalAtPosition(const float& x, const float& y) const;
   bool getDataAtPosition(const float& x, const float& y, float* height, Vec3* vecNormal) const;
   
   const char* getFileHeightMap() {return m_fileHeightMap;}
   const char* getFileTexture() {return m_fileTexture;}
   int getNumberOfQuadsMax() {return m_numberOfQuadsMax;}
   float getRepeatTexture() {return m_repeatTexture;}
   
   void draw();
   void drawNormals(const float& length) const;

protected:
   float getHeightMapAtPosition(const float& x, const float& y,
                                 const float& maxZ,const unsigned char* heightMap,
                                 const int heightMapCols,const int heightMapRows) const;
   void loadDisplayList();
};

#endif /* _terrain_ */
