#include "GroundInterface.h"

#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/parserInternals.h>
#include <string>
#include <vector>

void groundXMLtokenize(const std::string& str,
	      std::vector<std::string>& tokens,
	      const std::string& delimiters) 
{
  std::string::size_type lastPos = str.find_first_not_of(delimiters, 0);
  std::string::size_type pos = str.find_first_of(delimiters, lastPos);
  
  while (std::string::npos != pos || std::string::npos != lastPos) {
    tokens.push_back(str.substr(lastPos, pos - lastPos));
    lastPos = str.find_first_not_of(delimiters, pos);
    pos = str.find_first_of(delimiters, lastPos);
  }
}

void groundXMLgetFloatArray(xmlNodePtr node, char* attribname, float x[]) 
{
  std::string text = (const char*) xmlGetProp(node, (const xmlChar *) attribname);
  std::vector<std::string> tokens;
  groundXMLtokenize(text, tokens, " ");
  for (int i=0; i<(int)tokens.size(); i++) 
    {
    std::string test = tokens[i];
    x[i] = atof(test.c_str());
    }
}


bool groundLoadNode(xmlNodePtr nodeGround)
{
//variables en attente d'initialisation
float area[]={groundGetXMin(),groundGetYMin(),groundGetXMax(),groundGetYMax()};
float max_height=groundGetMaxHeight();
const char* height_map=groundGetFileHeightMap();
const char* texture=groundGetFileTexture();
int quads_number=groundGetNumberOfQuadsMax();
float texture_repeat=groundGetRepeatTexture();
	 
//parcours	 
xmlAttrPtr attr = nodeGround->properties;
while (attr != NULL) 
   {
   std::string attribute = (const char*) attr->name;
   if (attribute=="area")
       {
       groundXMLgetFloatArray(nodeGround, "area", area);
       }
   if (attribute=="height_map")
       {
       height_map = (const char*) xmlGetProp(nodeGround, (const xmlChar *) "height_map");
       }
   if (attribute=="max_height")
       {
       max_height = atof((const char*) xmlGetProp(nodeGround, (const xmlChar*) "max_height"));
       }
   if (attribute=="texture")
       {
       texture = (const char*) xmlGetProp(nodeGround, (const xmlChar *) "texture");
       }
   if (attribute=="texture_repeat")
       {
       texture_repeat=atof((const char*) xmlGetProp(nodeGround, (const xmlChar*) "texture_repeat"));
       }
   if (attribute=="quads_number")
       {
       float r[1];groundXMLgetFloatArray(nodeGround, "quads_number", r);
       quads_number=int(r[0]);
       }
   attr=attr->next;
   }
groundInit(area,max_height,quads_number,
	   height_map,texture,texture_repeat);
return true;
}

bool groundLoadXML(xmlNodePtr nodePrairie)
{
xmlNodePtr child = nodePrairie->children;
while (child != NULL) 
   {
   if (child->type == XML_ELEMENT_NODE)
      {   
      std::string name = (const char*) child->name;
      if (name == "ground")
         {
	 return (groundLoadNode(child));
	 }
      }
   child = child->next;
   }
return false;
}


bool groundLoadXML(const char* filename)
{
xmlNodePtr p_node;
xmlDocPtr  p_doc;

//initialisation
if (filename==NULL) return false;
p_doc = xmlParseFile(filename);
if (p_doc == NULL) return false;
p_node = xmlDocGetRootElement(p_doc);

//validation
xmlValidCtxt cvp; 
cvp.userData = stderr;
cvp.error = (xmlValidityErrorFunc) fprintf;
cvp.warning = (xmlValidityWarningFunc) fprintf;  
if (!xmlValidateDocument(&cvp, p_doc)) return false;

xmlNodePtr child = p_node->children;
while (child != NULL) 
   {
   if (child->type == XML_ELEMENT_NODE)
      {   
      std::string name = (const char*) child->name;
      if (name == "prairie")
	 groundLoadXML(child);
      }
   child = child->next;
   }
return true;
}


void groundSaveXML(xmlNodePtr p_nodePrairie)
{
char value[256];
xmlNodePtr p_nodeGround = xmlNewNode (NULL, (const xmlChar*)"ground");

//area
sprintf(value,"%f %f %f %f", 0.0, 0.0, groundGetLength(), groundGetWidth());
xmlNewProp(p_nodeGround,(const xmlChar*)"area",(const xmlChar*)value);

//max_height
sprintf(value,"%f", groundGetMaxHeight());
xmlNewProp(p_nodeGround,(const xmlChar*)"max_height",(const xmlChar*)value);

//quads_number
sprintf(value,"%d", groundGetNumberOfQuadsMax());
xmlNewProp(p_nodeGround,(const xmlChar*)"quads_number",(const xmlChar*)value);

//height_map
sprintf(value,"%s", groundGetFileHeightMap());
xmlNewProp(p_nodeGround,(const xmlChar*)"height_map",(const xmlChar*)value);

//texture
sprintf(value,"%s", groundGetFileTexture());
xmlNewProp(p_nodeGround,(const xmlChar*)"texture",(const xmlChar*)value);

//texture_repeat
sprintf(value,"%f", groundGetRepeatTexture());
xmlNewProp(p_nodeGround,(const xmlChar*)"texture_repeat",(const xmlChar*)value);

xmlAddChild(p_nodePrairie,p_nodeGround);
}
