#include "Image.h"

#include <stdio.h>
#include <iostream.h>
#include <string.h>

#include "GeometrieBaseGround.hpp"

Image::Image()
	{
	_map=NULL;
	_cols=0;
	_rows=0;
	_textureId=0;
	}
	
Image::~Image()
	{
	if (_map!=NULL) delete _map;
	if (_textureId!=0) glDeleteTextures(1,&_textureId);
	}


bool Image::loadMap(const char * filename)
        {
	//cerr << "Image::loadMap " << filename << endl;
	const char *pStr;
	if (_map!=NULL) {delete _map;_map=NULL;}
	pStr=filename;
	while (strcmp(pStr,".ppm") && strcmp(pStr,".bmp")) { pStr++; }
        if (!strcmp(pStr,".ppm")) make_with_PPM(filename);
        else if (!strcmp(pStr,".bmp")) make_with_BMP(filename);
		else cerr << "Image::loadMap: wrong filename " << filename << endl;
        return (_map!=NULL);
	}

void Image::unloadMap()
	{
	if (_map!=NULL) {delete _map;_map=NULL;}
	}

void Image::loadGLTexture(bool mipmap)
	{
	if (_map==NULL) return;
	if (_textureId!=0) {glDeleteTextures(1,&_textureId); _textureId=0;}
	
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
        glGenTextures(1, &_textureId);
        glBindTexture(GL_TEXTURE_2D,_textureId);

        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER,GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,GL_LINEAR);
        
	
	if (mipmap)
		{
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,
				GL_NEAREST_MIPMAP_NEAREST);
		gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB, _cols, _rows,
				  GL_RGB, GL_UNSIGNED_BYTE, _map);
		}
	else
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, _cols, _rows,
		     0, GL_RGB, GL_UNSIGNED_BYTE, _map);
	}

void Image::unloadGLTexture()
	{
	if (_textureId!=0) {glDeleteTextures(1,&_textureId); _textureId=0;}
	}


int Image::swapInt(int i)
{
	return
	((i & 0x000000ff) << 24) |
	((i & 0x0000ff00) <<  8) |
	((i & 0x00ff0000) >>  8) |
	((i & 0xff000000) >> 24);
}

void Image::make_with_PPM(const char *filename)
{
FILE* ifp;
int ich;
int i;
        
ifp = fopen(filename, "r");
        
/* Lecture du Magic number */
getc( ifp );
ich = getc( ifp );
if(ich != '6') {fclose(ifp);return;}
    
/* Lecture des dimensions */
_cols = pm_getint( ifp );
_rows = pm_getint( ifp );

/* Allocation memoire  */
if (_cols+_rows==0) {fclose(ifp);return;}
_map=new GLubyte[_cols * _rows * 3];

for(i=0; i < _cols * _rows * 3; i+=3)
                {
         	_map[i+2] = pm_getrawbyte(ifp);
                 _map[i  ] = pm_getrawbyte(ifp);
                 _map[i+1] = pm_getrawbyte(ifp);        
                }
fclose(ifp);
}

void Image::make_with_BMP(const char *File)
{               
        FILE          *fichier;
        unsigned char Header[0x36];
        GLuint        DataPos,DataSize;
	
        fichier = fopen(File,"rb");if (!fichier){
						printf("  error chargement BMP code 1\n");
						return;
						}
        if (fread(Header,1,0x36,fichier)!=0x36) {
						fclose(fichier);
						printf("  error chargement BMP code 2\n");
						return;
						}
        if (Header[0]!='B' || Header[1]!='M')   {
						fclose(fichier);
						printf("  error chargement BMP code 3\n");
						return;}
        if (Header[0x1E]!=0)        		{
						fclose(fichier);
						printf("  error chargement BMP code 4\n");
						return;
						}
	if (Header[0x1C]!=24)       		{
						fclose(fichier);
						printf("  error chargement BMP (not 24bits)");
						printf("  -%dbits\n", (Header[0x1C]));
						return;
						}
        DataPos                 = (*(int*)&(Header[0x0A]));
        DataSize                = (*(int*)&(Header[0x22]));
        _cols                   = (*(int*)&(Header[0x12]));
        _rows                   = (*(int*)&(Header[0x16]));     
        
	#ifdef __sgi
	DataPos=swapInt(DataPos);	
	DataSize=swapInt(DataSize);
	_cols=swapInt(_cols);
	_rows=swapInt(_rows);
	#endif	
		
        //
        if (DataSize==0) DataSize=_rows*_cols*3;
        if (DataPos==0)  DataPos=0x36;

        fseek(fichier,DataPos,0);
        _map = new GLubyte[DataSize];
        if (!_map) {fclose(fichier);printf("  error chargement BMP (memory)\n");return;}

        if (fread(_map,1,DataSize,fichier)!=DataSize) 
        	{
                delete _map;_map=NULL;_cols=0;_rows=0;
                fclose(fichier);
                printf("  error chargement BMP (read)\n");
		return;
        	}
        
        fclose(fichier);

        unsigned char t;
        for (int x=0;x<_rows*_cols;x++) 
        	{
                t=_map[x*3];
                _map[x*3]=_map[x*3+2];
                _map[x*3+2]=t;
        	}	 
}


//lecture fichier, renvoi un char
char Image::pm_getc(FILE* file)
    {
    register int ich;
    register char ch;
 
    ich = getc( file );
    if ( ich == EOF )
        printf("read error" );
    ch = (char) ich;
    
    if ( ch == '#' )
        {
        do
            {
            ich = getc( file );
            if ( ich == EOF )
                printf("read error" );
            ch = (char) ich;
            }
        while ( ch != '\n' && ch != '\r' );
        }
 
    return ch;
    }

//lecture fichier ASCII, renvoi un int
int Image::pm_getint( FILE* file)
    {
    register char ch;
    register int i;
 
    do
        {
        ch = pm_getc( file );
        }
    while ( ch == ' ' || ch == '\t' || ch == '\n' || ch == '\r' );
 
    if ( ch < '0' || ch > '9' )
        printf( "read error" );
 
    i = 0;
    do
        {
        i = i * 10 + ch - '0';
        ch = pm_getc( file );
        }
    while ( ch >= '0' && ch <= '9' );
 
    return i;
    }

//lecture fichier non ASCII, renvoi un int      
int Image::pm_getrawbyte(FILE* file)
    {
    register int iby;
 
    iby = getc( file );
    if ( iby == EOF )
        printf("read error" );
    return (int) iby;
    }
