
#ifndef _information_primitive_
#define _information_primitive_

// primanim
#include "geometrie.h"

//include io
#include <stdio.h>
#include <stdlib.h>



/** \class InformationVent
\brief stockage des informations de vent
(==wind_info)


*/

class InformationVent
{
public:
	///la puissance du vent
	float _power;
	///le vecteur de vent
	float _vecteur[2];

public:
	/// constructeur par defaut
	/// le vecteur vent est nulle
	InformationVent()
		{
//		printf("constructeur informationVent...");
		_power=0;
		_vecteur[0]=0.0;
		_vecteur[1]=0.0;
		}
		
	/// constructeur
	InformationVent(float p,float v1,float v2)
		{
//		printf("constructeur informationVent...");
		_power=p;
		_vecteur[0]=v1;
		_vecteur[1]=v2;
		}
	
	/// destructeur	
	~InformationVent(){}
	
	/// constructeur par recopie
	InformationVent(const InformationVent &iv )
		{
//		printf("constructeur recopie informationVent...");
		_power=iv._power;
		_vecteur[0]=iv._vecteur[0];
		_vecteur[1]=iv._vecteur[1];
		}
	
	///initialisation
	void Initialise()
		{
		_power=0;
		_vecteur[0]=0.0;
		_vecteur[1]=0.0;
		}
	
	/// surdefinition de l'affectation
	InformationVent& operator=(const InformationVent& iv)
		{
//		printf("affectation informationVent...");
		_power=iv._power;
		_vecteur[0]=iv._vecteur[0];
		_vecteur[1]=iv._vecteur[1];	
		return *this;
		}
	
	///permet d'ajouter deux InformationVent
	void operator+=(const InformationVent iv)
		{
		_power+=iv._power;
		_vecteur[0]+=iv._power*iv._vecteur[0];
		_vecteur[1]+=iv._power*iv._vecteur[1];	
		}
		
	///normalisation des vecteurs 
	void normalize()
		{
		static const float seuil_att=30.0;
		static const float seuil_moy=1.0;
		
		if (_power>seuil_moy)
		{
			_vecteur[0]/=_power;
			_vecteur[1]/=_power;	
		}
		else
		{
			_vecteur[0]=0.0;
			_vecteur[1]=0.0;	
		}
		
		if (_power<seuil_att)
		{
			const float tmp=_power/seuil_att;
			// attenuation
			_vecteur[0]*=tmp;
			_vecteur[1]*=tmp;
		}
	
		}
		
	///permet d'ajouter deux InformationVent
	friend InformationVent operator+(const InformationVent& iv1,
									 const InformationVent& iv2)
		{
//		printf("ajout de deux informationVent...");
		InformationVent resultat;
		resultat._vecteur[0]=iv1._vecteur[0]+iv2._vecteur[0];
		resultat._vecteur[1]=iv1._vecteur[1]+iv2._vecteur[1];	
		return resultat;
		}
	
	///affichage des informations	
	friend std::ostream& operator<<(std::ostream& os, const InformationVent& obj)
		{
			os<<"power = "<<obj._power<<"   wind = ( "<<obj._vecteur[0]<<" , "<<obj._vecteur[1]<<")";
			return os;
		}
		
};

///l'information que les primitives de vent manipulent
typedef InformationVent wind_info;




#endif /* _information_primitive_ */
