#include "WindInterface.h"

#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/parserInternals.h>
#include <string>
#include <vector>

#include "bounding_box2d.h"

extern bounding_box2d g_terrain;
extern float g_heightForDrawingWind;
extern float g_breezePower;
extern float g_breezeSpeed;
extern double g_breezeTime;
extern float g_gustPower;
extern float g_gustSpeed;
extern float g_blastPower;
extern float g_blastParam;

//using std::string;

void windXMLtokenize(const std::string& str,
	      vector<std::string>& tokens,
	      const std::string& delimiters) 
{
  std::string::size_type lastPos = str.find_first_not_of(delimiters, 0);
  std::string::size_type pos = str.find_first_of(delimiters, lastPos);
  
  while (std::string::npos != pos || std::string::npos != lastPos) {
    tokens.push_back(str.substr(lastPos, pos - lastPos));
    lastPos = str.find_first_not_of(delimiters, pos);
    pos = str.find_first_of(delimiters, lastPos);
  }
}

void windXMLgetFloatArray(xmlNodePtr node, char* attribname, float x[]) 
{
  std::string text = (const char*) xmlGetProp(node, (const xmlChar *) attribname);
  vector<std::string> tokens;
  windXMLtokenize(text, tokens, " ");
  for (int i=0; i<(int)tokens.size(); i++) 
    {
    std::string test = tokens[i];
    x[i] = atof(test.c_str());
    }
}

bool windLoadNode(xmlNodePtr nodeWind)
{
//variables en attente d'initialisation
float area[]={g_terrain.min_x,g_terrain.min_y,g_terrain.max_x,g_terrain.max_y};
float height_drawing_gust=g_heightForDrawingWind;
float breeze_speed=g_breezeSpeed;
float breeze_power=g_breezePower;
float gust_speed=g_gustSpeed;
float gust_power=g_gustPower;
float blast_param=g_blastParam;
float blast_power=g_blastPower;
			 
xmlAttrPtr attr = nodeWind->properties;
while (attr != NULL) 
   {
   std::string attribute = (const char*) attr->name;
   if (attribute=="area")
   	{
	windXMLgetFloatArray(nodeWind, "area", area);
	}
   if (attribute=="height_drawing_gust")
   	{
   	height_drawing_gust = atof((const char*) xmlGetProp(nodeWind, (const xmlChar *)"height_drawing_gust"));
   	}
   if (attribute=="breeze_speed")
   	{
   	breeze_speed = atof((const char*) xmlGetProp(nodeWind, (const xmlChar *)"breeze_speed"));
   	}
   if (attribute=="breeze_power")
   	{
   	breeze_power = atof((const char*) xmlGetProp(nodeWind, (const xmlChar *)"breeze_power"));
   	}
   if (attribute=="gust_speed")
   	{
   	gust_speed = atof((const char*) xmlGetProp(nodeWind, (const xmlChar *)"gust_speed"));
   	}
   if (attribute=="gust_power")
   	{
   	gust_power = atof((const char*) xmlGetProp(nodeWind, (const xmlChar *)"gust_power"));
   	}
   if (attribute=="blast_param")
   	{
   	blast_param = atof((const char*) xmlGetProp(nodeWind, (const xmlChar *)"blast_param"));
   	}
   if (attribute=="blast_power")
   	{
   	blast_power = atof((const char*) xmlGetProp(nodeWind, (const xmlChar *)"blast_power"));
   	}

   attr=attr->next;
   }
windInit(area,height_drawing_gust);
windSetBreezePower(breeze_power);
windSetBreezeSpeed(breeze_speed);
windSetGustPower(gust_power);
windSetGustSpeed(gust_speed);
windSetBlastPower(blast_power);
windSetBlastParam(blast_param);
return true;  
}

bool windLoadXML(xmlNodePtr nodePrairie)
{
xmlNodePtr child = nodePrairie->children;
while (child != NULL) 
   {
   if (child->type == XML_ELEMENT_NODE)
      {   
      std::string name = (const char*) child->name;
      if (name == "wind")
         {
	 return (windLoadNode(child));	 
	 }	 
      }
   child = child->next;
   }
return false;
}


bool windLoadXML(const char* filename)
{
xmlNodePtr p_node;
xmlDocPtr  p_doc;

//initialisation
if (filename==NULL) return false;
p_doc = xmlParseFile(filename);
if (p_doc == NULL) return false;
p_node = xmlDocGetRootElement(p_doc);

//validation
xmlValidCtxt cvp; 
cvp.userData = stderr;
cvp.error = (xmlValidityErrorFunc) fprintf;
cvp.warning = (xmlValidityWarningFunc) fprintf;  
if (!xmlValidateDocument(&cvp, p_doc)) return false;

xmlNodePtr child = p_node->children;
while (child != NULL) 
   {
   if (child->type == XML_ELEMENT_NODE)
      {   
      std::string name = (const char*) child->name;
      if (name == "prairie")
	 windLoadXML(child);
      }
   child = child->next;
   }
return true;
}



void windSaveXML(xmlNodePtr p_nodePrairie)
{
char value[256];
xmlNodePtr p_nodeWind = xmlNewNode (NULL, (const xmlChar*)"wind");

//area
sprintf(value,"%f %f %f %f",g_terrain.min_x,g_terrain.min_y,
			    g_terrain.max_x,g_terrain.max_x);
xmlNewProp(p_nodeWind,(const xmlChar*)"area",(const xmlChar*)value);

//height_drawing_gust
sprintf(value,"%f", g_heightForDrawingWind);
xmlNewProp(p_nodeWind,(const xmlChar*)"height_drawing_gust",(const xmlChar*)value);

//breeze_speed
sprintf(value,"%f", windGetBreezeSpeed());
xmlNewProp(p_nodeWind,(const xmlChar*)"breeze_speed",(const xmlChar*)value);

//breeze_power
sprintf(value,"%f", windGetBreezePower());
xmlNewProp(p_nodeWind,(const xmlChar*)"breeze_power",(const xmlChar*)value);

//gust_speed
sprintf(value,"%f", windGetGustSpeed());
xmlNewProp(p_nodeWind,(const xmlChar*)"gust_speed",(const xmlChar*)value);

//gust_power
sprintf(value,"%f", windGetGustPower());
xmlNewProp(p_nodeWind,(const xmlChar*)"gust_power",(const xmlChar*)value);

//blast_param
sprintf(value,"%f", windGetBlastParam());
xmlNewProp(p_nodeWind,(const xmlChar*)"blast_param",(const xmlChar*)value);

//blast_power
sprintf(value,"%f", windGetBlastPower());
xmlNewProp(p_nodeWind,(const xmlChar*)"blast_power",(const xmlChar*)value);

xmlAddChild(p_nodePrairie,p_nodeWind);
}


