/* 
		alea_2d_float_field: corps de la classe

		Author: Frank Perbet

		Versions:
 			- [DATE]: (initial version) 
*/

/*************************************************************************/
#include "alea_2d_float_field.h"

#include "debug.h"
#include "math_perso.h"


//! get the 2d value of the function
geometrie::point2d alea_2d_float_field::get_value(float x,float y, float t)
{
	int size_x=pt_vol->size();
	int size_y=(&((*pt_vol)[0]))->size();
	int size_t=(&((*pt_vol)[0][0]))->size();
	
	float index_x=math_perso::modulo(x/coef_x,size_x);
	float index_y=math_perso::modulo(y/coef_y,size_y);
	float index_t=math_perso::modulo(t/coef_t,size_t);
	
	int int_index_x=math_perso::floor(index_x);
	int int_index_y=math_perso::floor(index_y);
	int int_index_t=math_perso::floor(index_t);
	
	index_x-=int_index_x;
	index_y-=int_index_y;
	index_t-=int_index_t;
	
	int aux_x=int_index_x*a_dec_x;
	int aux_y=int_index_y*a_dec_y;
	int aux_t=int_index_t*a_dec_t;
	
	float xx=math_perso::linear_interpolation8(
		(*pt_vol)[(aux_x+1)%size_x][(aux_y  )%size_y][(aux_t+1)%size_t],
		(*pt_vol)[(aux_x+1)%size_x][(aux_y  )%size_y][(aux_t  )%size_t],
		(*pt_vol)[(aux_x+1)%size_x][(aux_y+1)%size_y][(aux_t+1)%size_t],
		(*pt_vol)[(aux_x+1)%size_x][(aux_y+1)%size_y][(aux_t  )%size_t],
		(*pt_vol)[(aux_x  )%size_x][(aux_y  )%size_y][(aux_t+1)%size_t],
		(*pt_vol)[(aux_x  )%size_x][(aux_y  )%size_y][(aux_t  )%size_t],
		(*pt_vol)[(aux_x  )%size_x][(aux_y+1)%size_y][(aux_t+1)%size_t],
		(*pt_vol)[(aux_x  )%size_x][(aux_y+1)%size_y][(aux_t  )%size_t],
		index_x,
		index_y,
		index_t);
		
	aux_x=int_index_x*b_dec_x;
	aux_y=int_index_y*b_dec_y;
	aux_t=int_index_t*b_dec_t;

	float yy=math_perso::linear_interpolation8(
		(*pt_vol)[(aux_x+1)%size_x][(aux_y  )%size_y][(aux_t+1)%size_t],
		(*pt_vol)[(aux_x+1)%size_x][(aux_y  )%size_y][(aux_t  )%size_t],
		(*pt_vol)[(aux_x+1)%size_x][(aux_y+1)%size_y][(aux_t+1)%size_t],
		(*pt_vol)[(aux_x+1)%size_x][(aux_y+1)%size_y][(aux_t  )%size_t],
		(*pt_vol)[(aux_x  )%size_x][(aux_y  )%size_y][(aux_t+1)%size_t],
		(*pt_vol)[(aux_x  )%size_x][(aux_y  )%size_y][(aux_t  )%size_t],
		(*pt_vol)[(aux_x  )%size_x][(aux_y+1)%size_y][(aux_t+1)%size_t],
		(*pt_vol)[(aux_x  )%size_x][(aux_y+1)%size_y][(aux_t  )%size_t],
		index_x,
		index_y,
		index_t);
	
	return geometrie::point2d(xx,yy);
}

// the default constructor
alea_2d_float_field::alea_2d_float_field(ressource::ty_volume_float* pt,float cx,float cy,float ct)
:pt_vol(pt), coef_x(cx), coef_y(cy), coef_t(ct)
{
	a_dec_x=math_perso::floor(math_perso::random()*10);
	a_dec_y=math_perso::floor(math_perso::random()*10);
	a_dec_t=math_perso::floor(math_perso::random()*10);
	b_dec_x=math_perso::floor(math_perso::random()*10);
	b_dec_y=math_perso::floor(math_perso::random()*10);
	b_dec_t=math_perso::floor(math_perso::random()*10);
	
	a_dec_x= a_dec_x<1 ? 1 : a_dec_x;
	a_dec_y= a_dec_y<1 ? 1 : a_dec_y;
	a_dec_t= a_dec_t<1 ? 1 : a_dec_t;
	b_dec_x= b_dec_x<1 ? 1 : b_dec_x;
	b_dec_y= b_dec_y<1 ? 1 : b_dec_y;
	b_dec_t= b_dec_t<1 ? 1 : b_dec_t;
}

// the copy constructor
alea_2d_float_field::alea_2d_float_field(const alea_2d_float_field& obj)
{
	debug_frank::error("No copy constructor implemented in the class alea_2d_float_field.\n",1);
}

// the assignement constructor:
alea_2d_float_field& alea_2d_float_field::operator=(const alea_2d_float_field& obj)
{
	if (this != &obj)
	{
		debug_frank::error("No assignement implemented in the class alea_2d_float_field.\n",1);
	}
	return *this;
}

// the destructor: 
alea_2d_float_field::~alea_2d_float_field()
{
	debug_frank::error("No destructor implemented in the class alea_2d_float_field.\n",1);
}

// display
std::ostream& operator<<(std::ostream& os, const alea_2d_float_field& obj)
{
	os<<"No display implemented in the class alea_2d_float_field.\n";
	return os;
}
