/* 
		blast_of_air: specification

		Author: Frank Perbet

		Versions:
 			- 
*/


/* 
	Comments:

*/

/*************************************************************************/

#ifndef _blast_of_air_
#define _blast_of_air_

#include <iostream>
#include "wind_primitive.h"
#include "circle.h"
#include "geometrie.h"
#include "figure2d.h"
#include "life.h"
#include "GL/glut.h"

//! Class blast_of_air
/*!
MORE COMMENT ABOUT THE CLASS
*/
extern float g_blastPower;
extern float g_blastParam;

class blast_of_air : public wind_primitive
{
public:

	/////////////////////////////////
	// atributs of the class:

	//! life of the blast of wind prim
	life<blast_of_air>* my_life;
	
	/// position
	//geometrie::point2d* pos;
	
	// end of atributs of the class.
	/////////////////////////////////
	
	
	/////////////////////////////////
	// methods of the class

	inline virtual void ogl_draw() const
	{
	
		glPushAttrib(GL_LIGHTING_BIT);
		glPushAttrib(GL_TEXTURE_BIT);
		
		glDisable(GL_LIGHTING);
		glDisable(GL_TEXTURE_2D);
		
		glColor3f(0.8*power_coef,0.3*power_coef,0.3*power_coef);
		pt_fig->ogl_draw();
		
		glPopAttrib();
		glPopAttrib();
	}

	//! add a slice of life
	inline void add_slice_of_life(slice_of_life<blast_of_air>* s)
	{
		this->my_life->add_slice_of_life(s);
	}

	//! make the primitive live
	inline virtual bool up_to_date()
	{
		my_life->up_to_date();
		my_life->action();
//		pt_fig->r.origin.x=pos->x;
//		pt_fig->r.origin.y=pos->y;
		
		return !am_i_dead;
	}

	//! return the information emitted to the point p
	inline virtual info give_info(geometrie::point2d p)
	{
		//static circle* c=static_cast<circle*>(pt_fig);
		circle& c=static_cast<circle&>(*pt_fig);
		//geometrie::point2d tmp = geometrie::point2d(p-(c->r.origin));
		geometrie::point2d tmp = p-(c.r.origin);
		double dis = tmp.norme(); tmp = (1/dis)*tmp;
		//double marge_int=c->radius/5.0;
		//double marge_ext=c->radius/5.0;
		double marge_int=c.radius/5.0;
		double marge_ext=c.radius/5.0;
		long seed = static_cast<long>(p.y*9816.2165+p.x*32);
		srand(seed);
		
		double amplitude_max = 0.5*(double(rand()) / RAND_MAX);
		double amplitude_de_base_max = g_blastParam+0.1*(double(rand()) / RAND_MAX);
		double amplitude_min = 0.1*(double(rand()) / RAND_MAX);
		double amplitude_de_base_min = 0.1+0.2*(double(rand()) / RAND_MAX);
		double frequence_amplitude =(2.0+8.0*(double(rand()) / RAND_MAX))*(2*c.radius-dis) ;
		double phase_amplitude = 500*(double(rand()) / RAND_MAX);
		double t = date::horloge.get_date();
		if(dis > c.radius)
		{
			return wind_primitive::info();
		}
		else if(dis < marge_int)
		{
			//double coef = dis/marge_int;
			double amp = amplitude_de_base_max+amplitude_max*cos(t*frequence_amplitude+phase_amplitude);
			amp=(amp>1.0)?1.0:amp; amp=(amp<0.0)?0.0:amp;
			return wind_primitive::info(g_blastPower*power_coef,
				amp*tmp.x,
				amp*tmp.y);
		}
		else if(dis > c.radius-marge_ext)
		{
			double coef = (dis+marge_ext-c.radius)/(marge_ext);
			double amp = amplitude_de_base_min+amplitude_min*cos(t*frequence_amplitude+phase_amplitude);
			amp=(amp>1.0)?1.0:amp; amp=(amp<0.0)?0.0:amp;
			return wind_primitive::info(g_blastPower*(1-coef)*power_coef,
				amp*tmp.x,
				amp*tmp.y);
		}
		else
		{
			double coef = (dis-marge_int)/(c.radius-marge_int-marge_ext);
			double amp = 
				(coef*amplitude_de_base_max+(1-coef)*amplitude_de_base_min)+
				(coef*amplitude_max+(1-coef)*amplitude_min)*cos(t*frequence_amplitude+phase_amplitude);
			amp=(amp>1.0)?1.0:amp; amp=(amp<0.0)?0.0:amp;
			return wind_primitive::info(g_blastPower*power_coef,
				amp*tmp.x,
				amp*tmp.y);
		}
	}
	
	//! the default constructor
	/*! 
	*/
	blast_of_air(circle* f,birth<blast_of_air>* b,death<blast_of_air>* d);
	
	//! the copy constructor
	/*! 
	\param obj Object you want to copy.
	*/
	blast_of_air(const blast_of_air& obj);
	
	//! the assignement constructor
	/*! 
	\param obj Object you want to assign.
	*/
	blast_of_air& operator=(const blast_of_air& obj);
	
	//! the destructor
	/*! 
	destroy all the object attributs.
	*/
	virtual ~blast_of_air();

	inline std::ostream& write(std::ostream& os) const
	{
		os<<debug_frank::tab<<"blast_of_air:\n";
		os<<*(this->my_life)<<std::endl;
		return os;
	}		

	inline friend std::ostream& operator<<(std::ostream& os, const blast_of_air& obj)
	{
		debug_frank::process_tab();
		obj.primitive::write(os);
		obj.wind_primitive::write(os);
		obj.blast_of_air::write(os);
		debug_frank::unprocess_tab();
		return os;
	}

	// end of methods of the class
	/////////////////////////////////
};

#endif /* _blast_of_air_ */
