/* 
		breeze: specification

		Author: Frank Perbet

		Versions:
 			- 
*/


/* 
	Comments:

*/

/*************************************************************************/

#ifndef _breeze_
#define _breeze_

#include <iostream>
#include "wind_primitive.h"
#include "circle.h"
#include "geometrie.h"
#include "figure2d.h"
#include "life.h"
#include "GL/glut.h"

//! Class breeze
/*!
MORE COMMENT ABOUT THE CLASS
*/

class breeze : public wind_primitive
{
public:

	/////////////////////////////////
	// atributs of the class:

	//! life of the gust of wind prim
	life<breeze>* my_life;


	// end of atributs of the class.
	/////////////////////////////////
	
	
	/////////////////////////////////
	// methods of the class

	inline virtual void ogl_draw() const
	{
	
		glPushAttrib(GL_LIGHTING_BIT);
		glPushAttrib(GL_TEXTURE_BIT);
		
		glDisable(GL_LIGHTING);
		glDisable(GL_TEXTURE_2D);
		
		glColor3f(0.8*power_coef,0.3*power_coef,0.3*power_coef);
		pt_fig->ogl_draw();
		
		glPopAttrib();
		glPopAttrib();
	}

	//! add a slice of life
	inline void add_slice_of_life(slice_of_life<breeze>* s)
	{
		this->my_life->add_slice_of_life(s);
	}

	//! make the primitive live
	inline virtual bool up_to_date()
	{
		my_life->up_to_date();
		my_life->action();
		
		pt_fig->r.origin=geometrie::point2d(0.0,0.0);
		
		return !am_i_dead;
	}

	//! return the information emitted to the point p
	inline virtual info give_info(geometrie::point2d p)
	{
		double sqdis = (p-pt_fig->r.origin).norme_carre();
		static circle* c=static_cast<circle*>(pt_fig);
		
		if( (c->radius*c->radius) > sqdis)
		{
			//long seed = static_cast<long>(p.y*1000+p.x);
			//srand(seed); //linux srand48
			double t = date::horloge.get_date();
			
			double angle_de_base = 6.28*(double(rand()) / RAND_MAX);
			double amplitude_de_base = 0.4*(double(rand()) / RAND_MAX);
			double amplitude = 0.2+0.5*(double(rand()) / RAND_MAX);
			double frequence_amplitude = 0.3*(double(rand()) / RAND_MAX);
			double phase_amplitude = 100.0*(double(rand()) / RAND_MAX);
			
			double coef = cos(frequence_amplitude*t+phase_amplitude);
			
			return wind_primitive::info(wind_coef,
				(coef*amplitude+amplitude_de_base)*cos(angle_de_base),
				(coef*amplitude+amplitude_de_base)*sin(angle_de_base));
		}
		else
		{
			return wind_primitive::info();
		}
	}
	
	//! the default constructor
	/*! 
	*/
	breeze(figure2d* f,birth<breeze>* b,death<breeze>* d);
	
	//! the copy constructor
	/*! 
	\param obj Object you want to copy.
	*/
	breeze(const breeze& obj);
	
	//! the assignement constructor
	/*! 
	\param obj Object you want to assign.
	*/
	breeze& operator=(const breeze& obj);
	
	//! the destructor
	/*! 
	destroy all the object attributs.
	*/
	virtual ~breeze();

	inline std::ostream& write(std::ostream& os) const
	{
		os<<debug_frank::tab<<"breeze:\n";
		os<<*(this->my_life)<<std::endl;
		return os;
	}		

	inline friend std::ostream& operator<<(std::ostream& os, const breeze& obj)
	{
		debug_frank::process_tab();
		obj.primitive::write(os);
		obj.wind_primitive::write(os);
		obj.breeze::write(os);
		debug_frank::unprocess_tab();
		return os;
	}

	// end of methods of the class
	/////////////////////////////////
};

#endif /* _breeze_ */
