/* 
		father_primitive: corps de la classe

		Author: Frank Perbet

		Versions:
 			- [DATE]: (initial version) 
*/

/*************************************************************************/
#include "debug.h"
#include "date.h"
#include <typeinfo>

//! return the information to the point2d
template<class T> typename father_primitive<T>::ty_info father_primitive<T>::give_info(geometrie::point2d p,ty_info res_inf)
{
	// chaque carre: liste des prim active
	// envoyer en param de give_invo (ci-dessus) liste prec
	std::list<ty_info> list_info;
typename	std::list<T*>::iterator it=alive_primitive.begin();
typename	std::list<T*>::iterator end=alive_primitive.end();

//	std::cerr<<typeid(*this).name()<<std::endl;
//	std::cerr<<"information par defaut: "<<res_inf<<std::endl;
	while (it!=end)
	{
		const typename father_primitive<T>::ty_info& ii = (*it)->give_info(p);
//		std::cerr<<ii<<std::endl;
		res_inf+=ii;
		
		
		it++;
	}
	
	res_inf.normalize();
	
//	std::cerr<<"father send: "<<res_inf<<std::endl<<std::endl;
	
	return res_inf;
};

//! add a primitive in the not_born_primitive list
template<class T> void father_primitive<T>::add_not_born_primitive(T* p)
{
	assert( p->date_of_birth >= date::horloge );

typename	std::list<T*>::iterator it=not_born_primitive.begin();
typename	std::list<T*>::iterator end=not_born_primitive.end();
	
	while ((it!=end)&&((*it)->date_of_birth<p->date_of_birth))
	{
		it++;
	}
	not_born_primitive.insert(it,p);
};

//! add a primitive in the alive_primitive list
template<class T> void father_primitive<T>::add_alive_primitive(T* p)
{
	assert( (p->date_of_birth<=date::horloge)&&(p->date_of_death>=date::horloge) );

typename	std::list<T*>::iterator it=alive_primitive.begin();
typename	std::list<T*>::iterator end=alive_primitive.end();
	
	while ((it!=end)&&((*it)->date_of_death<p->date_of_death))
	{
		it++;
	}
	alive_primitive.insert(it,p);	
};

//! add a primitive in the dead_primitive list
template<class T> void father_primitive<T>::add_dead_primitive(T* p)
{
	assert( p->date_of_death<=date::horloge );
	
	dead_primitive.push_front(p);		
};

//! update primitive
template<class T> void father_primitive<T>::add_primitive(T* p)
{
	p->up_to_date();
	
	if (p->am_i_dead)
	{
		add_dead_primitive(p);
	}
	else if (p->have_i_born)
	{
		add_alive_primitive(p);
	}
	else
	{
		add_not_born_primitive(p);
	}
};


template<class T> void father_primitive<T>::clean_all()
{
	
typename	std::list<T*>::iterator it;
typename	std::list<T*>::iterator end;
	
	it=not_born_primitive.begin();
	end=not_born_primitive.end();
	
	while(it!=end)
	{
		delete (*it);
		it++;
	}

	it=alive_primitive.begin();
	end=alive_primitive.end();
	
	while(it!=end)
	{
		delete (*it);
		it++;
	}
	
	it=dead_primitive.begin();
	end=dead_primitive.end();
	
	while(it!=end)
	{
		delete (*it);
		it++;
	}
	
	 not_born_primitive.clear();
	 alive_primitive.clear();
	 dead_primitive.clear();
}

//! update primitive
template<class T> void father_primitive<T>::clean_dead_primitive()
{
typename	std::list<T*>::iterator it=dead_primitive.begin();
typename	std::list<T*>::iterator end=dead_primitive.end();
	
	while(it!=end)
	{
		delete (*it);
		it++;
	}
};

//! update primitive
template<class T> void father_primitive<T>::up_to_date()
{
typename	std::list<T*>::iterator it=not_born_primitive.begin();
typename	std::list<T*>::iterator end=not_born_primitive.end();
	
	while(it!=end)
	{
		(*it)->up_to_date();
		if((*it)->have_i_born)
		{
			add_alive_primitive(*it);
			it=not_born_primitive.erase(it);
		}
		else
		{
			it++;
		}
	}

	it=alive_primitive.begin();
	end=alive_primitive.end();	
	while((it!=end)&&(date::horloge>(*it)->date_of_birth))
	{
		(*it)->up_to_date();
		if((*it)->am_i_dead)
		{
			add_dead_primitive(*it);
			it=alive_primitive.erase(it);
		}
		else
		{
			it++;
		}
	}
	
// 	this->clean_dead_primitive();
};

// the default constructor
template<class T> father_primitive<T>::father_primitive()
{
};

// the copy constructor
template<class T> father_primitive<T>::father_primitive(const father_primitive& obj)
{
	debug_frank::error("No copy constructor implemented in the class father_primitive.\n",1);
};

// the assignement constructor:
template<class T> father_primitive<T>& father_primitive<T>::operator=(const father_primitive<T>& obj)
{
	if (this != &obj)
	{
		debug_frank::error("No assignement implemented in the class father_primitive.\n",1);
	}
	return *this;
};

// the destructor: 
template<class T> father_primitive<T>::~father_primitive()
{
typename	std::list<T*>::iterator it=not_born_primitive.begin();
typename	std::list<T*>::iterator end=not_born_primitive.end();
	
	while(it!=end)
	{
		delete (*it);
		it++;
	}
	
	it=alive_primitive.begin();
	end=alive_primitive.end();
	
	while(it!=end)
	{
		delete (*it);
		it++;
	}
	
	clean_dead_primitive();
	
	not_born_primitive.clear();
	alive_primitive.clear();
	dead_primitive.clear();
};

// display
template<class T> std::ostream& father_primitive<T>::display(std::ostream& os) const
{
	debug_frank::process_tab();
	os<<debug_frank::tab<<typeid(*this).name()<<":\n";
	os<<debug_frank::tab<<"list of not born primitives:\n";
typename	std::list<T*>::const_iterator it=not_born_primitive.begin();
typename	std::list<T*>::const_iterator end=not_born_primitive.end();
	
	while(it!=end)
	{
		os<<*(*it);
		it++;
	}
	
	os<<debug_frank::tab<<"list of alive primitives:\n";
	it=alive_primitive.begin();
	end=alive_primitive.end();
	
	while(it!=end)
	{
		os<<*(*it);
		it++;
	}
	
	os<<debug_frank::tab<<"list of dead primitives:\n";
	it=dead_primitive.begin();
	end=dead_primitive.end();
	
	while(it!=end)
	{
		os<<*(*it);
		it++;
	}
	
	debug_frank::unprocess_tab();
	return os;
};
