/* 
		geom: specification

		Auteur: Frank Perbet

		Version:
 			- 29/03/2000 (version initiale) 
*/


/* 
	COMMENTAIRES: Dans cette librairie sont presentes les classes:
		- point2d
		- point3d
		- point4d
		- coord
		
	Attention, cette classe n'est pas optimisee.
	Elle ne devrait etre utilisee que pour des precalculs.

*/

/*************************************************************************/

#ifndef _geometrie_
#define _geometrie_

#include <vector>
#include <iostream>
using namespace std;
#include "math_perso.h"

namespace geometrie
{

class point2d
{
public:
	float x;
	float y;
	
	//constructeur
	inline point2d(float xv=0, float yv=0)
		:x(xv), y(yv) {};

	// constructeur par copie:
	point2d(const point2d& obj): x(obj.x),y(obj.y) {};
	
	// constructeur par rotation:
	point2d(const point2d& obj,float teta);
	
	// assignement:
	inline point2d& operator=(const point2d& obj)
	{
		if (this != &obj)
		{
			this->x=obj.x;
			this->y=obj.y;
		}
		return *this;
	}
	
	point2d& rotation(float teta);
	
	inline point2d rotation_90()
	{
		return point2d(-y,x);
	}
	
	inline point2d rotation_moins_90()
	{
		return point2d(y,-x);
	}

	// normalisation
	inline point2d normalize() { float s=this->norme(); x=x/s; y=y/s; return *this;};

	// normalisation
	inline double normalize_bis() { float s=this->norme(); x=x/s; y=y/s; return s;};
	
	// norme:
	inline float norme() {return math_perso::sqrt(x*x+y*y);};
	
	// norme_carre:
	inline float norme_carre() {return x*x+y*y;};

	// norme_carre:
	inline float operator!=(point2d a) {return ((x!=a.x)||(y!=a.y));};

	// norme_carre:
	inline float operator==(point2d a) {return ((x==a.x)&&(y==a.y));};

	// produit scalaire:
	inline friend float scalaire(point2d a,point2d b)
		{return a.x*b.x+a.y*b.y; };
		
	// produit scalaire:
	inline friend float operator*(point2d a,point2d b)
		{return a.x*b.x+a.y*b.y; };
	
	// produit vectoriel:
	inline friend point2d vectoriel(point2d a)
		{return point2d(-a.y, a.x); };
	
	// multiplication entre point2d et float
	inline friend point2d operator*(const point2d& p, float a )
		{return point2d(p.x*a, p.y*a); };
	inline friend point2d operator*(float a, const point2d& p )
		{return point2d(p.x*a, p.y*a); };
		
	// multiplication entre point2d et float
	inline friend point2d operator/(const point2d& p, float a )
		{return point2d(p.x/a, p.y/a); };
	
	// multiplication entre point2d et double
	inline friend point2d operator*(const point2d& p, double a )
		{return point2d(p.x*a, p.y*a); };
	inline friend point2d operator*(double a,const point2d& p)
		{return point2d(p.x*a, p.y*a); };
	
	// multiplication entre point2d et entier
	inline friend point2d operator*(const point2d& p, int a )
		{return point2d(p.x*a, p.y*a); };
	inline friend point2d operator*(int a, const point2d& p )
		{return point2d(p.x*a, p.y*a); };

	
	// addition entre point2d et scalaire
	inline friend point2d operator+(const point2d& pa, const point2d& pb)
		{ return point2d(pa.x+pb.x, pa.y+pb.y); }
	
	// addition entre point2d et scalaire
	inline friend point2d linear_interpolation(const point2d& pa, const point2d& pb,float f)
		{ return pa*f+pb*(1-f); }
	
	// soustraction entre point2d et scalaire
	inline friend point2d operator-(const point2d& pa, const point2d& pb)
		{ return point2d(pa.x-pb.x, pa.y-pb.y); }
		
	inline friend float distance_max(const point2d& pa, const point2d& pb)
		{
			float xx=math_perso::fabs(pa.x-pb.x); float yy = math_perso::fabs(pa.y-pb.y);
			return xx<yy ? yy : xx;
		}
		
	inline friend float distance(const point2d& pa, const point2d& pb)
	{
		return math_perso::sqrt( (pa.x-pb.x)*(pa.x-pb.x)+(pa.y-pb.y)*(pa.y-pb.y) );
	}	
	
	inline friend float distance2(const point2d& pa, const point2d& pb)
	{
		return (pa.x-pb.x)*(pa.x-pb.x)+(pa.y-pb.y)*(pa.y-pb.y);
	}	
	
	// lit un point
	inline friend std::istream& operator>>(std::istream& is, point2d& obj)
		{ is>>obj.x>>obj.y; return is; };
	 
	 // affiche un point
	inline friend std::ostream& operator<<(std::ostream& os, const point2d& obj)
		{ os<<"("<<obj.x<<","<<obj.y<<")"; return os; };

	// Calcul de la hauteur par interpolation:
	inline float give_angle() const
	{
		float teta=math_perso::acos(x);
		return (y>=0.0) ? teta : (-teta+2*math_perso::PI);
	};
};
	
class point3d
{
public:
	float x;
	float y;
	float z;
	
	// constructeur
	point3d(float cx=0,float cy=0,float cz=0) { x=cx; y=cy; z=cz;};
	
	// normalisation
	void normalize() { float s=this->norme(); x=x/s; y=y/s; z=z/s; };
	
	// norme:
	float norme();
	
	// norme_carre:
	float norme_carre();
	
	// produit scalaire:
	friend float scalaire(point3d a,point3d b)
		{return a.x*b.x+a.y*b.y+a.z*b.z; };
	
	// produit vectoriel:
	friend point3d vectoriel(point3d a,point3d b)
		{return point3d(a.y*b.z-a.z*b.y, a.z*b.x-a.x*b.z, a.x*b.y-a.y*b.x); };
	
	// multiplication entre point3d et float
	friend point3d operator*(const point3d& p, float a )
		{return point3d(p.x*a, p.y*a ,p.z*a); };
	friend point3d operator*(float a, const point3d& p )
		{return point3d(p.x*a, p.y*a ,p.z*a); };
	
	// multiplication entre point3d et double
	friend point3d operator*(const point3d& p, double a )
		{return point3d(p.x*a, p.y*a ,p.z*a); };
	friend point3d operator*(double a,const point3d& p)
		{return point3d(p.x*a, p.y*a ,p.z*a); };
	
	// multiplication entre point3d et entier
	friend point3d operator*(const point3d& p, int a )
		{return point3d(p.x*a, p.y*a ,p.z*a); };
	friend point3d operator*(int a, const point3d& p )
		{return point3d(p.x*a, p.y*a ,p.z*a); };
	
	friend float distance_max(const point3d& pa, const point3d& pb)
		{
			float xx=math_perso::fabs(pa.x-pb.x); float yy = math_perso::fabs(pa.y-pb.y);
			return xx<yy ? yy : xx;
		}
	
	// addition entre point3d et scalaire
	friend point3d operator+(const point3d& pa, const point3d& pb)
		{ return point3d(pa.x+pb.x, pa.y+pb.y, pa.z+pb.z); }
	
	// soustraction entre point3d et scalaire
	friend point3d operator-(const point3d& pa, const point3d& pb)
		{ return point3d(pa.x-pb.x, pa.y-pb.y, pa.z-pb.z); }
	
	// lit un point
	friend std::istream& operator>>(std::istream& is, point3d& obj)
		{ is>>obj.x>>obj.y>>obj.z; return is; };
	 
	 // affiche un point
	friend std::ostream& operator<<(std::ostream& os, const point3d& obj)
		{ os<<"point3d: x= "<<obj.x<<" y= "<<obj.y<<" z= "<<obj.z; return os; };
		
	// constructeur par copie:
	point3d(const point3d& obj);

	// operateur de copie:
	point3d& operator=(const point3d& obj);
};
	
class point4d
{
public:
	float a;
	float b;
	float c;
	float d;
	
	// constructeur
	point4d(float xa=0,float xb=0,float xc=0,float xd=0) { a=xa; b=xb; c=xc; d=xd; };
	
	// normalisation "lineaire"
	void normalize() { float s=this->somme(); a=a/s; b=b/s; c=c/s; d=d/s; };
	
	// renvoie la somme des composants:
	float somme() const { return a+b+c+d; };
		
	// lit un point
	//friend std::istream& operator>>(std::istream& is, point4d& obj)
	//	{ is>>"p4d:">>"a=">>obj.a>>"b=">>obj.b>>"c=">>obj.c>>"d=">>obj.d; return is; };
	 
	 // affiche un point
	friend std::ostream& operator<<(std::ostream& os, const point4d& obj)
		{ os<<"p4d: a= "<<obj.a<<" b= "<<obj.b<<" c= "<<obj.c<<" d= "<<obj.d; return os; };
		
	// constructeur par copie:
	point4d(const point4d& obj);

	// operateur de copie:
	point4d& operator=(const point4d& obj);
};

class coord
{
public:
	int i;
	int j;

	// affiche des coordonnees:
	friend std::ostream& operator<<(std::ostream& os, const coord& obj)
		{ os<<"coord: i= "<<obj.i<<" j= "<<obj.j; return os; }
	
	// constructeur
	coord(int ii=0, int jj=0) : i(ii), j(jj) {};
	
	// constructeur
	coord(float ii, float jj) : i( int(ii) ), j( int(jj) ) {};
	
	// constructeur par copie:
	coord(const coord& obj)
		{ this->i=obj.i; this->j=obj.j; }
	
	// operateur de copie:
	coord& operator=(const coord& obj)
		{ if (this != &obj) { this->i=obj.i; this->j=obj.j; } return *this; }
		
// 	friend int distance_discrete(coord a,coord b)
// 		{ return abs(a.i-b.i)+abs(a.j-b.j); }


	friend int distance_discrete(coord a,coord b)
		{ return (int) math_perso::sqrt((a.i-b.i)*(a.i-b.i)+(a.j-b.j)*(a.j-b.j)); }

// 	friend int distance_discrete(coord a,coord b)
// 	{
// 		int x=fabs(a.i-b.i); int y=fabs(a.j-b.j);
// 		return x<y ? y : x;
// 	}
};

// test l'appartenance d'un point a un demi-plan:
// ( point de la droite, normale, point a tester)
bool appartient(point2d,point2d,point2d);
bool appartient(point3d,point2d,point3d);

// test l'appartenance d'un point a un demi-plan (a:point a tester):
// point c sur le plan, b vecteur normal au plan
// true si a n'est pas du meme cote que b
inline bool belong_to_halfplane(const point2d a,const point2d b,const point2d c)
{
	return ( b.x*(c.x-a.x) + b.y*(c.y-a.y) )>0;
};

// Calcul de la hauteur par interpolation:
inline float interpolation4(float pr_x, float pr_y ,float az ,float bz ,float cz, float dz)
{
	float m1=(1-pr_y)*az+(pr_y)*bz;
	float m2=(1-pr_y)*dz+(pr_y)*cz;
	float res=(1-pr_x)*m1+pr_x*m2;
	
// 	cout<<"coord. relative: pr_x="<<pr_x<<"  pr_y="<<pr_y<<"\n";
// 	cout<<"az="<<az<<" bz="<<bz<<" cz="<<cz<<" dz="<<dz<<"\n";
// 	cout<<"interp: m1="<<m1<<"  m2="<<m2<<"  res="<<res<<"\n";

 	return res;
}

// donne l'angle / l'axe x
inline float give_angle(point2d p)
{
	float teta=math_perso::acos(p.x);
	if (p.y>=0.0)
	{
		return teta;
	}
	else
	{
		return -teta+2*math_perso::PI;
	}
};

//! a direct repere in 2d
class repere2d
{
public:
	//! origin of the repere
	point2d origin;
	
	//! angle en radian
	float angle;

	//! return the x base vector
	inline point2d give_base_x() const
	{ return point2d(cos(angle),sin(angle)); }

	//! return the y base vector
	inline point2d give_base_y() const
	{ return point2d(sin(angle),-cos(angle)); }

	//! return the y base vector
	inline point2d to_this_repere(point2d p) const
	{ return point2d(p-origin,-angle); }
	
	//! return the y base vector
	inline point2d to_base_repere(point2d p) const
	{ return point2d(p,angle)+origin; }

	//! constructor
	inline repere2d(point2d oorigin,float aangle=0.0)
	:origin(oorigin), angle(aangle)
	{ assert((angle>=0)&&(angle<=2*math_perso::PI)); };
	
	//! constructor
	inline repere2d()
	:origin(point2d(0,0)), angle(0) {};
	
	//! copy
	inline repere2d(const repere2d& obj)
	{ origin=obj.origin; angle=obj.angle;}
	
	//! assignement
	inline repere2d& operator=(const repere2d& obj)
	{ if(this!=&obj) {origin=obj.origin; angle=obj.angle;} return *this;}
	
	//! destructor
	inline ~repere2d() {};
	
	inline friend std::ostream& operator<<(std::ostream& os, const repere2d& obj)
	{
		os<<"origin="<<obj.origin<<" angle="<<obj.angle;
		return os;
	}
};

inline double potentiel(const point2d	&p,
		     const point2d	&v0,
		     const point2d	&v1)
{
	const point2d  tmp2 = v1 - v0;
	const point2d  r = (v0 + ((p - v0) * tmp2) * tmp2) / distance2(v0,v1);
	const double   cos0 = sqrt(distance2(v0,r) / distance2(v0,p));
	const double   cos1 = sqrt(distance2(v1,r) / distance2(v1,p));
	const double   h2 = distance2(p,r);
	const double   tmp = r * tmp2;

	if (tmp < v0 * tmp2)
		return 0.5 * (cos1 - cos0) / h2;

	if (v1 * tmp2 < tmp)
		return  0.5 * (cos0 - cos1) / h2;
	
	return 0.5 * (cos0 + cos1) / h2;
}

};
#endif /* _geometrie_ */
