/* 
		gust_field: corps de la classe

		author: Frank Perbet

		Versions:
 			- [DATE]: (initial version) 
*/

/*************************************************************************/
#include "gust_field.h"

#include "debug.h"
#include "math_perso.h"

//! retun the value of the function (f between 0 and 1)
float gust_field::operator[](float f) const
{
	int index=math_perso::floor(f)*vector_of_values.size();
	float interp=f-index;

	// exception:
	assert((interp<0.0) || (interp>1.0));
	assert(index<0);
	assert(index>= vector_of_values.size());

	if (index==vector_of_values.size()-1)
	{
		return vector_of_values[index];
	}
	else
	{
		return vector_of_values[index]*(1-interp)+vector_of_values[index+1]*interp;
	}
}

// the default constructor
gust_field::gust_field(float nb_oscillation,float attenuation,int nb_discretiation) throw(std::invalid_argument)
{
	// exception:
	if (nb_oscillation<1)
	{
		throw std::invalid_argument("gust_field: constructor: not enough oscillation");
	}
	if(attenuation<0)
	{
		throw std::invalid_argument("gust_field: constructor: negative attenuation");
	}
	if(nb_discretiation<2)
	{
		throw std::invalid_argument("gust_field: constructor: not enough discretisation point");
	}
	
	float omega=2*math_perso::PI*nb_discretiation;
	
	vector_of_values.reserve(nb_discretiation);
	int i;
	for(i=0;i<nb_discretiation;i++)
	{
		float k=static_cast<float>(i)/nb_discretiation;
		vector_of_values.push_back(math_perso::sin(omega*k)/(1+attenuation*k));
	}
}

// the copy constructor
gust_field::gust_field(const gust_field& obj)
{
	debug_frank::error("No copy constructor implemented in the class gust_field.\n",1);
}

// the assignement constructor:
gust_field& gust_field::operator=(const gust_field& obj)
{
	if (this != &obj)
	{
		debug_frank::error("No assignement implemented in the class gust_field.\n",1);
	}
	return *this;
}

// the destructor: 
gust_field::~gust_field()
{
	debug_frank::error("No destructor implemented in the class gust_field.\n",1);
}

// display
std::ostream& operator<<(std::ostream& os, const gust_field& obj)
{
	os<<"No display implemented in the class gust_field.\n";
	return os;
}
