/* 
		life: corps de la classe

		Author: Frank Perbet

		Versions:
 			- [DATE]: (initial version) 
*/

/*************************************************************************/
#include "debug.h"

//! add a slice of life to active_event
template<class T> void life<T>::add_active_event(slice_of_life<T>* sol)
throw(std::invalid_argument)
{
	if(date::horloge<sol->start)
	{
		throw(std::invalid_argument("life.cpp:add_active_event: date conflict."));
	}
	
	ty_list_iterator it=list_of_active_events.begin();
	while((it!=list_of_active_events.end())&&(sol->end>(*it)->end))
	{
		it++;
	}
	
	sol->activate();
	list_of_active_events.insert(it,sol);
}	

//! add a slice of life to life
template<class T> void life<T>::add_slice_of_life(slice_of_life<T>* sol)
throw(std::invalid_argument)
{
	if(date::horloge>sol->start)
	{
		throw(std::invalid_argument("life.cpp:add_slice_of_life: date conflict."));
	}
	
	ty_list_iterator it=next_event;
	ty_list_iterator end=list_of_events.end();
	while( (it!=end) && (sol->start>(*it)->start) )
	{
		it++;
	}
	
	if (next_event==it)
	{
		next_event=list_of_events.insert(it,sol);
	}
	else
	{
		list_of_events.insert(it,sol);
	}
}

//! add a slice of life to life
template<class T> void life<T>::up_to_date()
throw(std::runtime_error)
{
	// check first event
	ty_list_iterator it=next_event;
	if(next_event!=list_of_events.begin())
	{
		it--;
		if((*it)->start>date::horloge)
		{
		throw(std::invalid_argument("life.cpp:up_to_date: date conflict."));	
		}
		it++;
	}
	
	// add new active event
	while((it!=list_of_events.end())&&((*it)->start<=date::horloge))
	{
		add_active_event(*it); it++;
	}
	next_event=it;
	
	// deactivate active event
	it=list_of_active_events.begin();
	while( (it!=list_of_active_events.end()) && ((*it)->end<=date::horloge) )
	{
		(*it)->deactivate();
		list_of_active_events.pop_front();
		it=list_of_active_events.begin();
	}	
	
}

//! action active event
template<class T> void life<T>::action()
{
	ty_list_iterator be=list_of_active_events.begin();
	ty_list_iterator en=list_of_active_events.end();
	while(be!=en) { (*be)->action(); be++;}
}

// the default constructor
template<class T> life<T>::life(birth<T>* b,death<T>* d)
:list_of_events(std::list< slice_of_life<T> * >())
,list_of_active_events(std::list< slice_of_life<T> * >())
{
	std::cerr<<"construction d'une vie: de "<<b->start<<" a "<<d->end<<"\n";
	assert(d->start>b->end);
	
	next_event=list_of_events.begin();
	add_slice_of_life(b);
	add_slice_of_life(d);
}

// the copy constructor
template<class T> life<T>::life(const life<T>& obj)
{
	debug_frank::error("No copy constructor implemented in the class life.\n",1);
}

// the assignement constructor:
template<class T> life<T>& life<T>::operator=(const life<T>& obj)
{
	if (this != &obj)
	{
		debug_frank::error("No assignement implemented in the class life.\n",1);
	}
	return *this;
}

// the destructor: 
template<class T> life<T>::~life()
{
	ty_list_iterator it=list_of_events.begin();
	ty_list_iterator en=list_of_events.end();
	while(it!=en) { delete *it; it++;}
	
	list_of_events.clear();
	list_of_active_events.clear();
}
