/* 
		straight_path: corps de la classe

		author: Frank Perbet

		Versions:
 			- [DATE]: (initial version) 
*/

/*************************************************************************/
#include "straight_path.h"

#include "debug.h"

//! give the tangent vector of the path at the f parameter.
float straight_path::give_tangent_vector_angle(float f) const
{
	return angle;
}

//! give the tangent vector of the path at the f parameter.
geometrie::point2d straight_path::give_tangent_vector(float f) const
{
	return direction;
}

//! give the tangent vector of the path at the f parameter.
geometrie::point2d straight_path::give_normal_vector(float f) const
{
	return geometrie::point2d(direction.y,-direction.x);
}

//! give the tangent vector of the path at the f parameter.
geometrie::point2d straight_path::give_position(float f) const
{
	return start+direction*f;
}

// the default constructor
straight_path::straight_path(geometrie::point2d s,geometrie::point2d d)
:start(s),direction(d),angle(d.give_angle())
{
	direction.normalize();
//	std::cerr<<angle<<"\n";
	assert( ((angle>=0.0) && (angle<=2.0*math_perso::PI)));
}

//! the default constructor
/*! 
*/
straight_path::straight_path(geometrie::point2d s,float aangle)
:start(s),direction(geometrie::point2d(math_perso::cos(aangle),math_perso::sin(aangle))),angle(aangle)
{
	 assert( ((angle>=0.0) && (angle<=2.0*math_perso::PI)));	
}

// the copy constructor
straight_path::straight_path(const straight_path& obj)
{
	debug_frank::error("No copy constructor implemented in the class straight_path.\n",1);
}

// the assignement constructor:
straight_path& straight_path::operator=(const straight_path& obj)
{
	if (this != &obj)
	{
		debug_frank::error("No assignement implemented in the class straight_path.\n",1);
	}
	return *this;
}

// the destructor: 
straight_path::~straight_path()
{
}

// display
std::ostream& operator<<(std::ostream& os, const straight_path& obj)
{
	debug_frank::process_tab();
	os<<debug_frank::tab<<"straight_path: origin="<<obj.start<<" direction="<<obj.direction<<std::endl;
	debug_frank::unprocess_tab();
	return os;
}
