
#include "MyPrairie.h"
#include "Rock.h"
#include "PrairieDefaultDirectoriesAndFiles.h"

#include "GroundInterface.h"
#include "WindInterface.h"
//#include "../libForest/ForestInterface.h"
#include "GrassInterface.h"

#include "gl.h"//#include <GL/gl.h>
#include "glu.h"//#include <GL/glu.h>
#include "glut.h"//#include <GL/glut.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <iostream>

//calcul processeur occupation
#include <sys/time.h>
//la fonction  gettimeofday(struct timeval * tv , struct timezone * tz ); n'existe pas sous win 
#ifdef WIN32 
#include <time.h>
#include <sys/timeb.h> 
int gettimeofday (struct timeval *tp, void *tz)
{   
 struct _timeb timebuffer;  
 _ftime (&timebuffer);  
 tp->tv_sec = timebuffer.time;  
 tp->tv_usec = timebuffer.millitm * 1000;  
 return 0;
 }
 #endif
// fin de l ajout de la fonction gettimeofday

timeval g_timeLastFrame;
timeval g_timeFrame;
timeval g_timeAfterGrassUpdate;
//---

//anim
bool gb_animation=false;
float g_dt=0.05;
bool gb_updateGrassDrawing=true;
bool gb_updateGrassDrawingOneTime=false;

//draw
bool gb_drawGround=true;
bool gb_drawTrees=false;
bool gb_drawGust=false;
bool gb_drawGrass=false;
bool gb_drawGrassNodes=false;
bool gb_drawGrassDensity=false;
bool gb_drawSky=false;
bool gb_drawWire=false;
bool gb_drawTexture2D5=false;
bool gb_drawTexture2D=false;
//rock
float g_rockSpeed=1.0;
Rock g_rock;

//-----------------------------------------------------------------------
//--------------fonctions definient dans les librairies------------------
//-----------------------------------------------------------------------
/*
void forestGetLocalWind(const float& x, const float& y,
		  	float& windPower, 
		  	float& windDirX, 
		  	float& windDirY, 
		  	float& windDirZ)
{
windGetLocalInfluence(x,y,0.0f,0.0f,0.0f,windPower,windDirX,windDirY);
windDirZ=0.0f;
windPower*=0.001f;
}

float forestGetHeightAtPosition(const float& x, const float& y)
{
return groundGetHeightAtPosition(x,y);
}
*/
void grassGetLocalWind(const float& x, const float& y,
                       const float& windInitPower,
		       const float& windInitDirX,
		       const float& windInitDirY,
		       float& windPower, 
		       float& windDirX, 
		       float& windDirY)
{
windGetLocalInfluence(x,y,windInitPower,windInitDirX,windInitDirY,
			 windPower,windDirX,windDirY);
}

float grassGetHeightAtPosition(const float& x, const float& y)
{
return groundGetHeightAtPosition(x,y);
}

//-----------------------------------------------------------------------
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
//----------------------fonctions locales--------------------------------
//-----------------------------------------------------------------------

void drawSky()
{
const float size=100.0;

glBegin(GL_TRIANGLE_FAN);
glColor3f(0.3,0.35,0.9);
glVertex3f(groundGetLength()*0.5,groundGetWidth()*0.5,groundGetMaxHeight()+size);

glColor3f(0.8,0.85,1.0);
glVertex3f(-size,-size,-1.0);
glVertex3f(groundGetLength()+size,-size,-1.0);
glVertex3f(groundGetLength()+size,groundGetWidth()+size,-1.0);
glVertex3f(-size,groundGetWidth()+size,-1.0);
glVertex3f(-size,-size,-1.0);
glEnd();

//base
glBegin(GL_TRIANGLE_STRIP);
glVertex3f(-size,-size,-1.0);
glVertex3f(groundGetLength()+size,-size,-1.0);
glVertex3f(-size,groundGetWidth()+size,-1.0);
glVertex3f(groundGetLength()+size,groundGetWidth()+size,-1.0);
glEnd();
}


//-----------------------------------------------------------------------
//-----------------------------------------------------------------------



//-----------------------------------------------------------------------
//----------------------fonctions principales----------------------------
//-----------------------------------------------------------------------

void prairieInit()
{
int i;
//GL
glClearColor(0.3,0.5,0.9,1.0);
glDisable(GL_COLOR_MATERIAL);

//initialisation terrain
float groundArea[]={0.0,0.0,100.0,100.0};
groundInit(groundArea, 5.0,
	   20*20,
	   GROUND_DEFAULT_HEIGHMAP,
 	   GROUND_DEFAULT_TEXTURE,
	   8.0);
windInit(groundGetArea(),groundGetMaxHeight());

//initialisation grass
grassInit(groundGetArea(),
	  60,60,
	  GRASS_DEFAULT_DENSITYMAP);
grassSetHeight(2.0);
grassSetWidth(0.02);
grassSetNumberPerSquareMeters(10);
grassSetDistance3D(0.0);
grassSetDistance2D5(0.0);
grassSetDistance2D(0.0);
grassSetPG3DGrass(3);
grassSetPG2D5Grass(1);
/*
//foret
forestInit();
forestSetDirectoryForData(FOREST_DEFAULT_DIRECTORYDATA);
forestActiveGraphicInterface(false);
forestResizeWindow(640,400,45.0f);
forestLoadARB(FOREST_DEFAULT_ARB);
ForestTree trees[10];
for (i=0;i<10;i++)
  {
  trees[i].m_position2d[0]=groundGetXMin() + groundGetLength()*(double(rand()) / RAND_MAX);//drand48();
  trees[i].m_position2d[1]=groundGetYMin() + groundGetWidth()*(double(rand()) / RAND_MAX);//drand48();
  }
forestSetTrees(10,trees);
forestRandomTreesForms();
*/
//rocher
float rockPosition[]={groundGetXMin() + groundGetLength()*0.5,
		      groundGetYMin() + groundGetWidth()*0.5};
g_rock.init(rockPosition);
}


void prairieDraw(float* cameraPosition, float* cameraVecteurZ)
{
//animation
if (gb_animation) 
	{
	g_rock.integrateTime(cameraVecteurZ,g_dt*g_rockSpeed);
	windUpdate(g_dt);
	}

//affichage
glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
if (gb_drawSky) drawSky();
	
//affichage du terrain	
static GLfloat LightAmbient[]=         { 1.0f, 1.0f, 1.0f, 1.0f };
static GLfloat LightDiffuse[]=         { 1.0f, 1.0f, 1.0f, 1.0f };
static GLfloat LightPosition[]=        { 250.0f, 50.0f, 50.0f, 1.0f };
static GLfloat LightNoSpecular[]=      { 0.0f, 0.0f, 0.0f, 1.0f };
glLightfv(GL_LIGHT0, GL_AMBIENT, LightAmbient);         
glLightfv(GL_LIGHT0, GL_DIFFUSE, LightDiffuse); 
glLightfv(GL_LIGHT0, GL_SPECULAR, LightNoSpecular);
glLightfv(GL_LIGHT0, GL_POSITION, LightPosition);
glEnable(GL_LIGHTING);
if (gb_drawGround) groundDraw();

//affichage rock
g_rock.draw();

//affichage grass

//light
static GLfloat LightSpecular[]=      { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat LightPositionGrassAndTrees[]={ groundGetLength()*0.5,
				       groundGetWidth()*0.5,
				       1000.0f, 1.0f };
glLightfv(GL_LIGHT0, GL_SPECULAR, LightSpecular);
glLightfv(GL_LIGHT0, GL_POSITION, LightPositionGrassAndTrees);

g_timeLastFrame=g_timeFrame;
gettimeofday(&g_timeFrame,NULL);

//forest  
if (gb_drawTrees)
   {
//   forestDrawTrees(cameraPosition,cameraVecteurZ);
   }

if (gb_drawGrass)
   {
   if (gb_animation) grassUpdate(g_dt,g_rock.m_pTrajectory, 1);
   if (!gb_updateGrassDrawing)
   	{
	if (gb_updateGrassDrawingOneTime) grassPreDraw(cameraPosition);
	gb_updateGrassDrawingOneTime=false;
	}
   else grassPreDraw(cameraPosition);
 gettimeofday(&g_timeAfterGrassUpdate,NULL);
   if (gb_drawWire) grassDrawInWireMode();
   else grassDraw();
   if (gb_drawGrassNodes) grassDrawNoeuds();
   if (gb_drawGrassDensity) grassDrawDensity();
   //if (gb_drawTrajectory) grassDrawTrajectory();
   //if (gb_drawCrushingVector) grassDrawCrushingVector();
   }
  

//affichage wind
glDisable(GL_LIGHTING);
if (gb_drawGust) windDrawGusts();

//affichage test texture
if (gb_drawTexture2D) grassDrawTexture2D();   
if (gb_drawTexture2D5) grassDrawTexture2D5();
}

void prairieSetWindowSize(int w, int h)
{
//forestResizeWindow(w, h, 45.0);
}

void prairieQuit()
{
groundQuit();
windQuit();
}

//-----------------------------------------------------------------------
//-----------------------------------------------------------------------






//-----------------------------------------------------------------------
//----------------------fonctions secondaires----------------------------
//-----------------------------------------------------------------------

//anim
void prairieSetAnim(bool b) {gb_animation=b;}
void prairieSetAnimDT(float f) {g_dt=f;}
void prairieSetUpdateGrassDrawing(bool b) {gb_updateGrassDrawing=b;
					   gb_updateGrassDrawingOneTime=!b;}

bool prairieGetAnim() {return gb_animation;}
float prairieGetAnimDT() {return g_dt;}
bool prairieGetUpdateGrassDrawing() {return gb_updateGrassDrawing;}

//draw
void prairieSetGroundDraw(bool b){gb_drawGround=b;}
void prairieSetTreesDraw(bool b){gb_drawTrees=b;}
void prairieSetGrassDraw(bool b){gb_drawGrass=b;}
void prairieSetGrassDrawNodes(bool b){gb_drawGrassNodes=b;}
void prairieSetGrassDensityDraw(bool b){gb_drawGrassDensity=b;}
void prairieSetSkyDraw(bool b){gb_drawSky=b;}
void prairieSetGustDraw(bool b){gb_drawGust=b;}
void prairieSetTexture2DDraw(bool b) {gb_drawTexture2D=b;}
void prairieSetTexture2D5Draw(bool b) {gb_drawTexture2D5=b;}
void prairieSetWireDraw(bool b)
	{
	gb_drawWire=b;
	if (gb_drawWire) glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
	else glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
	}

bool prairieGetGroundDraw(){return gb_drawGround;}
bool prairieGetTreesDraw(){return gb_drawTrees;}
bool prairieGetGrassDraw(){return gb_drawGrass;}
bool prairieGetGrassDrawNodes(){return gb_drawGrassNodes;}
bool prairieGetGrassDensityDraw(){return gb_drawGrassDensity;}
bool prairieGetSkyDraw(){return gb_drawSky;}
bool prairieGetGustDraw(){return gb_drawGust;}
bool prairieGetTexture2DDraw() {return gb_drawTexture2D;}
bool prairieGetTexture2D5Draw() {return gb_drawTexture2D5;}
bool prairieGetWireDraw(){return gb_drawWire;}


//rock
void prairieSetRockPosition(const float position[2])
	{
	g_rock.init(position);
	}
void prairieDirectTheRock(bool b){g_rock.mvtAleatoire(!b);}
void prairieSetRockBound(bool b){g_rock.setBound(b);}
void prairieDirectTheRockUp(){g_rock.mvtUp();}
void prairieDirectTheRockDown(){g_rock.mvtDown();}
void prairieDirectTheRockLeft(){g_rock.mvtLeft();}
void prairieDirectTheRockRight(){g_rock.mvtRight();}
void prairieSetRockSpeed(float f){g_rockSpeed=f;}
void prairieSetRockSize(float f){g_rock.setSize(f);}
void prairieSetRockCrushingPower(float f){g_rock.m_pTrajectory->m_power=f;}
void prairieSetRockCrushingPowerExp(float f){g_rock.m_pTrajectory->m_controlPowerExp=f;}
void prairieSetRockCrushingMysteryParameter(float f){g_rock.m_pTrajectory->m_coefCrushingAccordingToNormal=f;}
void prairieSetRockCrushingPropagation(float f){g_rock.m_pTrajectory->m_propagationThreshold=f;}


void prairieGetRockPosition(float position[2])
	{
	position[0]=g_rock.m_position[0];
	position[1]=g_rock.m_position[1];
	}
bool prairieIsDirectTheRock(){return g_rock.mb_directTheRock;}
bool prairieGetRockBound(){return g_rock.mb_bound;}
float prairieGetRockSpeed(){return g_rockSpeed;}
float prairieGetRockSize(){return g_rock.m_size;}
float prairieGetRockCrushingPower(){return g_rock.m_pTrajectory->m_power;}
float prairieGetRockCrushingPowerExp(){return g_rock.m_pTrajectory->m_controlPowerExp;}
float prairieGetRockCrushingMysteryParameter(){return g_rock.m_pTrajectory->m_coefCrushingAccordingToNormal;}
float prairieGetRockCrushingPropagation(){return g_rock.m_pTrajectory->m_propagationThreshold;}


//return infos
int prairieGetNbPolygonsDrawedPerFrame()
{
int nbPolygons=500; //terrain+rock
if (gb_drawTrees) {
//nbPolygons+=forestGetPolygonsDrawedPerFrame();
}
if (gb_drawGrass)
nbPolygons+=grassGetPolygonsDrawedPerFrame();
return nbPolygons;
}
int prairieGetNbPolygonsDrawedPerFrameForTrees()
{
int nbPolygons=0; 
if (gb_drawTrees) {
//nbPolygons+=forestGetPolygonsDrawedPerFrame();
}
return nbPolygons;
} 
int prairieGetNbPolygonsDrawedPerFrameForGrass()
{
int nbPolygons=0; 
if (gb_drawGrass)
nbPolygons+=grassGetPolygonsDrawedPerFrame();
return nbPolygons;
}
void prairieGetInfosOnCalculations(char str[64])
{
static float occ=0.0;
if (!gb_drawGrass)
   {
   sprintf(str,"COST: UpdateGrass:0.00 ; DrawGrassesAndTrees:1.00");
   return;
   }
double timeFrame=double(g_timeFrame.tv_sec-g_timeLastFrame.tv_sec)
	   +double(g_timeFrame.tv_usec-g_timeLastFrame.tv_usec)*0.000001;
float timeUpdateGrass=double(g_timeAfterGrassUpdate.tv_sec-g_timeFrame.tv_sec)
	   +double(g_timeAfterGrassUpdate.tv_usec-g_timeFrame.tv_usec)*0.000001;
occ=timeUpdateGrass/timeFrame;
float occ_1=1.0-occ;
sprintf(str,"COST: UpdateGrass:%.2f ; DrawGrassesAndTrees:%.2f",occ,occ_1);
}



// fin modif



/*void prairieLoadGroundHeightMap(const char* file)
{
strcpy(g_fileNameHeightMap, file);
initEnvironement();
}
void prairieLoadGroundTexture(const char* file)
{
strcpy(g_fileNameTextureGround, file);
initEnvironement();
}
void prairieSetGroundSize(float f)
{
g_groundSize=f;
initEnvironement();
}
void prairieSetGroundHeight(float f)
{
g_groundHeight=f;
initEnvironement();
}*/





/*void prairieSetTreesNumber(int i)
{
if (i<g_numberOfTrees) g_numberOfTrees=i;
else
   {
   for (int j=g_numberOfTrees;j<i;j++)
     {
     g_treesPositionsX[j]=g_groundSize*drand48();
     g_treesPositionsY[j]=g_groundSize*drand48();
     g_treesPositionsZ[i]=groundGetHeightAtPosition(g_treesPositionsX[i],
   						  g_treesPositionsY[i]); 
     }
   g_numberOfTrees=i;
   }
forestSetTrees(g_numberOfTrees,g_treesPositionsX,
		g_treesPositionsY,g_treesPositionsZ);
}
void prairieResetTreesPositions()
{
for (int i=0;i<g_numberOfTrees;i++)
  {
  g_treesPositionsX[i]=g_groundSize*drand48();
  g_treesPositionsY[i]=g_groundSize*drand48();
  g_treesPositionsZ[i]=groundGetHeightAtPosition(g_treesPositionsX[i],
   						  g_treesPositionsY[i]); 
  }
srand(0);
forestSetTrees(g_numberOfTrees,g_treesPositionsX,
		g_treesPositionsY,g_treesPositionsZ);
}
void prairieShowTreesInterface(bool b)
{
forestActiveGraphicInterface(b);
}
void prairieSetTreesNbPolygonsMax(int i)
{
forestSetMaxPolygonsPerFrame(i);
}
*/



/*void prairieSetGrassNbPatches1(int i)
{
g_numberOfGrassPatches1=i;
grassInit(groundGetLength(), groundGetWidth(),
	  g_numberOfGrassPatches1,
	  g_numberOfGrassPatches2,
	  g_fileNameDensityMap);
}
void prairieSetGrassNbPatches2(int i)
{
g_numberOfGrassPatches2=i;
grassInit(groundGetLength(), groundGetWidth(),
	  g_numberOfGrassPatches1,
	  g_numberOfGrassPatches2,
	  g_fileNameDensityMap);
}
void prairieSetNbGrassesPerPatche(int i)
{

}
void prairieSetGrassWidth(float f)
{
grassSetWidth(f);
}
void prairieSetGrassHeight(float f)
{
grassSetHeight(f);
}
void prairieSetGrassRadius3D(float f)
{
grassSetDistance3D(f);
}
void prairieSetGrassRadius2D5(float f)
{
grassSetDistance2D5(f);
}
void prairieSetGrassRadius2D(float f)
{
grassSetDistance2D(f);
}
void prairieSetGrassPolyg3D(int i)
{
grassSetPG3DGrass(short(i));
}
void prairieSetGrassPolyg2D5(int i)
{
grassSetPG2D5Grass(short(i));
}
void prairieSetGrassCrushingMaxTime(float f)
{
grassSetCrushingMaxTime(f);
}
void prairieSetGrassCrushingDamping(float f)
{
grassSetCrushingDamping(f);
}
void prairieLoadGrassDensityMap(const char* file)
{
strcpy(g_fileNameDensityMap,file);
grassInit(groundGetLength(), groundGetWidth(),
	  g_numberOfGrassPatches1,
	  g_numberOfGrassPatches2,
	  g_fileNameDensityMap);
}*/



/*void prairieSetBreezeSpeed(float f)
{
if (g_test1) {g_value1=f-4.0;printf("value wind X : %f\n", g_value1);}
windSetBreezeSpeed(f);
}
void prairieSetGustSpeed(float f)
{
windSetGustSpeed(f);
}
void prairieSetBreezePower(float f)
{
windSetBreezePower(f);
}
void prairieSetGustPower(float f)
{
windSetGustPower(f);
}
void prairieAddWindGust()
{
windAddGust(g_groundSize*(0.5+0.6*drand48()),
	    g_groundSize*(0.5+0.6*drand48()));
}
*/

























