
#include "MyPrairieViewer.h"
#include "MyPrairie.h"

//pour la gestion de l'interface graphique des arbres
//cette interface OpenGL est dans le Viewer.
//#include "../libForest/ForestInterface.h"
#include "WindInterface.h"
#include <qapplication.h>
#include <QTimer> // ajout pour compatibilite QT4, pas forcemment utile
#include <QMouseEvent>
#include <QKeyEvent>
#include <math.h>

using namespace qglviewer; // Vec

//-----------------------------------------------------------------------------
//---------------------------------VIEWER--------------------------------------
//-----------------------------------------------------------------------------

Viewer::Viewer(QWidget* parent, const QGLWidget* shareWidget, Qt::WFlags flags)
  : QGLViewer(parent,shareWidget,flags)
{
m_pTimerDrawLoop  = new QTimer(this);
connect(m_pTimerDrawLoop, SIGNAL(timeout()), this, SLOT(updateGL())); 
m_pTimerDrawLoop ->start(0);
m_displayNbPolygonsAndFPS=false;
m_displayProcessorOccupation=false;
m_displayHelp=false;
m_prairieSetTexture2D5Draw=false;
m_prairieSetTexture2DDraw=false;
}
 
Viewer::~Viewer() 
{
prairieQuit();
delete m_pTimerDrawLoop;
}

void Viewer::init()
{
QGLViewer::init();

help();
//initialisation camera
//scene center...
//restoreStateFromFile();
//on prends le cas ou la scene est la plus grande...
setSceneRadius(300.0f);
//camera()->setMode(qglviewer::ManipulatedFrame::FLY);
camera()->lookAt(Vec(0.0f,50.0f,0.0f));
setSceneCenter(Vec(300.0f,300.0f,0.0f));
camera()->setPosition(Vec(15.0f,0.0f,10.0f));

camera()->setFlySpeed(0.5);

setDisplayText(false);
setFPSIsDisplayed(false);
prairieInit();
prairieDirectTheRock(true);
}

void Viewer::resizeGL(int w, int h)
{
prairieSetWindowSize(w,h);
QGLViewer::resizeGL(w,h);
}
 
void Viewer::preDraw()
{
 camera()->loadProjectionMatrix();
 camera()->loadModelViewMatrix();
}

void Viewer::draw()
{
//camera
float cameraO[3];float cameraZ[3];
cameraO[0]=(camera()->position()).x;
cameraO[1]=(camera()->position()).y;
cameraO[2]=(camera()->position()).z;
Vec z = (camera()->frame()->inverseTransformOf(Vec(0.0f,0.0f,1.0f)));
z.normalize();cameraZ[0]=-z[0];cameraZ[1]=-z[1];cameraZ[2]=-z[2];
prairieDraw(cameraO,cameraZ);
drawText(100,10," "); // pour supprimer une erreur Color::setRgbF: RGB parameters out of range
if (m_dislpayText) displayText();
prairieSetTexture2D5Draw(m_prairieSetTexture2D5Draw);
prairieSetTexture2DDraw(m_prairieSetTexture2DDraw);
}

void Viewer::displayText()
{
char lstr[64];
//qglColor(foregroundColor());
glDisable(GL_LIGHTING);
glColor3f(1.0f,1.0f,1.0f);
   if (m_displayNbPolygonsAndFPS)
   {
   sprintf(lstr,"             polygons : %d",prairieGetNbPolygonsDrawedPerFrame());
   drawText(10,10,lstr);

   }
else if (m_displayProcessorOccupation)
   {
   prairieGetInfosOnCalculations(lstr);
   drawText(10,10,lstr);
   }
else if (m_displayHelp)
   {
   sprintf(lstr,"useful keys :");
   drawText(10,300,lstr);
   sprintf(lstr,"F5: display fps and nb polygons");
   drawText(10,270,lstr);
   sprintf(lstr,"F6: processor calculations");
   drawText(10,240,lstr);
   sprintf(lstr,"F9: show grass texture 2D5");
   drawText(10,210,lstr);
   sprintf(lstr,"F10: show grass texture 2D");
   drawText(10,180,lstr);
   sprintf(lstr,"Up,Down,Left,Right: move the rock");
   drawText(10,120,lstr);
   }
}

void Viewer::mousePressEvent(QMouseEvent *e)
{
if (e->button()==Qt::LeftButton)
   {
 //  if (!forestSendMouseButtonDown(e->x(), e->y()))
      QGLViewer::mousePressEvent(e);
   }
else
   QGLViewer::mousePressEvent(e);
} 

void Viewer::mouseReleaseEvent(QMouseEvent *e)
{ 
if (e->button()==Qt::LeftButton)
  {
 // if (!forestSendMouseButtonUp(e->x(), e->y()))
     QGLViewer::mouseReleaseEvent(e);
  }
else
   QGLViewer::mouseReleaseEvent(e);
} 

void Viewer::mouseMoveEvent (QMouseEvent *e)
{ 
//if (!forestSendMouseMove(e->x(), e->y()))
QGLViewer::mouseMoveEvent(e);
} 
 
void Viewer::keyPressEvent(QKeyEvent *e)
{
  QString fn;
  switch (e->key())
    {
      case Qt::Key_F1:
      	    m_displayHelp=!m_displayHelp;
	    m_displayProcessorOccupation=false;
	    m_displayNbPolygonsAndFPS=false;
	    setDisplayText(m_displayHelp);
	    setFPSIsDisplayed(false);
      	    break;
      case Qt::Key_F5:
      	    m_displayNbPolygonsAndFPS=!m_displayNbPolygonsAndFPS;
	    m_displayProcessorOccupation=false;
	    m_displayHelp=false;
	    setDisplayText(m_displayNbPolygonsAndFPS);
	    setFPSIsDisplayed(m_displayNbPolygonsAndFPS);
	    break;
      case Qt::Key_F6:
	    m_displayProcessorOccupation=!m_displayProcessorOccupation;
	    m_displayNbPolygonsAndFPS=false;
	    m_displayHelp=false;
	    setDisplayText(m_displayProcessorOccupation);
      	    setFPSIsDisplayed(false);
	    break;
//      case Qt::Key_F9:   prairieSetTexture2D5Draw(true);   break;
//      case Qt::Key_F10:   prairieSetTexture2DDraw(true);   break;
      case Qt::Key_F9:   m_prairieSetTexture2D5Draw=!m_prairieSetTexture2D5Draw;   break;
      case Qt::Key_F10:   m_prairieSetTexture2DDraw=!m_prairieSetTexture2DDraw;   break;
      case Qt::Key_Up:   prairieDirectTheRockUp();   break;
      case Qt::Key_Down:   prairieDirectTheRockDown();   break;
      case Qt::Key_Left:   prairieDirectTheRockLeft();   break;
      case Qt::Key_Right:   prairieDirectTheRockRight();  break;
//	  case Qt::Key_Space: ;     break;
      default:   QGLViewer::keyPressEvent(e);   break;
    }
} 
/*
void Viewer::keyReleaseEvent(QKeyEvent *e) // empeche l'utilisation simple du flymode
{
 switch (e->key())
    { 
      case Qt::Key_F9:   prairieSetTexture2D5Draw(false);   break;
      case Qt::Key_F10:   prairieSetTexture2DDraw(false);   break;
      default:   QGLViewer::keyPressEvent(e);   break;
    }
}
*/
  QString Viewer::helpString() const
{
  QString text("<h2>P r a i r i e V i e w e r</h2>");
  text += "Use the mouse to move the camera around the object. ";
  text += "You can respectively revolve around, zoom and translate with the three mouse buttons. ";
  text += "Left and middle buttons pressed together rotate around the camera view direction axis<br><br>";
  text += "Pressing <b>Alt</b> and one of the function keys (<b>F1</b>..<b>F12</b>) defines a camera keyFrame. ";
  text += "Simply press the function key again to restore it. Several keyFrames define a ";
  text += "camera path. Paths are saved when you quit the application and restored at next start.<br><br>";
  text += "Press <b>F</b> to display the frame rate, <b>A</b> for the world axis, ";
  text += "<b>Alt+Return</b> for full screen mode and <b>Control+S</b> to save a snapshot. ";
  text += "See the <b>Keyboard</b> tab in this window for a complete shortcut list.<br><br>";
  text += "Double clicks automates single click actions: A left button double click aligns the closer axis with the camera (if close enough). ";
  text += "A middle button double click fits the zoom of the camera and the right button re-centers the scene.<br><br>";
  text += "A left button double click while holding right button pressed defines the camera <i>Revolve Around Point</i>. ";
  text += "See the <b>Mouse</b> tab and the documentation web pages for details.<br><br>";
  text += "Press <b>Escape</b> to exit the viewer.";
  text += "Press <b>F1</b> to view help menu.";
  return text;
}




