#include "PrairieQTDriver.h"
#include "PrairieXML.h"
#include "PrairieDefaultDirectoriesAndFiles.h" 
#include <iostream>
#include "GroundInterface.h"
#include "WindInterface.h"
//#include "../libForest/ForestInterface.h"
#include "GrassInterface.h" 

#include <qapplication.h>
#include <qfiledialog.h>
#include <qcombobox.h>
#include "FormGrass.h"
#include <stdlib.h>

#define COEF_QT_GRASS_WIDTH 0.0001
#define COEF_QT_GRASS_HEIGHT 0.01
#define COEF_QT_GRASS_NOISE_POSITIONS 0.01
#define COEF_QT_GRASS_NOISE_NORMALS 0.01
#define COEF_QT_GRASS_RADIUS3D 1.0
#define COEF_QT_GRASS_RADIUS2D5 1.0
#define COEF_QT_GRASS_RADIUS2D 1.0
#define COEF_QT_GRASS_CRUSHING_MAXTIME 1.0
#define COEF_QT_GRASS_CRUSHING_DAMPING 0.1
#define COEF_QT_GRASS_ANTIALIASING 0.1

#define COEF_QT_GROUND_SIZE 0.25
#define COEF_QT_GROUND_HEIGHT 0.025

#define COEF_QT_WIND_BREEZE_SPEED 0.01
#define COEF_QT_WIND_BREEZE_POWER 0.1
#define COEF_QT_WIND_GUST_SPEED 0.01
#define COEF_QT_WIND_GUST_POWER 1.0
#define COEF_QT_WIND_BLAST_PARAM 0.001
#define COEF_QT_WIND_BLAST_POWER 1.0
extern FormGrass* formGrass;

PrairieQTDriver::PrairieQTDriver() : QGLWidget() //QObject()
{
}

PrairieQTDriver::~PrairieQTDriver()
{
}


//-----------------------------------------
//------------------------parametres ground
//-----------------------------------------	
void PrairieQTDriver::initGrassForestWind()
{
windInit(groundGetArea(),groundGetMaxHeight());
grassInit(groundGetArea(),
	  grassGetNumberPatchesInLength(),
	  grassGetNumberPatchesInWidth(),
	  grassGetFileDensityMap());
//pour reinitialiser la hauteur...
/*short numberTrees=forestGetNumberOfTrees();
ForestTree* trees = new ForestTree[numberTrees];
forestGetTrees(trees);
forestSetTrees(numberTrees,trees);
delete[] trees;*/
}


void PrairieQTDriver::loadGroundTexture()
{
QString fn;
fn = QFileDialog::getOpenFileName(this, "select bitmap", GROUND_DEFAULT_DIRECTORYDATA, "bitmap (*.bmp *.ppm)"); // pour QT4
//fn = Q3FileDialog::getOpenFileName(GROUND_DEFAULT_DIRECTORYDATA,"bitmap (*.bmp *.ppm)");//  QT3
if ( fn.isEmpty() ) {return;}

groundInit(groundGetArea(),
	   groundGetMaxHeight(),
	   groundGetNumberOfQuadsMax(),
	   groundGetFileHeightMap(),
	   fn.toAscii(),
	   groundGetRepeatTexture());
}
void PrairieQTDriver::loadGroundHeightMap()
{
QString fn;
fn = QFileDialog::getOpenFileName(this, "select bitmap", GROUND_DEFAULT_DIRECTORYDATA, "bitmap (*.bmp *.ppm)"); // pour QT4
//fn = Q3FileDialog::getOpenFileName(GROUND_DEFAULT_DIRECTORYDATA,"bitmap (*.bmp *.ppm)");// QT3
if ( fn.isEmpty() ) {return;}

groundInit(groundGetArea(),
	   groundGetMaxHeight(),
	   groundGetNumberOfQuadsMax(),
	   fn.toAscii(),
	   groundGetFileTexture(),
	   groundGetRepeatTexture());
initGrassForestWind();
}
void PrairieQTDriver::setGroundSize(int i)
{
float groundArea[]={0.0,0.0,float(i)*COEF_QT_GROUND_SIZE,float(i)*COEF_QT_GROUND_SIZE};
groundInit(groundArea,
	   groundGetMaxHeight(),
	   groundGetNumberOfQuadsMax(),
	   groundGetFileHeightMap(),
	   groundGetFileTexture(),
	   groundGetRepeatTexture());
initGrassForestWind();
} 
void PrairieQTDriver::setGroundHeight(int i)
{
groundInit(groundGetArea(),
	   float(i)*COEF_QT_GROUND_HEIGHT,
	   groundGetNumberOfQuadsMax(),
	   groundGetFileHeightMap(),
	   groundGetFileTexture(),
	   groundGetRepeatTexture());
initGrassForestWind();
} 



//-----------------------------------------
//--------------------------parametres wind
//----------------------------------------- 
void PrairieQTDriver::setBreezeSpeed(int i)
{
windSetBreezeSpeed(float(i)*COEF_QT_WIND_BREEZE_SPEED);
}
void PrairieQTDriver::setBreezePower(int i)
{
windSetBreezePower(float(i)*COEF_QT_WIND_BREEZE_POWER);
}
void PrairieQTDriver::setGustSpeed(int i)
{
windSetGustSpeed(float(i)*COEF_QT_WIND_GUST_SPEED);
}
void PrairieQTDriver::setGustPower(int i)
{
windSetGustPower(float(i)*COEF_QT_WIND_GUST_POWER);
}
void PrairieQTDriver::addWindGust()
{
float groundSize=0.5f*(groundGetLength()+groundGetWidth());
windAddGust(groundSize*(0.5+0.5*(double(rand()) / RAND_MAX)),
	    groundSize*(0.5+0.5*(double(rand()) / RAND_MAX)));
}
void PrairieQTDriver::setBlastParam(int i)
{
windSetBlastParam(float(i)*COEF_QT_WIND_BLAST_PARAM);
}
void PrairieQTDriver::setBlastPower(int i)
{
windSetBlastPower(float(i)*COEF_QT_WIND_BLAST_POWER);
}
void PrairieQTDriver::addWindBlast()
{
float groundSize=0.5f*(groundGetLength()+groundGetWidth());
windAddBlast(groundSize/4*(0.5+0.5*(double(rand()) / RAND_MAX)));
}


//-----------------------------------------
//-------------------------parametres grass
//----------------------------------------- 
void PrairieQTDriver::setGrassNbPatches1(int i)
{
grassInit(groundGetArea(),
	  i,
	  grassGetNumberPatchesInWidth(),
	  grassGetFileDensityMap());
}
void PrairieQTDriver::setGrassNbPatches2(int i)
{
grassInit(groundGetArea(),
	  grassGetNumberPatchesInLength(),
	  i,
	  grassGetFileDensityMap());
}
void PrairieQTDriver::setGrassNbMaxPerPatche(int i)
{
grassSetNumberPerSquareMeters(i);
}
void PrairieQTDriver::setGrassWidth(int i)
{
grassSetWidth(float(i)*COEF_QT_GRASS_WIDTH);
}
void PrairieQTDriver::setGrassHeight(int i)
{
grassSetHeight(float(i)*COEF_QT_GRASS_HEIGHT);
}
void PrairieQTDriver::setGrassNoisePositions(int i)
{
grassSetNoisePositions(float(i)*COEF_QT_GRASS_NOISE_POSITIONS);
}
void PrairieQTDriver::setGrassNoiseNormals(int i)
{
grassSetNoiseNormals(float(i)*COEF_QT_GRASS_NOISE_NORMALS);
}
void PrairieQTDriver::setGrassRadius3D(int i)
{
grassSetDistance3D(float(i)*COEF_QT_GRASS_RADIUS3D);
}
void PrairieQTDriver::setGrassRadius2D5(int i)
{
grassSetDistance2D5(float(i)*COEF_QT_GRASS_RADIUS2D5);
}
void PrairieQTDriver::setGrassRadius2D(int i)
{
grassSetDistance2D(float(i)*COEF_QT_GRASS_RADIUS2D);
}
void PrairieQTDriver::setGrassPolyg3D(int i)
{
grassSetPG3DGrass(i);
}
void PrairieQTDriver::setGrassPolyg2D5(int i)
{
grassSetPG2D5Grass(i);
}
void PrairieQTDriver::setGrassAjustment2D5Height(bool b)
{
grassSetAjustmentHeight2D5(b);
}
void PrairieQTDriver::setGrassCrushingMaxTime(int i)
{
grassSetCrushingMaxTime(float(i)*COEF_QT_GRASS_CRUSHING_MAXTIME);
}
void PrairieQTDriver::setGrassCrushingDamping(int i)
{
grassSetCrushingDamping(float(i)*COEF_QT_GRASS_CRUSHING_DAMPING);
}
void PrairieQTDriver::loadGrassDensityMap()
{
QString fn;
fn = QFileDialog::getOpenFileName(this, "select bitmap", GRASS_DEFAULT_DIRECTORYDATA, "bitmap (*.bmp *.ppm)"); // pour QT4
//fn = Q3FileDialog::getOpenFileName(GRASS_DEFAULT_DIRECTORYDATA,"bitmap (*.bmp *.ppm)");//QT3
if ( fn.isEmpty() ) {return;}
grassInit(groundGetArea(),
	  grassGetNumberPatchesInLength(),
	  grassGetNumberPatchesInWidth(),
	  fn.toAscii());
}

void PrairieQTDriver::setGrassTexture3D(bool b)
{
grassSetTextureOnGrass(b);
}

void PrairieQTDriver::loadGrassTexture3D()
{
QString fn;
fn = QFileDialog::getOpenFileName(this, "select bitmap", GRASS_DEFAULT_DIRECTORYDATA, "pixmap (*.bmp *.ppm *.tga)"); // pour QT4
//fn = Q3FileDialog::getOpenFileName(GRASS_DEFAULT_DIRECTORYDATA,"pixmap (*.bmp *.ppm *.tga)");//QT3
if ( fn.isEmpty() ) {return;}
grassSetFileTextureOnGrass(fn.toAscii());
}

void PrairieQTDriver::setGrassAntiAliasingTexture3D(int i)
{
grassSetAntiAliasingTextureOnGrass(float(i)*COEF_QT_GRASS_ANTIALIASING);
}

void PrairieQTDriver::regenerateGrassTexture2D5()
{
grassRegenerateTexture2D5();
}

void PrairieQTDriver::setGrassAntiAliasingTexture2D5(int i)
{

grassSetAntiAliasingTexture2D5(float(i)*COEF_QT_GRASS_ANTIALIASING);
}

void PrairieQTDriver::setGrassSizeTexture2D5(int i)
{
if (i==0) grassSetSizeTexture2D5(256,32);
if (i==1) grassSetSizeTexture2D5(256,64);
if (i==2) grassSetSizeTexture2D5(256,128);
if (i==3) grassSetSizeTexture2D5(512,32);
if (i==4) grassSetSizeTexture2D5(512,64);
if (i==5) grassSetSizeTexture2D5(512,128);
if (i==6) grassSetSizeTexture2D5(1024,32);
if (i==7) grassSetSizeTexture2D5(1024,64);
if (i==8) grassSetSizeTexture2D5(1024,128);
}
/*
//-----------------------------------------
//-------------------------parametres trees
//-----------------------------------------
void PrairieQTDriver::setTreesNumber(int i)
{
//double drand48(void); //ajout
short numberTrees=i;
short lastNumberTrees=forestGetNumberOfTrees();
if (lastNumberTrees<numberTrees)
	{
	ForestTree* trees=new ForestTree[numberTrees];
	forestGetTrees(trees);
	for (short j=lastNumberTrees;j<numberTrees;j++)
  		{
  		trees[j].m_position2d[0]=groundGetXMin() + groundGetLength()*(double(rand()) / RAND_MAX);
  		trees[j].m_position2d[1]=groundGetYMin() + groundGetWidth()*(double(rand()) / RAND_MAX);
  		}
	forestSetTrees(numberTrees,trees);
	delete[] trees;
	}
else
	{
	ForestTree* trees=new ForestTree[lastNumberTrees];
	forestGetTrees(trees);
	forestSetTrees(numberTrees,trees);
	delete[] trees;
	}
}
void PrairieQTDriver::loadARB()
{
QString fn;
fn = QFileDialog::getOpenFileName(this, "select arb", FOREST_DEFAULT_DIRECTORYDATA, "fichier type d'arbre (*.arb)"); // pour QT4
//fn = Q3FileDialog::getOpenFileName(FOREST_DEFAULT_DIRECTORYDATA,"fichier type d'arbre (*.arb)");//QT3
if ( fn.isEmpty() ) {return;}
forestLoadARB(fn.toAscii());
}
void PrairieQTDriver::resetTreesPositions()
{
//double drand48(void); //ajout
short i;
short numberTrees=forestGetNumberOfTrees();
ForestTree* trees=new ForestTree[numberTrees];
forestGetTrees(trees);
for (i=0;i<numberTrees;i++)
  {
  trees[i].m_position2d[0]=groundGetXMin() + groundGetLength()*(double(rand()) / RAND_MAX);
  trees[i].m_position2d[1]=groundGetYMin() + groundGetWidth()*(double(rand()) / RAND_MAX);
  }
forestSetTrees(numberTrees,trees);
delete[] trees;
}
void PrairieQTDriver::resetTreesForms()
{
forestRandomTreesForms();
}
void PrairieQTDriver::setTreesNbPolygonsMax(int i)
{
forestSetMaxPolygonsPerFrame(i);
}
void PrairieQTDriver::showTreesInterface(bool b)
{
forestActiveGraphicInterface(b);
}

*/

void PrairieQTDriver::loadQTInterface()
{
//----grass
setQTGrassNbPatches1(grassGetNumberPatchesInLength());
setQTGrassNbPatches2(grassGetNumberPatchesInWidth());
setQTGrassNbMaxPerPatche(grassGetNumberPerSquareMeters());
setQTGrassWidth(int(grassGetWidth()/COEF_QT_GRASS_WIDTH));
setQTGrassHeight(int(grassGetHeight()/COEF_QT_GRASS_HEIGHT));
setQTGrassNoisePositions(int(grassGetNoisePositions()/COEF_QT_GRASS_NOISE_POSITIONS));
setQTGrassNoiseNormals(int(grassGetNoiseNormals()/COEF_QT_GRASS_NOISE_NORMALS));
setQTGrassRadius3D(int(grassGetDistance3D()/COEF_QT_GRASS_RADIUS3D));
setQTGrassRadius2D5(int(grassGetDistance2D5()/COEF_QT_GRASS_RADIUS2D5));

setQTGrassRadius2D(int(grassGetDistance2D()/COEF_QT_GRASS_RADIUS2D));
setQTGrassPolyg3D(grassGetPG3DGrass());
setQTGrassPolyg2D5(grassGetPG2D5Grass());
setQTGrassAjustment2D5Height(grassGetAjustmentHeight2D5());
setQTGrassCrushingMaxTime(int(grassGetCrushingMaxTime()/COEF_QT_GRASS_CRUSHING_MAXTIME));
setQTGrassCrushingDamping(int(grassGetCrushingDamping()/COEF_QT_GRASS_CRUSHING_DAMPING));

setQTGrassTexture3D(grassIsTextureOnGrass());
setQTGrassAntiAliasingTexture3D(int(grassGetAntiAliasingTextureOnGrass()/COEF_QT_GRASS_ANTIALIASING));
setQTGrassAntiAliasingTexture2D5(int(grassGetAntiAliasingTexture2D5()/COEF_QT_GRASS_ANTIALIASING));

int x,y;
grassGetSizeTexture2D5(x,y);
if (x==256 && y==32) formGrass->ComboBox_SizeTexture2D5->setCurrentItem(0);
if (x==256 && y==64) formGrass->ComboBox_SizeTexture2D5->setCurrentItem(1);
if (x==256 && y==128) formGrass->ComboBox_SizeTexture2D5->setCurrentItem(2);
if (x==512 && y==32) formGrass->ComboBox_SizeTexture2D5->setCurrentItem(3);
if (x==512 && y==64) formGrass->ComboBox_SizeTexture2D5->setCurrentItem(4);
if (x==512 && y==128) formGrass->ComboBox_SizeTexture2D5->setCurrentItem(5);
if (x==1024 && y==32) formGrass->ComboBox_SizeTexture2D5->setCurrentItem(6);
if (x==1024 && y==64) formGrass->ComboBox_SizeTexture2D5->setCurrentItem(7);
if (x==1024 && y==128) formGrass->ComboBox_SizeTexture2D5->setCurrentItem(8);

//----ground
setQTGroundSize(int(groundGetLength()/COEF_QT_GROUND_SIZE));
setQTGroundHeight(int(groundGetMaxHeight()/COEF_QT_GROUND_HEIGHT));

//----trees
//setQTTreesNumber(forestGetNumberOfTrees());

//----wind
setQTBreezeSpeed(int(windGetBreezeSpeed()/COEF_QT_WIND_BREEZE_SPEED));
setQTBreezePower(int(windGetBreezePower()/COEF_QT_WIND_BREEZE_POWER));
setQTGustSpeed(int(windGetGustSpeed()/COEF_QT_WIND_GUST_SPEED));
setQTGustPower(int(windGetGustPower()/COEF_QT_WIND_GUST_POWER));
setQTBlastParam(int(windGetBlastParam()/COEF_QT_WIND_BLAST_PARAM));
setQTBlastPower(int(windGetBlastPower()/COEF_QT_WIND_BLAST_POWER));
}










