#include "PrairieXML.h"

#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/parserInternals.h>

#include <string>
#include <vector>

//prairie interface

#include "GroundInterface.h"
#include "WindInterface.h"
//#include "../libForest/ForestInterface.h"
#include "GrassInterface.h"


using namespace std;

//--------------------------------------------------------------------------
//---------------------------PRAIRIE----------------------------------------
//-------------------------------------------------------------------------- 
bool loadPrairieXML(xmlNodePtr p_parentNode)
{
xmlNodePtr childPrairie = p_parentNode->children;
while (childPrairie != NULL) 
   {
   if (childPrairie->type == XML_ELEMENT_NODE)
      {   
      string name = (const char*) childPrairie->name;
      if (name == "prairie")
         {
	 bool valid=true;
	 valid &= groundLoadXML(childPrairie);
	 valid &= windLoadXML(childPrairie);
	 valid &= grassLoadXML(childPrairie);
//	 valid &= forestLoadXML(childPrairie);
	 return valid; 
	 }
      }
   childPrairie = childPrairie->next;
   }
return false;
}



//--------------------------------------------------------------------------
//--------------------------------------------------------------------------
bool loadPrairieXML(const char* filename)
{
xmlNodePtr p_node;
xmlDocPtr  p_doc;

//initialisation
if (filename==NULL) return false;
p_doc = xmlParseFile(filename);
if (p_doc == NULL) return false;
p_node = xmlDocGetRootElement(p_doc);

//validation
xmlValidCtxt cvp;
  // print some information
  printf ("chargement de la scene : %s\n", filename);
  if (p_doc->name != NULL) printf("name : %s\n", p_doc->name);
  if (p_doc->compression == -1) printf("     no compression\n");
  else printf("     compression level : %d\n", p_doc->compression);
  printf("     XML version : %s\n", p_doc->version);
  if (p_doc->encoding == NULL) printf("     encoding not specified\n");
  else printf("     encoding : %s\n", p_doc->encoding);  
cvp.userData = stderr;
cvp.error = (xmlValidityErrorFunc) fprintf;
cvp.warning = (xmlValidityWarningFunc) fprintf;  
if (!xmlValidateDocument(&cvp, p_doc)) return false;

bool valid = loadPrairieXML(p_node);
xmlFreeDoc  (p_doc);
return valid;
}


void savePrairieXML(xmlNodePtr p_parentNode)
{
xmlNodePtr p_prairieNode=xmlNewNode (NULL, (const xmlChar*)"prairie");

groundSaveXML(p_prairieNode);
windSaveXML(p_prairieNode);
grassSaveXML(p_prairieNode);
//forestSaveXML(p_prairieNode);

xmlAddChild(p_parentNode,p_prairieNode);
}

//--------------------------------------------------------------------------
//--------------------------------------------------------------------------
void savePrairieXML(const char* filename)
{
xmlDocPtr p_doc = NULL;
xmlNodePtr p_sceneNode = NULL;

//doc
p_doc=xmlNewDoc((const xmlChar*)"1.0");

//root node
p_sceneNode=xmlNewNode (NULL, (const xmlChar*)"scene"); 
p_doc->children=p_sceneNode;

savePrairieXML(p_sceneNode);

//internal subset : dtd name and system id
//call after root node creation
xmlCreateIntSubset(p_doc,(const xmlChar*)"scene",
  		   NULL,(const xmlChar*)"scenePrairie.dtd");

xmlSaveFormatFileEnc (filename, p_doc, "UTF-8",1); 
xmlFreeDoc  (p_doc);
}
