#ifndef _rock_on_the_prairie_
#define _rock_on_the_prairie_

//include GL
#include "gl.h"//#include <GL/gl.h>
#include "glu.h"//#include <GL/glu.h>
#include "glut.h"//#include <GL/glut.h>

//
#include <stdlib.h>
#include <math.h>
#include <string.h>

//autres
#include "GrassInterface.h"		//for GrassTrajectory
#include "GroundInterface.h"	//for getHeightAtPosition

/** \class Rock
\brief modelise un rocher 

possede une trajectoire..

*/
class Rock
{
private:	
	///angle de rotation
	float m_rotation;
public:
	///position du rocher
	float m_position[3];
	///vitesse du rocher
	float m_speed[3];
	//taille du rocher
	float m_size;
	
	///trajectoire memorise du rocher
	GrassTrajectory *m_pTrajectory;

	//
	bool mb_directTheRock;
	bool mb_mvtRockUp;
	bool mb_mvtRockDown;
	bool mb_mvtRockLeft;
	bool mb_mvtRockRight;
	bool mb_bound;
public:
	///constructeur 
	Rock()
		{
		m_pTrajectory = new GrassTrajectory(0.2,
						    200.0,
						    0.001,
						    0.4);
		m_rotation=0.0;
		m_size=1.0;
		
		mb_mvtRockUp=false;
		mb_mvtRockDown=false;
		mb_mvtRockLeft=false;
		mb_mvtRockRight=false;
		mb_directTheRock=true;
		mb_bound=false;	
		}
	
	///Reinitialisation des donnees
	void init(const float* position2d)
		{
		m_position[0]=position2d[0];
		m_position[1]=position2d[1];
		m_position[2]=groundGetHeightAtPosition(m_position[0],
							m_position[1]);
		m_pTrajectory->init(m_position);					
		}
	
	void setSize(float size)
		{
		m_size=size;
		}
	
	void setTrajectoryMysteryParameter(float f)
		{
		m_pTrajectory->m_coefCrushingAccordingToNormal=f;
		}
	void setTrajectoryMysteryParameter2(float f)
		{
		m_pTrajectory->m_controlPowerExp=f;
		}
	
	void setTrajectoryPower(float power)
		{
		m_pTrajectory->m_power=power;
		}
	void setBound(bool b)
		{
		mb_bound=b;
		}
	
	///destructeur
	~Rock()
		{
		delete m_pTrajectory;
		}

	///dessine le rocher
	inline void draw()
		{
		//affichage des herbes
  		static GLfloat materialSpecular_rocher[]  = {0.3f, 0.4f, 0.3f, 1.0f} ;
  		static GLfloat materialShininess_rocher[] = {30.0f} ;
  		static GLfloat materialDiffuse_rocher[]   = {0.2f, 0.2f, 0.2f, 1.0f} ;
  		static GLfloat materialAmbient_rocher[]   = {0.1f, 0.1f, 0.1f, 1.0f} ;
		glMaterialfv(GL_FRONT, GL_SPECULAR, materialSpecular_rocher) ;
  		glMaterialfv(GL_FRONT, GL_SHININESS, materialShininess_rocher) ;
  		glMaterialfv(GL_FRONT, GL_DIFFUSE, materialDiffuse_rocher) ;
  		glMaterialfv(GL_FRONT, GL_AMBIENT, materialAmbient_rocher) ;
		
		glPushMatrix();
		glTranslated(m_position[0],m_position[1],m_position[2]+0.9*m_size);
		if (!mb_directTheRock) glRotatef(m_rotation,-m_speed[1],m_speed[0],0.0);
		glutSolidSphere(m_size, 10, 10);
		glPopMatrix();
		}
		
	inline void mvtUp()
		{
		mb_mvtRockUp=true;
		}
	inline void mvtDown()
		{
		mb_mvtRockDown=true;
		}
	inline void mvtLeft()
		{
		mb_mvtRockLeft=true;
		}
	inline void mvtRight()
		{
		mb_mvtRockRight=true;
		}
	inline void mvtAleatoire(bool b)
		{
		mb_directTheRock=!b;
		}
		
	inline void integrateTime(const float* cameraVecteurZ, const float& dt)
		{
		//mise a jour position x,y
		if (mb_directTheRock)
			{
			float cameraVecteurX[]={cameraVecteurZ[1],-cameraVecteurZ[0],0.0};
			if (mb_mvtRockUp)
				mvtUp(cameraVecteurZ,dt);
			if (mb_mvtRockDown)
				mvtDown(cameraVecteurZ,dt);
			if (mb_mvtRockLeft)
				mvtLeft(cameraVecteurX,dt);
			if (mb_mvtRockRight)
				mvtRight(cameraVecteurX,dt);
			mb_mvtRockUp=false;
			mb_mvtRockDown=false;
			mb_mvtRockLeft=false;
			mb_mvtRockRight=false;
			}
		else
			mvtCrazy(dt);
		
		if (Abs(m_speed[0])<0.05f) m_speed[0]=0.0f;
		if (Abs(m_speed[1])<0.05f) m_speed[1]=0.0f;
		m_position[0]+=m_speed[0]*dt;
		m_position[1]+=m_speed[1]*dt;
		m_rotation +=(Abs(m_speed[0]*dt)+Abs(m_speed[1]*dt))*57.3/m_size; 	
		if (!groundIsPositionInside(m_position[0],0.0f))
			{
			m_position[0]-=m_speed[0]*dt;
			m_speed[0]=-m_speed[0];	
			}
		if (!groundIsPositionInside(0.0f,m_position[1]))
			{
			m_position[1]-=m_speed[1]*dt;
			m_speed[1]=-m_speed[1];	
			}
		if (!groundIsPositionInside(m_position[0],m_position[1]))
			{
			m_position[0]=groundGetXMin() + groundGetLength()*0.5;
			m_position[1]=groundGetYMin() + groundGetWidth()*0.5;
			}	
			
		if (mb_bound)
		   {
		   m_position[2]=m_position[2]+m_speed[2];
		   m_speed[2]-=0.75f*dt;
		   
		   float heightGround=
		   groundGetHeightAtPosition(m_position[0],
					     m_position[1]);
		   
		  // if (m_position[2]-heightGround<1.0)
		   //	{
			m_pTrajectory->addInformation(m_position,true);
			
		//	}
		 //  else
		   //	link=false;	
		   	
		   if (m_position[2]<=heightGround)
		       {
		       m_speed[2]=1.0f;
		       m_position[2]=heightGround;
		       }
		   }
		else
		   {
		   m_position[2]=groundGetHeightAtPosition(m_position[0],
							   m_position[1]);
		   m_pTrajectory->addInformation(m_position,true);
		   }
		}
private:
	
	inline float Abs(const float& f)
		{
		if (f<0.0) return -f;
		return f;
		}
	inline float Signe(const float& f)
		{
		if (f<0.0) return -1.0;
		return 1.0;
		}	
	inline void mvtUp(const float* cameraZ, const float& dt)
		{
		m_speed[0]+=cameraZ[0];
		m_speed[1]+=cameraZ[1];
		
		if (Abs(m_speed[0])>Abs(cameraZ[0])) m_speed[0]=cameraZ[0];
		if (Abs(m_speed[1])>Abs(cameraZ[1])) m_speed[1]=cameraZ[1];
		}
	inline void mvtDown(const float* cameraZ, const float& dt)
		{
		m_speed[0]-=cameraZ[0];
		m_speed[1]-=cameraZ[1];
		
		if (Abs(m_speed[0])>Abs(cameraZ[0])) m_speed[0]=-cameraZ[0];
		if (Abs(m_speed[1])>Abs(cameraZ[1])) m_speed[1]=-cameraZ[1];
		}
	inline void mvtLeft(const float* cameraX, const float& dt)
		{
		m_speed[0]-=cameraX[0];
		m_speed[1]-=cameraX[1];
		
		if (Abs(m_speed[0])>Abs(cameraX[0])) m_speed[0]=-cameraX[0];
		if (Abs(m_speed[1])>Abs(cameraX[1])) m_speed[1]=-cameraX[1];
		}
	inline void mvtRight(const float* cameraX, const float& dt)
		{
		m_speed[0]+=cameraX[0];
		m_speed[1]+=cameraX[1];
		
		if (Abs(m_speed[0])>Abs(cameraX[0])) m_speed[0]=cameraX[0];
		if (Abs(m_speed[1])>Abs(cameraX[1])) m_speed[1]=cameraX[1];
		}
		
	inline void mvtCrazy(const float& dt)
		{
		static double time=0.0;
		time+=dt;
		m_speed[0]=(0.5*cos(time*0.03)+0.5*cos(time*0.01))
			   *Signe(m_speed[0]);
	        m_speed[1]=(0.5*cos(time*0.02)+0.5*cos(time*0.015))
			   *Signe(m_speed[1]);
		}
	};

#endif /* _rock_on_the_prairie_ */
